//*******************************************************************************************
//
// Filename : Dlg.h
//	
//				Definitions of CDlg, CDlgAbout, CDlgOption, CDlgFilter, CDlgEdit
//                             CDlgPlaylist
//
//
// Copyright (c) 2001 JL BOYER. All rights reserved
//
//*******************************************************************************************
#include "priv.h"
#include "MP3Info.h"
#include "MP3MultiInfo.h"


#ifndef _Dlg_H_
#define _Dlg_H_


// ***************************************************
// Dialog Generic class
class CDlg
{
public:
	CDlg() {m_hDlg=NULL;}
	~CDlg() {}
	int DoModal(UINT idRes, HWND hParent);
	HWND DoModeless(UINT idRes, HWND hParent);

protected:
	static BOOL CALLBACK DlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	HWND m_hDlg;

private:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
};





#define IDC_STATUS 400

// ***************************************************
// Dialog "Playlist"
class CDlgPlaylist: public CDlg
{

public:
	HWND m_hwndList;
	MP3MultiInfo m_minfo;		// store the info of all files
	CMP3Info **m_mp3iSelArray;	
	int m_nSelCount;
	tstring m_filterArtist;		// filter
	tstring m_filterAlbum;
	tstring m_filterComment;

private:
	int  m_realColumn[COLUMN_MAX];
	int  m_last_sort;
	BOOL m_bShowDuplicate;
	BOOL m_bRefreshSelLength;	// use :ST_UpdateLengthSelected
								//      Need to compute selected length
	long m_nTotalLength;
	long m_nAvgLength;
	long m_nSelLength;
	int  m_nFilterBy;
	HWND m_hwndStat;
	HICON m_hIcon;
	enum { COMDLG_OPEN = 0, COMDLG_SAVE};
	enum { NO_FILTER = 0, FILTER_GENRE = 1,FILTER_ARTIST = 2,FILTER_ALBUM = 4, FILTER_COMMENT = 8};

public:
	CDlgPlaylist() ;
	~CDlgPlaylist() {};
	void LV_ColumnGetRealOrder(int order[]);
	static void GetFrameInfo(CMP3Info *mp3i, int fid, tstring &str);

	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
	

private:
	void Init();
	void LoadParam(void);
	void SaveParam(void);

	void Resize(LPARAM lParam);
	LRESULT NotifyHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, HINSTANCE hInst);
	int InsertFile(tstring& file);

	
	static int CALLBACK ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, int nSubItem);

	BOOL doContextMenu(WPARAM wParam, LPARAM lParam);

	void LV_ColumnResetDisplay(BOOL firstime);
	void LV_ColumnSort(int column, BOOL bColumn, BOOL bReverse=TRUE);
	int  LV_ColumnGetNum();

	void LV_ColumnSaveSetting();

	void LV_InitIcon();
	void LV_ColumnsGrid();

	void UpdateMenu(HMENU hMenu); 
	void MenuButton(int nButton);				
	void MenuCheckItem(int nItem);				
	void OwnerDrawBtn (LPDRAWITEMSTRUCT lpdis);
	void SetActive();
	
	void LV_DropFiles(HDROP hFilesInfo);
	void LV_AddFiles();
	void LV_AddDirectory();
	void LV_AddDir2List(TCHAR* dir);
	void LV_New();
	void LV_ComDlg(int OPENSAVE);
	void LV_Import();
	void LV_Export();
	void AddFileExtension(TCHAR *szFile, TCHAR *szExt); 
	void LV_Playlist(int);
	void LV_PlaylistWinamp(int);
	void LV_LoadEZ3(TCHAR *filename);
	void LV_LoadM3U(TCHAR *filename);
	void LV_LoadPLS(TCHAR *filename);
	void LV_WriteEZ3(TCHAR*);
	BOOL LV_WriteM3U(TCHAR*);
	BOOL LV_WritePLS(TCHAR*);
	BOOL LV_WriteHTML_Table(TCHAR*);
	BOOL LV_WriteHTML_Listing(TCHAR*);
	void LV_WriteHTML_TableColumn(FILE *file, CMP3Info *mp3i, int iCol, int iColArtist);
	void LV_WriteHTML_CSS(FILE *file, TCHAR *title);
	void LV_WriteHTML_Footer(FILE *file, TCHAR *szCol, unsigned long uTotalSize, long nTotalLength, int nCountSelect);
	BOOL LV_WriteASCII(TCHAR*);
	BOOL LV_WriteASCIIFixed(TCHAR* filename);
	
	void LV_SelectAll(BOOL bSelect);
	void LV_SelectToggle();
	void LV_SelectWithSame(int nCol);
	void LV_SelectCrop();
	void LV_SelectDel();
	void LV_SelectRemoveFile();
	void LV_UpdateStatus();
	void LV_StatGenre();
	void LV_UpdatePlaylist();

	void LV_DetermineFilter();
	BOOL LV_Filter(CMP3Info *mp3i);
	void LV_TitleWindowFilter();

	void LV_ShowDuplicate();
	void LV_DeadFiles();
	void LV_Redraw(BOOL bUpdate);
	void LV_GetInfoCol(CMP3Info *mp3i, int col, TCHAR *szCol, int iItem = -1);
	void LV_MatchTime(int nLengthMax);
	
	void GetMP3Selected();
	void SetMP3Selected();
	
	void ST_UpdateLengthSelected();
	BOOL ST_CreateStatus(int nMode);
	BOOL ST_ChangeText(LPTSTR szText, int iPart );

	void winamp_enqueue_to_playlist(BOOL bPlay);
	void winamp_enqueue_all_to_playlist(BOOL bPlay);

	void UpdateStatus();
};






// ***************************************************
// Dialog "About"
class CDlgAbout: public CDlg
{
public:
	CDlgAbout() {};
	~CDlgAbout() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void ShowHelp();

};




// ***************************************************
// Dialog "Column"
class CDlgColumn: public CDlg
{
public:

	HWND m_hListAvailable;
	HWND m_hListShown;

    int  m_shown[COLUMN_MAX];
    int  m_order[COLUMN_MAX];
    
	CDlgPlaylist *m_pThisPlaylist;

	CDlgColumn(CDlgPlaylist*);
	~CDlgColumn() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	BOOL  m_bColumnsChange;
	BOOL  LoadParam();
	BOOL  SaveParam();
	
	void DoMoveItem( HWND hWnd, BOOL bDown);
	int DoExchangeItem(HWND hFrom, HWND hTo);
	void DoExchangeItemALL(HWND hFrom, HWND hTo);
};


// ***************************************************
// Dialog "Column"
class CDlgOption: public CDlg
{
public:
	CDlgOption() {};
	~CDlgOption() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	BOOL  LoadParam();
	BOOL  SaveParam();
};


// ***************************************************
// Dialog "Reset settings"
class CDlgReset: public CDlg
{
public:
	CDlgReset() {};
	~CDlgReset() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	int ResetSettings();
};


// ***************************************************
// Dialog "Match Time"
class CDlgMatchTime: public CDlg
{
public:
	CDlgMatchTime() {};
	~CDlgMatchTime() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
};


// ***************************************************
// Dialog "Style HTML"
class CDlgStyleHTML: public CDlg
{
private:
	BOOL m_bIsLoading;
	TEMPLATESTRUCT* m_pTemplate;		// modele de style HTML

public:
	CDlgStyleHTML() {};
	~CDlgStyleHTML() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	BOOL  LoadParam();
	BOOL  SaveParam();

	void doChangeStyle();
	void doChooseColor(int nButton);
	void doDrawBtnColor (LPDRAWITEMSTRUCT lpdis);
	void doEditHexColor(int nEditColor);
	
	void doSaveFontName(int nIndex);
	void doSaveFontStyle(int nIndex);
	void doSaveFontSize(int nIndex);

	COLORREF GetColorButton(int i);	
	void SetColorButton(int i, COLORREF cColorNew);


	
};


// ***************************************************
// Dialog "Filter"

#define MAX_TYPEFILTER 4

class CDlgFilter: public CDlg
{

private:
	static const TCHAR *m_listTypeFilter[MAX_TYPEFILTER];
	
	CDlgPlaylist *m_pThisPlaylist;
	tstring m_newArtist;
	tstring m_newAlbum;
	tstring m_newComment;

	StrArray m_StrArray;
	HIMAGELIST m_hIml;
	int  m_nTypeFilter;
	BOOL m_bFilterChange;
	BOOL m_nOldSortColumnFilter;
	BOOL m_bSortOrderAscending;


public:
	CDlgFilter(CDlgPlaylist*) ;
	~CDlgFilter() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	static int CALLBACK ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);

	LRESULT NotifyHandler(UINT uMsg, WPARAM wParam, LPARAM lParam,	HINSTANCE hInst);
	void Init();
	void ShowCtrl();
	void RefreshTitle();
	void LoadGenre();
	int SaveGenre();
	void LoadFilter();
	void SaveFilter();

	int AddVector(tstring& strNew);
	int FindVector(tstring& strNew);

	
	BOOL DoContextMenu(WPARAM wParam, LPARAM lParam);
	void SelectAll(BOOL bSelect);
	void SelectToggle();
};



// ***************************************************
// Template Class "ID3 Pages"	Generic class
#define MAX_RES2FRAME 12

class CDlgID3Template: public CDlg
{
protected:
	HWND m_hwndList;
	static const TCHAR *m_listFrmt[MAX_COL_FRMT];
	static const int m_Res2Frame[MAX_RES2FRAME][2];
	CDlgPlaylist *m_pThisPlaylist;

public:
	CDlgID3Template(CDlgPlaylist*) ;
	~CDlgID3Template() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}


protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam) {return TRUE;};
	
	void InitFileList(BOOL bShortFileName = TRUE);

	int  GetResID(int fid);
	int  GetFrameID(int resid);
	void SetTextMP3info(int fid, CMP3Info *mp3i);
	void GetTextMP3info(int fid, CMP3Info *mp3i);
	void ShowMP3Info(CMP3Info *mp3i);
	
	void FrmtCheck2List(int nID);
	void FrmtRemoveCol();
	void FrmtUpdate();
	void FrmtUpdateSeparator();
	void FrmtSelectAll(int);
	
	void LOGEnable();
	void LOGChooseFile();

	void SaveSelectedItem();
	void OwnerDrawBtn(LPDRAWITEMSTRUCT lpdis);
	void MenuButton(int nIDButton, int nIDMenu, int nSubMenu);

	void CapitalizeString(TCHAR* szTemp);
	void ConvertString(TCHAR* szTemp);

	BOOL MsgConfirm(int nCount, int *pArray, int nPage);

	void GetFrameInfo(CMP3Info *mp3i, int fid, tstring &str);
};



// ***************************************************
// "Page - ID3 EditTag"
class CDlgID3Edit: public CDlgID3Template
{
private:
	int m_dwCurSel;
	BOOL m_bChange;
	BOOL m_bIsLoading;

public:
	CDlgID3Edit(CDlgPlaylist *pThis) : CDlgID3Template(pThis) 
		{m_dwCurSel = 0; m_bChange = FALSE;	m_bIsLoading =  TRUE;} ;

	~CDlgID3Edit() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void Init();
	void InitFileCbo();
	void ShowInfo();
	void GotoFile(int nMove);
	void ShowCoreMP3Info(CMP3Info *mp3i);
	void Changed(BOOL bEnabled);
	void EnabledID3v1();
	void EnabledID3v2();
	void CopyID3v2_ID3v1();
	void CopyID3v1_ID3v2();
	void IsDirty();
	void WriteTag();
	void SwapFieldID3(int nIDsrc, int nIDdest);
	void OwnerDrawBtnSwap(LPDRAWITEMSTRUCT lpdis);
	
};

// ***************************************************
// "MassTag Page"

class CDlgID3MassTag: public CDlgID3Template
{
public:
	CDlgID3MassTag(CDlgPlaylist *pThis) : CDlgID3Template(pThis) {};
	~CDlgID3MassTag() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void Init();
	void LoadParam();
	void SaveParam();
	void CheckTag2Edit(int nID);
	void EnableGOButton();
	void BtnGO();
	BOOL MassTAG(CMP3Info *mp3i);
};


// ***************************************************
// "Rename File fromTag Page"
class CDlgID3RenameFileFromTag: public CDlgID3Template
{
public:
	CDlgID3RenameFileFromTag(CDlgPlaylist *pThis) : CDlgID3Template(pThis) {};
	~CDlgID3RenameFileFromTag() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void LoadParam();
	void SaveParam();
	void BtnGO();
	BOOL RenameFileFromTAG(CMP3Info *mp3i);
	void ValidateFilename(TCHAR *filename);
};



// ***************************************************
// "Write TAG base on filename Page"
class CDlgID3WriteTagFromFile: public CDlgID3Template
{
public:
	CDlgID3WriteTagFromFile(CDlgPlaylist *pThis) : CDlgID3Template(pThis) {};
	~CDlgID3WriteTagFromFile() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void LoadParam();
	void SaveParam();
	void BtnGO();
	BOOL WriteTAGFromFile(CMP3Info *mp3i, int iItem);
	BOOL ExtractPath(TCHAR* pFileSpec, TCHAR* pPath);    
};


// ***************************************************
// "ID3 Conversion /Remove Tag Page"
class CDlgID3ConversionTag: public CDlgID3Template
{
public:
	CDlgID3ConversionTag(CDlgPlaylist *pThis) : CDlgID3Template(pThis) {};
	~CDlgID3ConversionTag() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
private:
	void BtnGO(int nAction);
	BOOL CopyID3v1_ID3v2(int iItem);
	BOOL CopyID3v2_ID3v1(int iItem);
	BOOL RemoveID3Tag(int iItem, int nID3Version);
	void ActiveButton(BOOL bEnabled);
	void CopyInfoID3v1_ID3v2(CMP3Info *mp3i, struct ID3V1MP3Info *id3i, int fid);
};

// ***************************************************
// "Oragnize "
class CDlgOrganize: public CDlgID3Template
{
private:	
	HWND m_hwndList;
	CDlgPlaylist *m_pThisPlaylist;

public:
	CDlgOrganize(CDlgPlaylist *pThis) : CDlgID3Template(pThis) {};
	~CDlgOrganize() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void LoadParam();
	void SaveParam();
	void BtnGO();
	BOOL OrganizeFile(CMP3Info *mp3i, TCHAR *szPathDest);
	void GetField2Dir(CMP3Info *mp3i, TCHAR *szDir);
	BOOL DirectoryExists(TCHAR *szName) ;
	BOOL FileExists(TCHAR *szName) ;
	void ChooseDirectory();
	BOOL MakeFullDir(TCHAR *szDir);
};



// ***************************************************
// Dialog "Tools ID3Tag"
#define NUMID3PAGES 6

class CDlgEditMain: public CDlg
{
private: 
	CDlgPlaylist *m_pThisPlaylist;

	HWND m_hwndTabCtrl;					// Handle Tab Control
		//HWND m_ID3PagesWnd[NUMID3PAGES];	// Array handle Pages
	int	 m_ID3PageActive;				// the active page
	CDlg *m_CDlgPage;
	HWND m_hwndPage;	

public:
	CDlgEditMain(CDlgPlaylist*, int nPart) ;
	~CDlgEditMain() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void InitTab();
	void UpdateID3Page();
	void UpdateBanner(HDC hDC);
};









// ***************************************************
// Dialog "Property"
class CDlgProperty: public CDlg
{
private: 
	CDlgPlaylist *m_pThisPlaylist;
	BOOL m_bAllPlaylist;
	StrArray m_StrArray;

public:
	CDlgProperty(CDlgPlaylist*, BOOL);
	~CDlgProperty() {};
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void ShowProperty();
	int FindVector(tstring& str);
	int AddVector(tstring& str);	

};






#endif // _Dlg_H_
