#include "priv.h"

#include "gen.h"			//general purpose plugin Winamp
#include "resource.h"
#include "definition.h"
#include "Dlg.h"
#include "Util.h"

// external declaration
extern COLUMNSSTRUCT gDefault_column[COLUMN_MAX];
extern INITSTRUCT gParam;
extern winampGeneralPurposePlugin plugin;

/* ************************************************************************ */
CDlgColumn::CDlgColumn(CDlgPlaylist *pThis): CDlg()
{ 
	 m_bColumnsChange = FALSE; 
	 m_pThisPlaylist = pThis;
}


/* ************************************************************************ */
BOOL CDlgColumn::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg)
	{
    case WM_INITDIALOG:
		LoadParam();
        return TRUE;

   
	case WM_COMMAND :
	   {
			switch (LOWORD(wParam))
			{
				
				case IDC_CHK_SHOWNUMBER:
				case IDC_CHK_SHOWNGRID:
					if (HIWORD(wParam) == BN_CLICKED)
						m_bColumnsChange  = TRUE;
						
					break;
			
			case IDC_BUTTONADD:
					DoExchangeItem(m_hListAvailable, m_hListShown);
					break;
				case IDC_BUTTONREMOVE:
					DoExchangeItem(m_hListShown, m_hListAvailable);
					break;
				
				case IDC_BUTTONADDALL:
					DoExchangeItemALL(m_hListAvailable, m_hListShown);
					break;
				case IDC_BUTTONREMOVEALL:
					DoExchangeItemALL(m_hListShown, m_hListAvailable);
					break;


				case IDC_LISTAVAILABLE:
					if (HIWORD(wParam) == LBN_DBLCLK)
						DoExchangeItem(m_hListAvailable, m_hListShown);
					break;
				case IDC_LISTSHOWN:
					if (HIWORD(wParam) == LBN_DBLCLK)
						DoExchangeItem(m_hListShown, m_hListAvailable);
					break;

				case IDC_BUTTONMOVEDOWN:
					DoMoveItem(m_hListShown, FALSE);
					break;
				case IDC_BUTTONMOVEUP:
					DoMoveItem(m_hListShown, TRUE);
					break;

				case IDOK :
					// Save users choices
					if (SaveParam() == TRUE)
						EndDialog(m_hDlg, (int)m_bColumnsChange);
					return TRUE;
                
				case IDCANCEL :
					EndDialog(m_hDlg, 0);
					return TRUE;

			}	
		}
		
	}
	return FALSE;
}


/* ************************************************************** */
BOOL  CDlgColumn::LoadParam()
{
	int i;
	const TCHAR *szID3v[] = {_T("ID3v1"), _T("ID3v2"), _T("ID3v2 / ID3v1")};

	// Column options
	m_bColumnsChange  = FALSE;


	m_hListShown     = GetDlgItem(m_hDlg, IDC_LISTSHOWN);
    m_hListAvailable = GetDlgItem(m_hDlg, IDC_LISTAVAILABLE);
    
	SendMessage(m_hListShown, LB_RESETCONTENT, 0, 0);
	SendMessage(m_hListAvailable, LB_RESETCONTENT, 0, 0);

	GetColumnOrder(m_order);
    GetColumnShown(m_shown);
                    
    
    /* Get the Column Order and save it */
    m_pThisPlaylist->LV_ColumnGetRealOrder(m_order);

    for (i = 0 ; i < COLUMN_MAX; i++)
    {        
        HWND hList = (m_shown[m_order[i]] == 1) ? m_hListShown : m_hListAvailable;
        
		int idx = SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)(LPTSTR)gDefault_column[m_order[i]].szName);		
		SendMessage(hList, LB_SETITEMDATA, idx, (LPARAM)m_order[i]);
    }
    
	
	
	
	// Show number in playlist
	CheckDlgButton(m_hDlg, IDC_CHK_SHOWNUMBER, (UINT)(gParam.bShowNumber == 1 ? BST_CHECKED : BST_UNCHECKED));  
	// Show Grid option
	CheckDlgButton(m_hDlg, IDC_CHK_SHOWNGRID,  (UINT)(gParam.bShowGrid == 1   ? BST_CHECKED : BST_UNCHECKED));  
	

	// Show option information from ID3v1 or ID3v2
	SendDlgItemMessage(m_hDlg, IDC_CBO_SHOW_COL_ID3V1,  CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(m_hDlg, IDC_CBO_SHOW_COL_ID3V1,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szID3v[0]);	
	SendDlgItemMessage(m_hDlg, IDC_CBO_SHOW_COL_ID3V1,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szID3v[1]);	
	SendDlgItemMessage(m_hDlg, IDC_CBO_SHOW_COL_ID3V1,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szID3v[2]);	
	SendDlgItemMessage(m_hDlg, IDC_CBO_SHOW_COL_ID3V1,  CB_SETCURSEL, gParam.nShowID3v1, 0);

	return TRUE;
}


/* ************************************************************** */
BOOL  CDlgColumn::SaveParam()
{
	int nShown = SendMessage(m_hListShown,     LB_GETCOUNT, 0, 0);
	int nAvail = SendMessage(m_hListAvailable, LB_GETCOUNT, 0, 0);
	int nCount = 0;
	int i;
	int idx;

	if (nShown == 0)
	{	
		MessageBox(m_hDlg, TEXT_MSG_SELECT_COLUMN, TEXT_ERROR, MB_OK | MB_ICONSTOP);
		return FALSE;
	}


	for (i = 0; i < nShown ; i++)
	{
		idx = (int)SendMessage(m_hListShown, LB_GETITEMDATA, (WPARAM)i, 0);
		m_order[nCount++] = idx;
		m_shown[idx] = TRUE;
	}
	
	for (i = 0; i < nAvail ; i++)
	{
		idx = (int)SendMessage(m_hListAvailable, LB_GETITEMDATA, (WPARAM)i, 0);
		m_order[nCount++] = idx;
		m_shown[idx] = FALSE;
	}

	SetColumnOrder(m_order);
	SetColumnShown(m_shown);

	gParam.bShowNumber = (IsDlgButtonChecked(m_hDlg, IDC_CHK_SHOWNUMBER) == BST_CHECKED ? 1 : 0);
	gParam.bShowGrid   = (IsDlgButtonChecked(m_hDlg, IDC_CHK_SHOWNGRID) == BST_CHECKED ? 1 : 0);
	gParam.nShowID3v1  = SendDlgItemMessage(m_hDlg, IDC_CBO_SHOW_COL_ID3V1,  CB_GETCURSEL, 0, 0);

	return TRUE;
}

/* ************************************************************** */
// Returns TRUE if successful
int CDlgColumn::DoExchangeItem(HWND hFrom, HWND hTo)
{
   	int iFrom, iTo;
	TCHAR szTmp[40];
    int nFrom, nTo;
	int order;
	
    nFrom = SendMessage(hFrom, LB_GETCOUNT, 0, 0);
    nTo   = SendMessage(hTo,   LB_GETCOUNT, 0, 0);

	if (nFrom == 0) return FALSE;

	iFrom = SendMessage(hFrom, LB_GETCURSEL, 0, 0);

	if (iFrom == LB_ERR) 
		return FALSE;

	SendMessage(hFrom, LB_GETTEXT, iFrom, (LPARAM) (LPCTSTR)szTmp );
	order =  (int)SendMessage(hFrom, LB_GETITEMDATA, (WPARAM)iFrom, 0);
	SendMessage(hFrom, LB_DELETESTRING, iFrom, 0);	
	SendMessage(hFrom, LB_SETCURSEL, iFrom, 0);

	iTo = SendMessage(hTo, LB_ADDSTRING, 0, (LPARAM)(LPTSTR)szTmp);		
	SendMessage(hTo, LB_SETITEMDATA, iTo, (LPARAM)order);	


	m_bColumnsChange  = TRUE;

    return FALSE;
}


/* ************************************************************** */
void CDlgColumn::DoExchangeItemALL(HWND hFrom, HWND hTo)
{
   	int nCountFrom = SendMessage(hFrom, LB_GETCOUNT, 0, 0);

	if (nCountFrom == 0) 
		return;

	for (int i=0; i<nCountFrom; i++)
	{
		SendMessage(hFrom, LB_SETCURSEL, 0, 0);
		DoExchangeItem(hFrom, hTo);
	}

	return;
}

/* ************************************************************** */
void CDlgColumn::DoMoveItem( HWND hwndlist, BOOL bMoveUp)
{
	int nCount, i;
	TCHAR szTmp[40];
	int pos1, pos2;
	int order;

	nCount = SendMessage(hwndlist, LB_GETCOUNT, 0, 0);

	i = SendMessage(hwndlist, LB_GETCURSEL, 0, 0);

	if (i == LB_ERR) 
		return;
	if (nCount == 0)
		return;
	if (i <= 0 && bMoveUp == TRUE)
		return;
	if ((i+1) >= nCount && bMoveUp== FALSE)
		return;

	if (bMoveUp == FALSE)
	{
		pos1 = i;	pos2 = i+1;
	}
	else
	{
		pos1 = i;	pos2 = i-1;
	}
	
	// get the string from the listbox
	SendMessage(hwndlist,LB_GETTEXT, pos1, (LPARAM)szTmp);	
	order =  (int)SendMessage(hwndlist, LB_GETITEMDATA, (WPARAM)pos1, 0);
	// Delete old Item
	SendMessage(hwndlist,LB_DELETESTRING, pos1, 0);

	// insert Item at new position
	i = pos2;
	if (i >= SendMessage(hwndlist,LB_GETCOUNT, 0, 0)) 
		i = -1;	// if last item, append tstring
	
	SendMessage(hwndlist,LB_INSERTSTRING, i, (LPARAM)szTmp);
	SendMessage(hwndlist, LB_SETITEMDATA, i, (LPARAM)order);
	SendMessage(hwndlist,LB_SETCURSEL, pos2, 0);

	m_bColumnsChange  = TRUE;
}




// //////////////////////////////
// End part for columns Dialog
// //////////////////////////////