#include "priv.h"
#include <commctrl.h>
#include <winbase.h>		// MoveFile: Rename file

#include "gen.h"
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"


// external declaration
extern winampGeneralPurposePlugin plugin;
extern BOOL CALLBACK Edit_WndProc(HWND , UINT , WPARAM , LPARAM );

// number of pages in the main TAB-window
struct 
{
	int DlgID;
	LPCTSTR Name;
	int iconID;
} 
ID3Pages[NUMID3PAGES] = 
{
	{ IDD_PAGE_EDIT,			 TEXT_PAGE_EDITTAG				, IDI_ICON8  },
	{ IDD_PAGE_MASSTAG,			 TEXT_PAGE_MASSTAG				, IDI_ICON9  },
	{ IDD_PAGE_RENAMEFILETAG,	 TEXT_PAGE_RENAMEFILE			, IDI_ICON10 },
	{ IDD_PAGE_WRITETAGFROMFILE, TEXT_PAGE_WRITETAG				, IDI_ICON11 },
	{ IDD_PAGE_CONVERSION,		 TEXT_PAGE_CONVERSIONREMOVE		, IDI_ICON12 },
	{ IDD_PAGE_ORGANIZE,		 TEXT_PAGE_ORGANIZE				, IDI_ICON14 }
};


/* ************************************************************************ */
CDlgEditMain::CDlgEditMain(CDlgPlaylist* pThis, int nPageCurrent): CDlg()
{
	m_pThisPlaylist = pThis;
	m_ID3PageActive = nPageCurrent;

	m_hwndPage = NULL;
	m_CDlgPage = NULL;
}

/* ************************************************************************ */
BOOL CDlgEditMain::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		InitTab();
		return FALSE;
	
	case WM_DESTROY:
		return 0;


	case WM_PAINT:
		{
			HDC hDC;
			PAINTSTRUCT ps;

			hDC = BeginPaint(m_hDlg, &ps);
			UpdateBanner(hDC);
			EndPaint(m_hDlg, &ps);
		}
		return TRUE;
		
	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
			case IDOK:
			case IDCANCEL:
				if (m_hwndPage != NULL)
				{	SendMessage(m_hwndPage, WM_CLOSE, 0, 0);
					delete m_CDlgPage;
				}
				EndDialog(m_hDlg,0);
				return TRUE;
		}
		return TRUE;

	case WM_NOTIFY:
		{
			LPNMHDR hdr = (LPNMHDR)lParam;

			if (hdr->code == TCN_SELCHANGE)
				UpdateID3Page();
			break;
		}
	}
	return FALSE;
}


/* ************************************************************************ */
void CDlgEditMain::InitTab()
{
	// Add the ID3Pages
	TC_ITEM tie;

	m_hwndTabCtrl = GetDlgItem(m_hDlg, IDC_ID3_TAB);

	for (int i=0; i<NUMID3PAGES; i++)
	{
		tie.mask = TCIF_TEXT;
		tie.pszText = (LPTSTR) ID3Pages[i].Name;
		TabCtrl_InsertItem(m_hwndTabCtrl, i, &tie);
	}


	TabCtrl_SetCurSel(m_hwndTabCtrl, m_ID3PageActive);


	UpdateID3Page();
}


/* ************************************************************************ */
void CDlgEditMain::UpdateID3Page()
{
	CDlg *dlg = NULL;
	
	m_ID3PageActive = TabCtrl_GetCurSel(m_hwndTabCtrl);

	HDC hDC;
	hDC = GetDC(m_hDlg);
	UpdateBanner(hDC);
	ReleaseDC(m_hDlg, hDC);


	// Destroy the current child dialogbox if any
	if (m_hwndPage != NULL)
	{
		SendMessage(m_hwndPage, WM_CLOSE, 0, 0);
		//::DestroyWindow(m_hwndPage);
		delete m_CDlgPage;
		m_CDlgPage = NULL;

	}
		
	switch(m_ID3PageActive)
	{
		case 0:	dlg = new CDlgID3Edit(m_pThisPlaylist);					break;
		case 1:	dlg = new CDlgID3MassTag(m_pThisPlaylist);				break;
		case 2:	dlg = new CDlgID3RenameFileFromTag(m_pThisPlaylist);	break;
		case 3:	dlg = new CDlgID3WriteTagFromFile(m_pThisPlaylist);		break;
		case 4:	dlg = new CDlgID3ConversionTag(m_pThisPlaylist);		break;
		case 5:	dlg = new CDlgOrganize(m_pThisPlaylist);				break;
	}
	
	if (dlg != NULL)
	{

		m_hwndPage = dlg->DoModeless(ID3Pages[m_ID3PageActive].DlgID, m_hDlg);

		m_CDlgPage = dlg;
	
		// move the window in the right position
		RECT tabWndPos;
	
		GetWindowRect(m_hwndTabCtrl, &tabWndPos);
		ScreenToClient(m_hDlg, (LPPOINT)&tabWndPos);
		TabCtrl_AdjustRect(m_hwndTabCtrl, FALSE, &tabWndPos);
		SetWindowPos(m_hwndPage, HWND_TOP, tabWndPos.left, tabWndPos.top, 0, 0, SWP_NOSIZE|SWP_SHOWWINDOW);
	}

	
}

/* ************************************************************************ */
void CDlgEditMain::UpdateBanner(HDC hDC)
{
	RECT rect;
	
	HFONT hFont, hOldFont;
	TCHAR szTitle[100];
	COLORREF OldColor;
	
	GetClientRect(m_hDlg, &rect);

	rect.bottom = 32;
	FillRect(hDC, &rect, (HBRUSH)GetStockObject(WHITE_BRUSH));

	SetBkMode(hDC, TRANSPARENT);
	hFont = CreateFont(28, 0, 0, 0, FW_BOLD, FALSE, FALSE,0,0,0,0,0,0, _T("Arial") );

	hOldFont = (HFONT)SelectObject(hDC, hFont);
	// draw text in DC
	DWORD dwSysCol = GetSysColor(COLOR_3DFACE);

	if (dwSysCol == RGB(255,255,255))
		dwSysCol = RGB(0,0,0);
	OldColor = SetTextColor(hDC,  dwSysCol); 
	
	rect.left = 4;
	_tcscpy(szTitle, ID3Pages[m_ID3PageActive].Name);
	DrawText(hDC,  szTitle, _tcslen(szTitle), &rect, DT_SINGLELINE | DT_LEFT );

	// restore old text color, Font
	SetTextColor(hDC, OldColor);
	SelectObject(hDC, hOldFont);

	if (hFont != NULL)
		DeleteObject (hFont);

	// Draw Icon
	HICON hIcon = LoadIcon(plugin.hDllInstance, MAKEINTRESOURCE(ID3Pages[m_ID3PageActive].iconID));
	if(hIcon)	
		DrawIconEx(hDC, rect.right-32, 0, hIcon, 0, 0, 0, NULL, DI_NORMAL );

	if (hIcon != NULL)
		DeleteObject (hIcon);


}



