#include "priv.h"
#include <commctrl.h>

#include "gen.h"			//general purpose plugin Winamp
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"
#include "ID3V1.h"


// external declaration
extern winampGeneralPurposePlugin plugin;
extern STATGENRESTRUCT gStatGenre[ID3TAG_MAX_GENRE+2];
extern INITSTRUCT gParam;

// Internal declaration
typedef struct tag_myitemfilter
{	LPTSTR szName;
	int nCount;
	BOOL bCheck;
} MYITEMFILTER; 


#ifndef ListView_SetCheckState
   #define ListView_SetCheckState(hwndLV, i, fCheck)	\
      ListView_SetItemState(hwndLV, i,					\
		INDEXTOSTATEIMAGEMASK((fCheck)+1), LVIS_STATEIMAGEMASK)
#endif


// Array List type filter
const TCHAR *CDlgFilter::m_listTypeFilter[MAX_TYPEFILTER] = 
{
	TEXT_GENRE,
	TEXT_ARTIST, 
	TEXT_ALBUM, 
	TEXT_COMMENT
};

#define BMWIDTH				16
#define BMHEIGHT			16
#define RGBREPLACE			0x00FF00FF // solid magenta


/* ************************************************************************ */
CDlgFilter::CDlgFilter(CDlgPlaylist* pThis): CDlg()
{
	m_pThisPlaylist = pThis;
	
	m_newArtist  = m_pThisPlaylist->m_filterArtist;
	m_newAlbum   = m_pThisPlaylist->m_filterAlbum;
	m_newComment = m_pThisPlaylist->m_filterComment;

	m_nOldSortColumnFilter = 0; 
	m_bFilterChange        = FALSE; 
	m_bSortOrderAscending  = TRUE; 

	m_hIml = NULL;
}




/* ************************************************************** */
BOOL CDlgFilter::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg) 
    {
    case WM_INITDIALOG:
		Init();
		LoadGenre();
        return FALSE;

	case WM_NOTIFY:
		NotifyHandler(uMsg, wParam, lParam, plugin.hDllInstance);
		return TRUE;

	case WM_CONTEXTMENU: 
		if(DoContextMenu(wParam, lParam)) 
			return FALSE; 
		break; 
	
	case WM_DESTROY:
		m_StrArray.clear();
		if(m_hIml)
		{
			ImageList_Destroy(m_hIml);
			m_hIml = NULL;
		}
		return TRUE;
	
	case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			case IDC_CBO_TYPE_FILTER:	    
				if (HIWORD(wParam) == CBN_SELCHANGE)
				{
					HCURSOR hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
					
					SaveFilter();
					LoadFilter();
					ShowCtrl();

					SetCursor(hcurOld);

				}
				return TRUE;
			
			case ID_FILTER_SELECTALL:	// from menu
				SelectAll(TRUE);
				return FALSE;

			case ID_FILTER_SELECTNONE:	
				SelectAll(FALSE);
				return FALSE;
			
			case ID_FILTER_SELECTTOGGLE:
				SelectToggle();
				return FALSE;

			case IDOK:
			case IDCANCEL:
				SaveGenre();
				SaveFilter();
				m_pThisPlaylist->m_filterArtist.assign(m_newArtist);
				m_pThisPlaylist->m_filterAlbum.assign(m_newAlbum);
				m_pThisPlaylist->m_filterComment.assign(m_newComment);
				EndDialog(m_hDlg, (int)m_bFilterChange);
				return TRUE;

			case IDC_BUTTON_NOFILTER:
				{
					int i;
					for (i = 0; i<ID3TAG_MAX_GENRE+2; i++)
					{
						if (gStatGenre[i].nCount > 0)
							gStatGenre[i].bCheck = TRUE;
					}
	
					m_pThisPlaylist->m_filterArtist.assign(_T(""));
					m_pThisPlaylist->m_filterAlbum.assign(_T(""));
					m_pThisPlaylist->m_filterComment.assign(_T(""));
				}
				m_bFilterChange = TRUE;
				EndDialog(m_hDlg, (int)m_bFilterChange);
				return TRUE;

			}

    }
    return FALSE;
}

/* ************************************************************** */
void CDlgFilter::Init()
{
	LV_COLUMN lvc;				// list view column structure
	TCHAR szText[80];
	HWND hlvwfilter;

	// ///////////////////////////
	// Initialize combox Filter by
	SendDlgItemMessage(m_hDlg, IDC_CBO_TYPE_FILTER,  CB_RESETCONTENT, 0, 0);
	for (int i=0; i<MAX_TYPEFILTER; i++)
		SendDlgItemMessage(m_hDlg, IDC_CBO_TYPE_FILTER,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)m_listTypeFilter[i]);	
	SendDlgItemMessage(m_hDlg, IDC_CBO_TYPE_FILTER,  CB_SETCURSEL, 0, 0);	

	m_nTypeFilter = 0;

	// ///////////////////	
	// Initialize Listview
	hlvwfilter = GetDlgItem(m_hDlg, IDC_LVW_FILTER);
	ListView_SetExtendedListViewStyle(hlvwfilter, LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES );

	// Delete columns
	ListView_DeleteColumn(hlvwfilter,0);
	ListView_DeleteColumn(hlvwfilter,1);

	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvc.pszText = szText;
	
	lvc.iSubItem = 0;							// Genre
	lvc.fmt = LVCFMT_LEFT;	
	lvc.cx = 140;			
	_tcscpy(szText, TEXT_COLUMNFILTER_GENRE);
	ListView_InsertColumn (hlvwfilter, 0, &lvc);
	lvc.iSubItem = 1;							// Count genre
	lvc.fmt = LVCFMT_RIGHT;	
	lvc.cx = 70;			
	_tcscpy(szText, TEXT_COLUMNFILTER_COUNT);
	ListView_InsertColumn (hlvwfilter, 1, &lvc);

	// ///////////////////	
	// Initialize Treeview
	m_hIml = ImageList_LoadBitmap(plugin.hDllInstance, MAKEINTRESOURCE(IDB_FILTERTYPE), BMWIDTH, 0, RGBREPLACE);

	if (m_hIml != NULL)
		TreeView_SetImageList(GetDlgItem(m_hDlg, IDC_TREEVIEW_FILTER), m_hIml, 0);

	
	ShowCtrl();



}

/* ************************************************************** */
void CDlgFilter::ShowCtrl()
{
	m_nTypeFilter = SendDlgItemMessage(m_hDlg, IDC_CBO_TYPE_FILTER,  CB_GETCURSEL, 0, 0);
	
	BOOL bShowGenre = (m_nTypeFilter == 0 ? TRUE : FALSE);

	ShowWindow(GetDlgItem(m_hDlg, IDC_LVW_FILTER), bShowGenre ? SW_SHOW : SW_HIDE);
	ShowWindow(GetDlgItem(m_hDlg, IDC_FILTERSELECTALL), bShowGenre ? SW_SHOW : SW_HIDE);
	ShowWindow(GetDlgItem(m_hDlg, IDC_FILTERSELECTNONE), bShowGenre ? SW_SHOW : SW_HIDE);
	ShowWindow(GetDlgItem(m_hDlg, IDC_FILTERSELECTTOGGLE), bShowGenre ? SW_SHOW : SW_HIDE);
	ShowWindow(GetDlgItem(m_hDlg, IDC_TREEVIEW_FILTER), bShowGenre ? SW_HIDE : SW_SHOW);

	RefreshTitle();
}



/* ************************************************************** */
void CDlgFilter::RefreshTitle()
{
	TCHAR szTitle[MAX_PATH];
	TCHAR *szID3v[] = { _T("ID3v1"), _T("ID3v2"), _T("ID3 ?") };

	_stprintf(szTitle, TEXT_TITLEFILTER, m_listTypeFilter[m_nTypeFilter], szID3v[gParam.nShowID3v1]);
	SetWindowText(m_hDlg, (LPCTSTR)szTitle);
}



/* ************************************************************** */
void CDlgFilter::LoadGenre()
{
	LV_ITEM lvi;				// list view item structure
	TCHAR szText[80];
	int i, iItem;
	HWND hlvwfilter;

	hlvwfilter = GetDlgItem(m_hDlg, IDC_LVW_FILTER);

	ListView_DeleteAllItems(hlvwfilter); 

	 // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
	lvi.iImage = 0;
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	iItem = 0;
	for (i = 0; i<ID3TAG_MAX_GENRE+2; i++)
	{
		if (gStatGenre[i].nCount > 0)
		{
			BOOL bCheck;
			MYITEMFILTER *pItem;

			_tcscpy(szText, GenreInt2Str((unsigned char)i));

			pItem = (MYITEMFILTER *)LocalAlloc(LPTR, sizeof(MYITEMFILTER));          
			pItem->szName = DupString(szText);
			pItem->nCount = gStatGenre[i].nCount;
			pItem->bCheck = gStatGenre[i].bCheck;

			lvi.iItem = iItem; 
			lvi.iSubItem = 0;         
			lvi.lParam = (LPARAM) pItem;    // item data  
			ListView_InsertItem(hlvwfilter, &lvi);  

			bCheck = gStatGenre[i].bCheck;

			ListView_SetCheckState(hlvwfilter, iItem, bCheck);

			iItem++;
		}		
	}
}


/* ************************************************************** */
void CDlgFilter::LoadFilter()
{
	TV_INSERTSTRUCT	tvis;
	HTREEITEM		hItem, hItemSelected, hRootItem;
	TCHAR			szTemp[MAX_PATH];
	tstring strFilter;
	HWND hwndTV = GetDlgItem(m_hDlg, IDC_TREEVIEW_FILTER);
	int i, fid = ID3TAG_TPE1;
	tstring str;

	m_nTypeFilter = SendDlgItemMessage(m_hDlg, IDC_CBO_TYPE_FILTER,  CB_GETCURSEL, 0, 0);

	switch(m_nTypeFilter)
	{
		case 1: 
			fid = ID3TAG_TPE1; 
			strFilter = m_newArtist;
			break;
		case 2: 
			fid = ID3TAG_TALB; 
			strFilter = m_newAlbum;
			break;
		case 3: 
			fid = ID3TAG_COMM; 
			strFilter = m_newComment;
			break;
	}

	// Erase all items and all elts in vector
	m_StrArray.clear();
	TreeView_DeleteAllItems(hwndTV);

	// root nodes All xxx
	_stprintf(szTemp, TEXT_ALLFILTER, m_listTypeFilter[m_nTypeFilter]);
	
	tvis.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	tvis.hParent             = NULL;
	tvis.hInsertAfter        = NULL;
	tvis.item.iImage         = 0;
	tvis.item.iSelectedImage = 0;
	tvis.item.lParam         = NULL;
	tvis.item.pszText        = szTemp;		
	hRootItem = TreeView_InsertItem(hwndTV, &tvis);


	// Scan all playlist  from memory 
	for (i=0; i < m_pThisPlaylist->m_minfo.GetCount(); i++)
	{
		CMP3Info *mp3i = m_pThisPlaylist->m_minfo.GetInfo(i);
		str.assign(_T(""));
		m_pThisPlaylist->GetFrameInfo(mp3i, fid, str);
		if (str.empty() == FALSE)
			AddVector(str);
	}

	hItemSelected = NULL;
	for (i=0; i < m_StrArray.size(); i++)
	{
		_tcscpy(szTemp, (LPTSTR)m_StrArray[i].c_str());
		tvis.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;
		tvis.hParent             = hRootItem;
		tvis.item.pszText        = szTemp;	
		tvis.item.iImage         = m_nTypeFilter;
		tvis.item.iSelectedImage = m_nTypeFilter;
		hItem = TreeView_InsertItem(hwndTV, &tvis);
		
		if (m_StrArray[i].compare(strFilter)==0)
			hItemSelected = hItem;
	}
	
	TreeView_Expand(hwndTV, hRootItem, TVE_EXPAND);

	// No found filter --> select all xxx
	if (strFilter.empty())
		hItemSelected = hRootItem;

	TreeView_SelectItem(hwndTV, hItemSelected);
	TreeView_EnsureVisible(hwndTV, hItemSelected);
	
	SetFocus(hwndTV);
}

/* ************************************************************** */
void CDlgFilter::SaveFilter()
{
	if (m_nTypeFilter == 0)
		return;

	HWND hwndTV = GetDlgItem(m_hDlg, IDC_TREEVIEW_FILTER);
	HTREEITEM hItem     = TreeView_GetSelection(hwndTV);
	HTREEITEM hItemRoot = TreeView_GetRoot(hwndTV);
	TCHAR szText[1024];
	
	if (!hItem)
		return;

	TV_ITEM	tvItem;
	tvItem.mask = TVIF_TEXT;
	tvItem.hItem = hItem;
	tvItem.pszText = szText;
	tvItem.cchTextMax = 1024;
	if (TreeView_GetItem(hwndTV, &tvItem) == FALSE)
		return;

	if (hItem == hItemRoot)
	{
		szText[0] = '\0'; // no filtre all xxx
	}

	switch(m_nTypeFilter)
	{
		case 1:	m_newArtist.assign(szText);		break;
		case 2: m_newAlbum.assign(szText);		break;
		case 3: m_newComment.assign(szText);	break;
	}

/*
	if (m_pThisPlaylist->m_filterArtist.compare(m_newArtist) >0)
		m_bFilterChange = TRUE;
	if (m_newArtist.compare(m_pThisPlaylist->m_filterArtist) >0)
		m_bFilterChange = TRUE;

	if (m_pThisPlaylist->m_filterAlbum.compare(m_newAlbum) >0)
		m_bFilterChange = TRUE;
	if (m_newAlbum.compare(m_pThisPlaylist->m_filterAlbum) >0)
		m_bFilterChange = TRUE;

	if (m_pThisPlaylist->m_filterComment.compare(m_newComment) >0)
		m_bFilterChange = TRUE;
	if (m_newComment.compare(m_pThisPlaylist->m_filterComment) >0)
		m_bFilterChange = TRUE;
*/

	if (m_pThisPlaylist->m_filterArtist.compare(m_newArtist))
		m_bFilterChange = TRUE;
	if (m_pThisPlaylist->m_filterAlbum.compare(m_newAlbum))
		m_bFilterChange = TRUE;
	if (m_pThisPlaylist->m_filterComment.compare(m_newComment))
		m_bFilterChange = TRUE;
	
}

int CDlgFilter::AddVector(tstring& str)
{
	// add into the array. since this is sorted, find the right place for the insertion
	int pos = FindVector(str);
	
	switch(pos)
	{
		case -2:	// existe deja pas d'ajout
			return pos;
		
		case -1:	// ajoute a la fin
			m_StrArray.push_back(str);
			pos = m_StrArray.size()-1;
			break;
		default:	// ajoute juste apres
			m_StrArray.insert(&m_StrArray.at(pos), str);
			break;
	}

	return pos;
}

/**
 * find the string which is greater than the current Array
 * this is used, to maintain a sorted list
 * returns -1 if no bigger info is found
 */
int CDlgFilter::FindVector(tstring& str)
{
	for (int i=0; i<m_StrArray.size(); i++)
	{
		int ret = m_StrArray[i].compare(str);
		
		if (ret > 0)
			return(i);			
		else if (ret == 0)
			return -2;			// Existe deja
	}
	return -1;
}



/* ************************************************************** */
int CDlgFilter::SaveGenre()
{
	int i, nCount, idx, nCountCheck;
	HWND hlvwfilter;
	BOOL bCheckState;
	LV_ITEM lvi;
	MYITEMFILTER *pItem;

	hlvwfilter = GetDlgItem(m_hDlg, IDC_LVW_FILTER);
	
	nCount = ListView_GetItemCount(hlvwfilter);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask = LVIF_PARAM ; 
 
	for (nCountCheck = 0, i = 0; i < nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(hlvwfilter, &lvi);

		pItem = (MYITEMFILTER*)lvi.lParam;    // item data  
			
		idx = GenreStr2Int(pItem->szName);
			
		if (ListView_GetCheckState(hlvwfilter, i))
		{
			bCheckState = TRUE;
			nCountCheck++;
		}
		else
			bCheckState  = FALSE;

		if (gStatGenre[idx].bCheck != bCheckState) 
				m_bFilterChange = TRUE;

		
		gStatGenre[idx].bCheck = bCheckState;
			
	}
	return (nCountCheck);
}



/****************************************************************************
* 
*    FUNCTION: NotifyHandler(HWND, UINT, UINT, LONG)
*
*    PURPOSE: This function is the handler for the WM_NOTIFY that is 
*    sent to the parent of the list view window.
*
****************************************************************************/
LRESULT CDlgFilter::NotifyHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, HINSTANCE hInst)
{
	LV_DISPINFO FAR * pnmv = (LV_DISPINFO FAR *) lParam;
	NM_LISTVIEW *pNm = (NM_LISTVIEW *)lParam;
	
	
	switch(((LPNMHDR) lParam)->code)
	{
		case LVN_GETDISPINFO:
			// Provide the item or subitem's text, if requested. 
		    if (pnmv->item.mask & LVIF_TEXT) 
			{ 
				TCHAR szCol[80];
				MYITEMFILTER *pItem = (MYITEMFILTER *) (pnmv->item.lParam); 


				switch(pnmv->item.iSubItem)
				{
					case 0: _tcscpy(szCol, (LPTSTR)pItem->szName); break;
					case 1: _stprintf(szCol, _T("%d"), pItem->nCount); break;
				}

				_tcscpy(pnmv->item.pszText, szCol);
				
			}
			break;

		case LVN_COLUMNCLICK:
			if (m_nOldSortColumnFilter == pNm->iSubItem)
				m_bSortOrderAscending = !m_bSortOrderAscending;

			m_nOldSortColumnFilter = pNm->iSubItem;
			ListView_SortItems(pNm->hdr.hwndFrom, ListViewCompareProc,	this);
			break;

		default:
			break;
	}
	return 0L;
}


/**************************************************************************** */
int CALLBACK CDlgFilter::ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort )
{     
	MYITEMFILTER *pItem1 = (MYITEMFILTER *) lParam1; 
    MYITEMFILTER *pItem2 = (MYITEMFILTER *) lParam2;      
	CDlgFilter *pThis = (CDlgFilter *)lParamSort;
	int iCmp; 

	// Compare the specified column. 
	switch(pThis->m_nOldSortColumnFilter)
	{
		case 0:
			iCmp = lstrcmpi(pItem1->szName, pItem2->szName);
			break;
		case 1:
			if (pItem1->nCount == pItem2->nCount)
				iCmp = 0;
			else if (pItem1->nCount > pItem2->nCount)
				iCmp = 1;
			else
				iCmp = -1;
			break;

	}

	if (pThis->m_bSortOrderAscending == FALSE)
		iCmp *= -1;
	
	return (iCmp); 
}

/* ************************************************************ */
BOOL CDlgFilter::DoContextMenu(WPARAM wParam, LPARAM lParam) 
{ 
	
	if (SendDlgItemMessage(m_hDlg, IDC_CBO_TYPE_FILTER,  CB_GETCURSEL, 0, 0) == 0)
	{
		RECT rect;
		POINT pt;
		pt.x = LOWORD(lParam);
		pt.y = HIWORD(lParam);

		GetWindowRect(GetDlgItem(m_hDlg,IDC_LVW_FILTER), &rect);

		if (PtInRect(&rect, pt))
		{
			HMENU hMenuLoad, hMenu; 
			hMenuLoad = LoadMenu(plugin.hDllInstance, MAKEINTRESOURCE(IDR_MENU1)); 
			hMenu = GetSubMenu(hMenuLoad, 2); 

			TrackPopupMenu(hMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
						  pt.x, pt.y, 0, m_hDlg, NULL); 

			DestroyMenu(hMenuLoad); 
		}
	}
	return TRUE; 
} 

/* ************************************************************** */
void CDlgFilter::SelectAll(BOOL bSelect)
{
	int i, nCount;
	HWND hlvwfilter = GetDlgItem(m_hDlg, IDC_LVW_FILTER);
	nCount = ListView_GetItemCount(hlvwfilter);
 
	for (i = 0; i < nCount; i++)
		ListView_SetCheckState(hlvwfilter, i, (bSelect ? 1: 0));
}

/* ************************************************************** */
void CDlgFilter::SelectToggle()
{
	int i, nCount;
	BOOL bCheckState;

	HWND hlvwfilter = GetDlgItem(m_hDlg, IDC_LVW_FILTER);
	nCount = ListView_GetItemCount(hlvwfilter);
 
	for (i = 0; i < nCount; i++)
	{
		if (ListView_GetCheckState(hlvwfilter, i))
			bCheckState = 0;
		else
			bCheckState = 1;

		ListView_SetCheckState(hlvwfilter, i, bCheckState);
	}
}

