#include "priv.h"
#include <commctrl.h>
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"


// external declaration
extern INITSTRUCT gParam;

extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);


/* ************************************************************************ */
BOOL CDlgID3ConversionTag::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 
		InitFileList();
		return FALSE;
	
	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;
	
		case IDC_TAG_LIST_FILE: 	
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;

		case IDC_TAG_BTN_ID3V1TOID3V2:
		case IDC_TAG_BTN_ID3V2TOID3V1:
		case IDC_TAG_BTN_REMOVEID3V1:
		case IDC_TAG_BTN_REMOVEID3V2:
		case IDC_TAG_BTN_REMOVEID3V1_ID3V2:
			{
				int nAction = (LOWORD(wParam) - IDC_TAG_BTN_ID3V1TOID3V2);
				BtnGO(nAction);
				return TRUE;
			}

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		
		}
	}
	return FALSE;
}



/* ************************************************************************ */
void CDlgID3ConversionTag::BtnGO(int nAction)
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected, bLog;
	const TCHAR *Msg_label[] = {TEXT_FAILED, TEXT_SUCCESS};
	TCHAR szMsg[1024];

	bSelected  = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ONLYSELECTED)== BST_CHECKED);
	

	if (!bSelected)
	{	// All Items
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,	LB_GETCOUNT, 0, 0);
	}
	else
	{	// Only selected item 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,	LB_GETSELCOUNT, 0, 0);
		if (nCount == LB_ERR || nCount == 0)
		{	MessageBox(m_hDlg, TEXT_MSG_SELECT_FILE, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return; 
		}
	
		pArray = (int*)malloc(nCount*sizeof(int));
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELITEMS, (WPARAM)nCount, (LPARAM)pArray);
	}
	

	// Msgbox confirm before execute ?
	if (MsgConfirm(nCount, pArray, 3+nAction) == FALSE)
	{
		if (bSelected == TRUE)
			free(pArray);
		return;
	}

	
	
	bLog = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_LOG)== BST_CHECKED);
	
	if (bLog)
	{
		TCHAR szFileLog[MAX_PATH];
		szFileLog[0] = '\0';
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG), (LPTSTR)szFileLog, sizeof(szFileLog));
		
		if (_tcslen(szFileLog) == 0)
		{	MessageBox(m_hDlg, TEXT_MSG_CHOOSELOG, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return;
		}
		if ((filelog = _tfopen(szFileLog, _T("w+t"))) == NULL)
		{	MessageBox(m_hDlg, TEXT_MSG_ERRLOG, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return;
		}
	
		switch(nAction)
		{
		case 0: _ftprintf(filelog, TEXT_LOG_COPYID3V1_ID3V2, nCount);  break;
		case 1: _ftprintf(filelog, TEXT_LOG_COPYID3V2_ID3V1, nCount);	break;
		case 2: _ftprintf(filelog, TEXT_LOG_REMOVEID3V1, nCount);	break;
		case 3: _ftprintf(filelog, TEXT_LOG_REMOVEID3V2, nCount);	break;
		case 4: _ftprintf(filelog, TEXT_LOG_REMOVEID3V1_ID3V2, nCount); break;
		}

		_ftprintf(filelog, TEXT_LOG_DATE, GetFormattedDateTime());
	}

	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	ActiveButton(FALSE);
	

	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_CONVERSION_STATUS);
	ShowWindow(hProgress, SW_SHOW);
	ShowWindow(hStatus, SW_SHOW);
	SendMessage(hProgress, PBM_SETPOS, 1, 0);	// these two are required to display the empty bar
	SendMessage(hProgress, PBM_SETPOS, 0, 0);
	SendMessage(hProgress, PBM_SETRANGE, 0, MAKELPARAM(0, nCount)); 
	SendMessage(hProgress, PBM_SETSTEP, (WPARAM) 1, 0);

	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		bOK = FALSE;
		DWORD dwLen = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETITEMDATA , (WPARAM)(bSelected ? pArray[i] : i), 0);
		CMP3Info *mp3i = (CMP3Info *)dwLen;

		switch(nAction)
		{
			case 0: 
				bOK = CopyID3v1_ID3v2(bSelected ? pArray[i]:i); 
				break;
			case 1: 
				bOK = CopyID3v2_ID3v1(bSelected ? pArray[i]:i); 
				break;
			case 2: 
				bOK = RemoveID3Tag(bSelected ? pArray[i]:i, VERSION_ID3V1); 	
				break;
			case 3: 
				bOK = RemoveID3Tag(bSelected ? pArray[i]:i, VERSION_ID3V2); 	
				break;
			case 4: 
				bOK = RemoveID3Tag(bSelected ? pArray[i]:i, VERSION_ID3V1+VERSION_ID3V2);
				break;
		}
		
		if (bLog)
			_ftprintf(filelog, TEXT_LOG_FILE, i+1, Msg_label[bOK], mp3i->GetCFilename());	
		
		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, TEXT_STATUS_WORKING, i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);
		
	}
	
	ActiveButton(TRUE);
	SetCursor(hcurOld);

	if (bSelected)
		free(pArray);

	if (bLog)
		fclose(filelog);

	// Show resume action in MsgBox
	switch(nAction)
	{
		case 0: _stprintf(szMsg, TEXT_RESULT_COPYID3V1_ID3V2, nCount);	break;
		case 1: _stprintf(szMsg, TEXT_RESULT_COPYID3V2_ID3V1, nCount);	break;
		case 2: _stprintf(szMsg, TEXT_RESULT_REMOVEID3V1, nCount);	break;
		case 3: _stprintf(szMsg, TEXT_RESULT_REMOVEID3V2, nCount);	break;
		case 4: _stprintf(szMsg, TEXT_RESULT_REMOVEID3V1_ID3V2, nCount);	break;
	}
	MessageBox(m_hDlg, szMsg, TEXT_INFORMATION, MB_OK | MB_ICONINFORMATION);

	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}



/* ************************************************************************ */
BOOL CDlgID3ConversionTag::RemoveID3Tag(int iItem, int nID3Version)
{
	CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,	LB_GETITEMDATA , (WPARAM)iItem, 0);
	if (!mp3i)	return FALSE;

	// ////////////////////
	// Remove TAG ID3v2
	// ////////////////////
	mp3i->DeleteTag(nID3Version);
	if (mp3i->SaveTag() != ID3V2_ERR_OK)
		return FALSE;

	mp3i->ParseFilename();
	return TRUE;
}


/* ************************************************************************ */
BOOL CDlgID3ConversionTag::CopyID3v1_ID3v2(int iItem)
{
	CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,	LB_GETITEMDATA , (WPARAM)iItem, 0);
	if (!mp3i)	
		return FALSE;

	if (mp3i->HaveID3v1() == FALSE)
		return FALSE;

	struct ID3V1MP3Info *id3i = mp3i->GetID3v1();

	if (!id3i)
		return FALSE;

	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TPE1); // Artist
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TIT2); // Title, song name
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TALB); // Album
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TYER); // Year
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TCON); // Genre
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_COMM); // Comment
	CopyInfoID3v1_ID3v2(mp3i, id3i, ID3TAG_TRCK); // Track
	
	mp3i->SetTagVersion(VERSION_ID3V1 + VERSION_ID3V2);

	if (mp3i->SaveTag() != ID3V2_ERR_OK)
		return FALSE;
	
	return TRUE;
}
/* ************************************************************************ */
BOOL CDlgID3ConversionTag::CopyID3v2_ID3v1(int iItem)
{
	CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,	LB_GETITEMDATA , (WPARAM)iItem, 0);
	if (!mp3i)	
		return FALSE;

	if (mp3i->HaveID3v2() == FALSE)
		return FALSE;

	struct ID3V1MP3Info id3iNew;
	memset(&id3iNew, 0, sizeof(struct ID3V1MP3Info));


	// ////////////////////////////////////
	// Copy information from ID3v2 to ID3v1
	// ////////////////////////////////////
	CMP3StringFrame *sf;

	// Artist
	if (sf = mp3i->GetStringFrame(ID3TAG_TPE1)) {
		t2ancpy((LPSTR)&id3iNew.Artist, sf->GetString(), sizeof(id3iNew.Artist));
	}

	// Title,  SongName
	if (sf = mp3i->GetStringFrame(ID3TAG_TIT2)) {
		t2ancpy((LPSTR)&id3iNew.SongName, sf->GetString(), sizeof(id3iNew.SongName));
	}

	// Album
	if (sf = mp3i->GetStringFrame(ID3TAG_TALB)) {
		t2ancpy((LPSTR)&id3iNew.Album, sf->GetString(), sizeof(id3iNew.Album));
	}

	// Year
	if (sf = mp3i->GetStringFrame(ID3TAG_TYER)) {
		t2ancpy((LPSTR)&id3iNew.Year, sf->GetString(), sizeof(id3iNew.Year));
	}

	// genre
	if (sf = mp3i->GetStringFrame(ID3TAG_TCON)) {
		id3iNew.Genre = GenreStr2Int(sf->GetString().c_str());
	}

	// Comment
	if (sf = mp3i->GetStringFrame(ID3TAG_COMM)) {
		t2ancpy((LPSTR)&id3iNew.Comment, sf->GetString(), sizeof(id3iNew.Comment));
	}
	
	// Track
	if (sf = mp3i->GetStringFrame(ID3TAG_TRCK)) {
		id3iNew.Track = _ttoi(sf->GetString().c_str());
	}

	
	mp3i->SetTagVersion(VERSION_ID3V1 + VERSION_ID3V2);
	mp3i->SetID3v1(&id3iNew);

	if (mp3i->SaveTag() != ID3V2_ERR_OK)
		return FALSE;
	
	return TRUE;
}

/* ************************************************************************ */
void CDlgID3ConversionTag::ActiveButton(BOOL bEnabled)
{
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_ID3V1TOID3V2), bEnabled);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_ID3V2TOID3V1), bEnabled);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_REMOVEID3V1), bEnabled);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_REMOVEID3V2), bEnabled);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_REMOVEID3V1_ID3V2), bEnabled);
}


/* ************************************************************************ */
void CDlgID3ConversionTag::CopyInfoID3v1_ID3v2(CMP3Info *mp3i, struct ID3V1MP3Info *id3i, int fid)
{
	CMP3Frame *mp3f = NULL;
	tstring str = _T("");

	// ////////////////////////////////////
	// Copy information from ID3v1 to ID3v2
	// ////////////////////////////////////
	switch(fid)
	{
		case ID3TAG_COMM: // read the Comment
			if (!IsStringEmpty((LPSTR)id3i->Comment)) 
				a2tcpy(str, (LPSTR)id3i->Comment);
			break;

		case ID3TAG_TYER: // read the Year
			if (!IsStringEmpty((LPSTR)id3i->Year)) 
				a2tcpy(str, (LPSTR)id3i->Year);
			break;

		case ID3TAG_TALB: // read the Album
			if (!IsStringEmpty((LPSTR)id3i->Album)) 
				a2tcpy(str, (LPSTR)id3i->Album);
			break;

		case ID3TAG_TPE1: // read the Artist
			if (!IsStringEmpty((LPSTR)id3i->Artist)) 
				a2tcpy(str, (LPSTR)id3i->Artist);
			break;

		case ID3TAG_TIT2:  // read the Title:Song name
			if (!IsStringEmpty((LPSTR)id3i->SongName)) 
				a2tcpy(str, (LPSTR)id3i->SongName);
			break;

		case ID3TAG_TCON: // read the genre
			a2tcpy(str, (LPSTR)GenreInt2Str(id3i->Genre));
			break;

		case ID3TAG_TRCK: // read the Track
			if (id3i->Track != 0) 
			{
				TCHAR szTrack[4];
				_stprintf(szTrack, _T("%d"), id3i->Track);
				a2tcpy(str, (LPSTR)szTrack);
			}

	}


	mp3f = new CMP3StringFrame(fid, str);

	if (mp3f)
		mp3i->ReplaceFrame(mp3f);

	return;
}


