#include "priv.h"
#include <commctrl.h>

#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"

#define   ID3V2_CREATE_ERROR_ARRAY 1
#include "ID3V2Errors.h"
#undef    ID3V2_CREATE_ERROR_ARRAY

// external declaration util.cpp
void SetHookAccelEditTag(BOOL bActive, int idAccel, HWND hDlg);


/* ************************************************************************ */
BOOL CDlgID3Edit::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		SetHookAccelEditTag(TRUE, IDR_ACCELERATOR2, m_hDlg);
		Init();
		InitFileCbo();
		EnabledID3v1();
		EnabledID3v2();
		return FALSE;

	case WM_CLOSE:
		IsDirty();
		SetHookAccelEditTag(FALSE, IDR_ACCELERATOR2, NULL);
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_DRAWITEM:
		OwnerDrawBtnSwap((LPDRAWITEMSTRUCT)lParam);		
		return TRUE;

	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDOK:
		case IDCANCEL:
			IsDirty();
			EndDialog(m_hDlg,0);
			return TRUE;

		case IDK_EDIT_PREVIOUS:	// shortcut goto Previous file
			GotoFile(-1);
			return TRUE;
		case IDK_EDIT_NEXT:		// shortcut goto next file
			GotoFile(+1);
			return TRUE;

		case IDC_TAG_LIST_FILE:	    
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				IsDirty();	
				ShowInfo();	
			}
			return TRUE;

		case IDC_TAG_EDIT_TITLE:	// ID3v1	    
		case IDC_TAG_EDIT_ARTIST:	    
		case IDC_TAG_EDIT_ALBUM:	    
		case IDC_TAG_EDIT_COMMENT:	    
		case IDC_TAG_EDIT_YEAR:	  
		case IDC_TAG_EDIT_TRACK:	

		case IDC_TAG_EDIT_TITLE2:	 // ID3v2
		case IDC_TAG_EDIT_ARTIST2:	    
		case IDC_TAG_EDIT_ALBUM2:	    
		case IDC_TAG_EDIT_COMMENT2:	    
		case IDC_TAG_EDIT_YEAR2:	  
		case IDC_TAG_EDIT_TRACK2:	
		case IDC_TAG_EDIT_INITKEY2:
		case IDC_TAG_EDIT_BPM2:
		case IDC_TAG_EDIT_SUBTITLE2:
		case IDC_TAG_EDIT_REMIXED2:
		case IDC_TAG_EDIT_COMPOSER2:

			if (HIWORD(wParam) == EN_CHANGE) 
				Changed(TRUE);
			return TRUE;
		

		case ID_SWAPID3V1_TITLE_ARTIST:
		case ID_SWAPID3V1_ARTIST_TITLE:
			SwapFieldID3(IDC_TAG_EDIT_ARTIST, IDC_TAG_EDIT_TITLE);
			return TRUE;
		case ID_SWAPID3V1_ALBUM_TITLE:
		case ID_SWAPID3V1_TITLE_ALBUM:
			SwapFieldID3(IDC_TAG_EDIT_ALBUM, IDC_TAG_EDIT_TITLE);
			return TRUE;
		case ID_SWAPID3V1_ARTIST_ALBUM:
		case ID_SWAPID3V1_ALBUM_ARTIST:
			SwapFieldID3(IDC_TAG_EDIT_ARTIST, IDC_TAG_EDIT_ALBUM);
			return TRUE;

		case ID_SWAPID3V2_ARTIST_TITLE:
		case ID_SWAPID3V2_TITLE_ARTIST:
			SwapFieldID3(IDC_TAG_EDIT_ARTIST2, IDC_TAG_EDIT_TITLE2);
			return TRUE;
		case ID_SWAPID3V2_ALBUM_TITLE:
		case ID_SWAPID3V2_TITLE_ALBUM:
			SwapFieldID3(IDC_TAG_EDIT_ALBUM2, IDC_TAG_EDIT_TITLE2);
			return TRUE;
		case ID_SWAPID3V2_ARTIST_ALBUM:
		case ID_SWAPID3V2_ALBUM_ARTIST:
			SwapFieldID3(IDC_TAG_EDIT_ARTIST2, IDC_TAG_EDIT_ALBUM2);
			return TRUE;


		case IDC_TAG_EDIT_GENRE:	    
			if (HIWORD(wParam) == CBN_SELCHANGE)
				Changed(TRUE);
			return TRUE;

		case IDC_BTN_TAG_EDIT_WRITE:
			WriteTag();
			return TRUE;

		case IDC_TAG_EDIT_SWAPFIELD_ID3V1_TITLE:
			MenuButton(LOWORD(wParam), IDR_MENUSWAP, 0); // 0:SwapID3v1 SubMenu
			return TRUE;
		case IDC_TAG_EDIT_SWAPFIELD_ID3V1_ARTIST:
			MenuButton(LOWORD(wParam), IDR_MENUSWAP, 1); 
			return TRUE;
		case IDC_TAG_EDIT_SWAPFIELD_ID3V1_ALBUM:
			MenuButton(LOWORD(wParam), IDR_MENUSWAP, 2); 
			return TRUE;
	case IDC_TAG_EDIT_SWAPFIELD_ID3V2_TITLE:
			MenuButton(LOWORD(wParam), IDR_MENUSWAP, 3); // 0:SwapID3v2 SubMenu
			return TRUE;
		case IDC_TAG_EDIT_SWAPFIELD_ID3V2_ARTIST:
			MenuButton(LOWORD(wParam), IDR_MENUSWAP, 4); 
			return TRUE;
		case IDC_TAG_EDIT_SWAPFIELD_ID3V2_ALBUM:
			MenuButton(LOWORD(wParam), IDR_MENUSWAP, 5); 
			return TRUE;


		case IDC_TAG_CHK_EDIT_ID3V1:
			EnabledID3v1();
			return TRUE;
		
		case IDC_TAG_CHK_EDIT_ID3V2:
			EnabledID3v2();
			return TRUE;
		
		case IDC_BTN_TAG_EDIT_ID3V1_ID3V2:
			CopyID3v1_ID3v2();
			return TRUE;

		case IDC_BTN_TAG_EDIT_ID3V2_ID3V1:
			CopyID3v2_ID3v1();
			return TRUE;

		
		}
	}
	return FALSE;
}





/* ************************************************************************ */
void CDlgID3Edit::Init()
{
	HWND hWndCbo, hWndCbo2;

	// Set MAX LENGTH Edit TAG
	SendDlgItemMessage( m_hDlg, IDC_TAG_EDIT_TRACK, EM_SETLIMITTEXT , 3L, 0L);
	SendDlgItemMessage( m_hDlg, IDC_TAG_EDIT_YEAR, EM_SETLIMITTEXT , 4L, 0L);
	SendDlgItemMessage( m_hDlg, IDC_TAG_EDIT_TITLE, EM_SETLIMITTEXT , 30L, 0L);
	SendDlgItemMessage( m_hDlg, IDC_TAG_EDIT_ARTIST, EM_SETLIMITTEXT , 30L, 0L);
	SendDlgItemMessage( m_hDlg, IDC_TAG_EDIT_ALBUM, EM_SETLIMITTEXT , 30L, 0L);
	SendDlgItemMessage( m_hDlg, IDC_TAG_EDIT_COMMENT, EM_SETLIMITTEXT , 30L, 0L);


	// Load Genre in Combobox
	hWndCbo  = GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE);
	hWndCbo2 = GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE2);
	
	SendMessage(hWndCbo,  CB_RESETCONTENT, 0, 0);
	SendMessage(hWndCbo2, CB_RESETCONTENT, 0, 0);

	for (LPTSTR *genre=ID3_Genres; *genre; genre++)
	{
		SendMessage(hWndCbo,  CB_ADDSTRING, 0, (LPARAM)*genre);	
		SendMessage(hWndCbo2, CB_ADDSTRING, 0, (LPARAM)*genre);	
	}

}


/* ************************************************************************ */
void CDlgID3Edit::GotoFile(int nMove)
{
	int nCount, idx;

	IsDirty();	

	nCount = ListView_GetItemCount(m_hwndList);


	idx = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  CB_GETCURSEL, 0, 0);

	idx += nMove;

	if (idx < 0) 
		idx = 0;
	if (idx > nCount) 
		idx = nCount;

	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  CB_SETCURSEL, idx, 0);
	
	ShowInfo();

}


/* ************************************************************************ */
void CDlgID3Edit::InitFileCbo()
{
	int i, nCount, idx, idxSelected;
	LV_ITEM lvi;
	
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, CB_RESETCONTENT, 0, 0);

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_PARAM; 
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) 
			continue;
			
		idx = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, CB_ADDSTRING, 0, (LPARAM)(LPTSTR)mp3i->GetCFilename());		
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, CB_SETITEMDATA, idx, (LPARAM)mp3i);		

		if (m_pThisPlaylist->m_mp3iSelArray	 != NULL)	
			if (m_pThisPlaylist->m_mp3iSelArray[0] == mp3i)
				idxSelected = idx;
	}		

	// select info for item seletec song from the playlist
	// Se positionne sur l'item selectionne dans la playlist 
	// en Mode Edit ID3Tag uniquement
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  CB_SETCURSEL, idxSelected, 0);
	ShowInfo();
}


/* ************************************************************************ */
void CDlgID3Edit::ShowInfo()
{
	DWORD dwLen, dwIndex;
	CMP3Info *mp3i;

	m_bIsLoading = TRUE;

	dwIndex = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  CB_GETCURSEL, 0, 0);
	if (dwIndex == CB_ERR) 	return;

	m_dwCurSel = dwIndex;

	dwLen = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  CB_GETITEMDATA , (WPARAM)dwIndex, 0);
	if (dwLen == CB_ERR)	return;

	mp3i = (CMP3Info *)dwLen;
	
	if (!mp3i) 
		return;

	// Si pas de selection cree en une
	if (m_pThisPlaylist->m_nSelCount == 0)
	{	if (m_pThisPlaylist->m_mp3iSelArray == NULL)
		{
			m_pThisPlaylist->m_nSelCount = 1;
			m_pThisPlaylist->m_mp3iSelArray = (CMP3Info **)calloc(m_pThisPlaylist->m_nSelCount, sizeof(CMP3Info *));
		}
	}

	m_pThisPlaylist->m_mp3iSelArray[0] = mp3i;

	TCHAR szNULL[1];
	szNULL[0] = '\0';

	
#define SETTEXT(id, str) \
		SetWindowText(GetDlgItem(m_hDlg, id), (LPSTR)str);

	// ////////////
	// ID3v2
	if (mp3i->HaveID3v2()==FALSE)
	{
		CheckDlgButton(m_hDlg,IDC_TAG_CHK_EDIT_ID3V2, BST_UNCHECKED);
		SETTEXT( IDC_TAG_EDIT_ARTIST2,  szNULL);
		SETTEXT( IDC_TAG_EDIT_TITLE2,   szNULL);
		SETTEXT( IDC_TAG_EDIT_ALBUM2,   szNULL);
		SETTEXT( IDC_TAG_EDIT_COMMENT2, szNULL);
		SETTEXT( IDC_TAG_EDIT_YEAR2,    szNULL);
		SETTEXT( IDC_TAG_EDIT_TRACK2,   szNULL);
		SETTEXT( IDC_TAG_EDIT_GENRE2,   szNULL);
		
		SETTEXT( IDC_TAG_EDIT_INITKEY2, szNULL);
		SETTEXT( IDC_TAG_EDIT_BPM2, 	szNULL);
		SETTEXT( IDC_TAG_EDIT_REMIXED2, szNULL);
		SETTEXT( IDC_TAG_EDIT_SUBTITLE2,szNULL);
		SETTEXT( IDC_TAG_EDIT_COMPOSER2,szNULL);

	}
	else
	{	
		CheckDlgButton(m_hDlg,IDC_TAG_CHK_EDIT_ID3V2, BST_CHECKED);

		for (int fid=0; fid<ID3TAG_NUMTAGS; fid++)
			SetTextMP3info(fid, mp3i);
	}
	EnabledID3v2();


	// ////////////
	// ID3v1
	if (mp3i->HaveID3v1()==FALSE)
	{
		CheckDlgButton(m_hDlg,IDC_TAG_CHK_EDIT_ID3V1, BST_UNCHECKED);	
		SETTEXT( IDC_TAG_EDIT_ARTIST, szNULL);
		SETTEXT( IDC_TAG_EDIT_TITLE,  szNULL);
		SETTEXT( IDC_TAG_EDIT_ALBUM,  szNULL);
		SETTEXT( IDC_TAG_EDIT_COMMENT,szNULL);
		SETTEXT( IDC_TAG_EDIT_YEAR,   szNULL);
		SETTEXT( IDC_TAG_EDIT_TRACK,  szNULL);
		SendDlgItemMessage(m_hDlg, IDC_TAG_EDIT_GENRE,  CB_SETCURSEL, 0, 0);
	}
	else
	{
		CheckDlgButton(m_hDlg,IDC_TAG_CHK_EDIT_ID3V1, BST_CHECKED);

		struct ID3V1MP3Info *id3i = mp3i->GetID3v1();
		if (!id3i)	return;

		
		SETTEXT( IDC_TAG_EDIT_ARTIST,  id3i->Artist);
		SETTEXT( IDC_TAG_EDIT_TITLE,   id3i->SongName);
		SETTEXT( IDC_TAG_EDIT_ALBUM,   id3i->Album);
		SETTEXT( IDC_TAG_EDIT_YEAR,    id3i->Year);
		SETTEXT( IDC_TAG_EDIT_COMMENT, id3i->Comment);
		{
			int idx  = SendMessage(GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE),CB_SELECTSTRING, -1, (LPARAM)(LPCSTR)GenreInt2Str(id3i->Genre)); 
			SendMessage(GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE), CB_SETCURSEL, idx, 0);
		}
		
		if (id3i->Track != 0)
		{	
			TCHAR szTrack[4];
			_stprintf(szTrack, _T("%d"), id3i->Track);
			SETTEXT( IDC_TAG_EDIT_TRACK, szTrack);
		}
		else
			SETTEXT( IDC_TAG_EDIT_TRACK,  szNULL);

		
	}
	
	EnabledID3v1();

#undef SETTEXT
	
	ShowCoreMP3Info(mp3i);

	m_bIsLoading = FALSE;
}


/* ************************************************************************ */
void CDlgID3Edit::ShowCoreMP3Info(CMP3Info *mp3i)
{
	TCHAR temp[1024];
	TCHAR mpeglayer[100];
	TCHAR bitrate[40];

	if (mp3i->isVBR())
		_stprintf(bitrate, _T("%dkb/s (VBR)"), mp3i->GetBitrate());
	else
		_stprintf(bitrate, _T("%dkb/s"), mp3i->GetBitrate());

	
	mp3i->GetMPEGVersionLayer(mpeglayer);

	temp[0] = '\0';
	_stprintf(temp, TEXT_MSG_EDITTAG_MP3INFO1,
		mp3i->GetFileSize(),
		mp3i->GetPlayLength(),
		mpeglayer,	
		bitrate,
		mp3i->GetFrames());

	SetWindowText(GetDlgItem(m_hDlg, IDC_TAG_MPEGINFO1), (LPCTSTR)temp);

	temp[0] = '\0';
	_stprintf(temp, TEXT_MSG_EDITTAG_MP3INFO2,
			mp3i->GetFrequency(),
			ModeName[mp3i->GetMode()], 
			BoolName[mp3i->GetCRC()], 
			BoolName[mp3i->GetCopyright()], 
			BoolName[mp3i->GetOriginal()], 
			EmphasisName[mp3i->GetEmphasis()]);
	
	SetWindowText(GetDlgItem(m_hDlg, IDC_TAG_MPEGINFO2), (LPCTSTR)temp);
}

/* ************************************************************************ */
void CDlgID3Edit::Changed(BOOL bEnabled)
{
	if (m_bIsLoading)
		m_bChange = FALSE;
	else
		m_bChange = bEnabled;
}

/* ************************************************************************ */
void CDlgID3Edit::EnabledID3v1()
{
	int nIDRes[] = {	
						IDC_TAG_EDIT_ARTIST, 
						IDC_TAG_EDIT_TITLE,
						IDC_TAG_EDIT_ALBUM,  
						IDC_TAG_EDIT_COMMENT,
						IDC_TAG_EDIT_YEAR, 
						IDC_TAG_EDIT_TRACK,
						IDC_TAG_EDIT_GENRE,
						IDC_BTN_TAG_EDIT_ID3V1_ID3V2,
						IDC_TAG_EDIT_SWAPFIELD_ID3V1_TITLE,
						IDC_TAG_EDIT_SWAPFIELD_ID3V1_ARTIST,
						IDC_TAG_EDIT_SWAPFIELD_ID3V1_ALBUM 
					};
	
	BOOL bEnabled = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_EDIT_ID3V1)== BST_CHECKED);
	
	for (int i= 0; i <NUMELEMENTS(nIDRes); i++)
		EnableWindow(GetDlgItem(m_hDlg,nIDRes[i]), bEnabled);


}

/* ************************************************************************ */
void CDlgID3Edit::EnabledID3v2()
{
	int nIDRes[] = {	
						IDC_TAG_EDIT_ARTIST2, 
						IDC_TAG_EDIT_TITLE2,
						IDC_TAG_EDIT_ALBUM2,  
						IDC_TAG_EDIT_COMMENT2,
						IDC_TAG_EDIT_YEAR2, 
						IDC_TAG_EDIT_GENRE2,
						IDC_TAG_EDIT_TRACK2,
						IDC_TAG_EDIT_INITKEY2, 
						IDC_TAG_EDIT_BPM2, 	
						IDC_TAG_EDIT_REMIXED2, 
						IDC_TAG_EDIT_SUBTITLE2,
						IDC_TAG_EDIT_COMPOSER2,
						IDC_BTN_TAG_EDIT_ID3V1_ID3V2,
						IDC_TAG_EDIT_SWAPFIELD_ID3V2_TITLE,
						IDC_TAG_EDIT_SWAPFIELD_ID3V2_ARTIST,
						IDC_TAG_EDIT_SWAPFIELD_ID3V2_ALBUM 
					};

	
	BOOL bEnabled = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_EDIT_ID3V2)== BST_CHECKED);

	for (int i= 0; i <NUMELEMENTS(nIDRes); i++)
		EnableWindow(GetDlgItem(m_hDlg,nIDRes[i]), bEnabled);
	
}



/* ************************************************************************ */
void CDlgID3Edit::CopyID3v1_ID3v2()
{
	TCHAR szTemp[32];
	int i, j;

	for (i=IDC_TAG_EDIT_TITLE, j=IDC_TAG_EDIT_TITLE2; i <= IDC_TAG_EDIT_COMMENT; i++, j++)
	{
		HWND hEditv1 = GetDlgItem(m_hDlg, i);
		HWND hEditv2 = GetDlgItem(m_hDlg, j);	

		int len = GetWindowTextLength(hEditv1);
		GetWindowText(hEditv1, szTemp, len+1);
		SetWindowText(hEditv2, szTemp);			
	}	


	CheckDlgButton(m_hDlg,IDC_TAG_CHK_EDIT_ID3V2, BST_CHECKED);	
	EnabledID3v2();
}


/* ************************************************************************ */
void CDlgID3Edit::CopyID3v2_ID3v1()
{
	TCHAR szTemp[32];
	int i, j;

	for (i=IDC_TAG_EDIT_TITLE, j=IDC_TAG_EDIT_TITLE2; i <= IDC_TAG_EDIT_COMMENT; i++, j++)
	{
		HWND hEditv1 = GetDlgItem(m_hDlg, i);	
		HWND hEditv2 = GetDlgItem(m_hDlg, j);

		switch(i)
		{
			case IDC_TAG_EDIT_TITLE:	
			case IDC_TAG_EDIT_ARTIST:	    
			case IDC_TAG_EDIT_ALBUM:	    
			case IDC_TAG_EDIT_COMMENT:	    
				GetWindowText(hEditv2, szTemp, 30+1);
				szTemp[30] = '\0';
				SetWindowText(hEditv1, szTemp);	
				break;
			
			case IDC_TAG_EDIT_YEAR:	  
				GetWindowText(hEditv2, szTemp, 4+1);
				szTemp[4] = '\0';
				SetWindowText(hEditv1, szTemp);	
				break;
			
			case IDC_TAG_EDIT_TRACK:	
				GetWindowText(hEditv2, szTemp, 3+1);
				szTemp[3] = '\0';
				SetWindowText(hEditv1, szTemp);	
				break;


			case IDC_TAG_EDIT_GENRE:
				{
					int len = GetWindowTextLength(hEditv2);
					GetWindowText(hEditv2, szTemp, len+1);

					if (GenreStr2Int(szTemp) == 0)
						_tcscpy(szTemp, GenreInt2Str(0));			

					int idx  = SendMessage(hEditv1,CB_SELECTSTRING, -1, (LPARAM)(LPCSTR)szTemp); 
					SendMessage(hEditv1, CB_SETCURSEL, idx, 0);
				}
				break;

		}
		
				
	}
	
	CheckDlgButton(m_hDlg,IDC_TAG_CHK_EDIT_ID3V1, BST_CHECKED);	
	EnabledID3v1();
}



/* ************************************************************************ */
void CDlgID3Edit::IsDirty()
{
	if (m_bChange == TRUE)
	{
		int ret = MessageBox(m_hDlg, TEXT_MSG_EDITTAG_CONFIRM, TEXT_MSG_EDITTAG_INFO, 
			MB_OKCANCEL | MB_ICONQUESTION );

		if (ret == IDOK)
			WriteTag();
	}

	m_bChange = FALSE;
}

/* ************************************************************************ */
void CDlgID3Edit::WriteTag()
{
	DWORD dwLen, dwIndex;
	CMP3Info *mp3i, *mp3iSave; 
	bool bOldID3v2 = FALSE;

	// Save info Edit TAG from type user
	dwIndex = m_dwCurSel;
	
	if (dwIndex == CB_ERR)
	{	MessageBox(m_hDlg, TEXT_MSG_EDITTAG_SELECTFILE, TEXT_MSG_EDITTAG_ERR, MB_OK | MB_ICONSTOP );
		m_bChange = FALSE;
		return;
	}

	dwLen   = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  CB_GETITEMDATA , (WPARAM)dwIndex, 0);
	if (dwLen == CB_ERR)
	{	MessageBox(m_hDlg, TEXT_MSG_EDITTAG_SELECTFILE, TEXT_MSG_EDITTAG_ERR, MB_OK | MB_ICONSTOP );
		m_bChange = FALSE;
		return;
	}

	mp3i = (CMP3Info *)dwLen;
	
	if (!mp3i)
		return;

	
	mp3iSave = new CMP3Info(mp3i->GetFilename());
		
	if (!mp3iSave)
	{
		MessageBox(m_hDlg, TEXT_MSG_EDITTAG_CANTWRITE,  TEXT_MSG_EDITTAG_ERR, MB_OK | MB_ICONSTOP);
		m_bChange = FALSE;	
		return;		
	}
	
	bOldID3v2 = mp3iSave->HaveID3v2(); 

	// Set default save all ID3vx
	mp3iSave->SetTagVersion(VERSION_ID3V1 + VERSION_ID3V2);

	// ///////////////////
	// Save ID3v2 if exist
	// ///////////////////
	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_EDIT_ID3V2)== BST_UNCHECKED)
	{
		mp3iSave->DeleteTag(VERSION_ID3V2);
	}
	else
	{	// Retrieve ID3Tag info
		for (int fid=0; fid<ID3TAG_NUMTAGS; fid++)
			GetTextMP3info(fid, mp3iSave);
	}
	
	// ///////////////////
	// Save ID3v1 if exist
	// ///////////////////
	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_EDIT_ID3V1)== BST_UNCHECKED)
	{
		mp3iSave->DeleteTag(VERSION_ID3V1);
	}
	else
	{
		struct ID3V1MP3Info id3i;
		TCHAR szTemp[100];
		
		memset(&id3i, 0, sizeof(struct ID3V1MP3Info));

		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_TITLE), (LPSTR)&id3i.SongName, sizeof(id3i.SongName));
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_ARTIST), (LPSTR)&id3i.Artist, sizeof(id3i.Artist));
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_ALBUM), (LPSTR)&id3i.Album, sizeof(id3i.Album));
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_COMMENT), (LPSTR)&id3i.Comment, sizeof(id3i.Comment));
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_YEAR), (LPSTR)&id3i.Year, sizeof(id3i.Year));
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_TRACK), (LPSTR)szTemp, 4);
		id3i.Track = _ttoi(szTemp);

		int len = GetWindowTextLength(GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE));
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE), szTemp, len+1);
		id3i.Genre = GenreStr2Int(szTemp);

		mp3iSave->SetID3v1(&id3i);
	}


	if (bOldID3v2 != mp3iSave->HaveID3v2())   
	{	// Show status title bar when ID3v2 changing
		SetWindowText(GetParent(m_hDlg), TEXT_MSG_EDITTAG_STATUSUPDATING);
	}
	

	int nErrID3 = mp3iSave->SaveTag();

	if (nErrID3 != ID3V2_ERR_OK)
	{
		MessageBox(m_hDlg, ID3V2ErrorStrings[nErrID3],  TEXT_MSG_EDITTAG_ERR, MB_OK | MB_ICONSTOP);
		goto finWriteTag;
	}

	mp3i->CopyAll(mp3iSave);
	delete mp3iSave;


finWriteTag:
	SetWindowText(GetParent(m_hDlg), TEXT_TITLE_ID3TOOLS);
	m_bChange = FALSE;
	return;

}




/* ************************************************************************ */
void CDlgID3Edit::SwapFieldID3(int nIDsrc, int nIDdest)
{
	HWND hEditSrc  = GetDlgItem(m_hDlg, nIDsrc);
	HWND hEditDest = GetDlgItem(m_hDlg, nIDdest);	

	int lenSrc  = GetWindowTextLength(hEditSrc);
	int lenDest = GetWindowTextLength(hEditDest);

	LPTSTR szTempSrc  = (LPTSTR)calloc(sizeof(TCHAR), lenSrc+1);
	LPTSTR szTempDest = (LPTSTR)calloc(sizeof(TCHAR), lenDest+1);

	GetWindowText(hEditSrc,  szTempSrc,  lenSrc+1);
	GetWindowText(hEditDest, szTempDest, lenDest+1);
	
	SetWindowText(hEditDest, szTempSrc);
	SetWindowText(hEditSrc, szTempDest);

	free(szTempSrc);
	free(szTempDest);
}

/* ************************************************************************ */
void CDlgID3Edit::OwnerDrawBtnSwap(LPDRAWITEMSTRUCT lpdis)
{
	HDC hDC;
    RECT rect;
	int offset = 0;

	BOOL bIsDisabled = (lpdis->itemState & ODS_DISABLED);

    hDC = lpdis->hDC;
    CopyRect (&rect, &(lpdis->rcItem));    

	
	UINT uState = DFCS_BUTTONPUSH;
	
	switch (lpdis->itemAction) 
	{	
		case ODA_DRAWENTIRE:  
		case ODA_FOCUS:
			break;
		case ODA_SELECT:
			if (lpdis->itemState & ODS_SELECTED)
			{
				uState = DFCS_BUTTONPUSH | DFCS_PUSHED;
				offset++;
			}
			break;

		default:
			return;

	}

	DrawFrameControl(hDC, &rect, DFC_BUTTON, uState);



	// Dessine les 2 fleches bleues SWAP
    HPEN hPenShadow, hPenOld;
	COLORREF crColor;
	
	if (bIsDisabled)
		crColor = GetSysColor(COLOR_3DSHADOW);
	else
		crColor = RGB(0,0,153);		// Dark Blue	


    hPenShadow = CreatePen(PS_SOLID, 1, crColor);
	
    hPenOld = (HPEN)SelectObject(hDC, hPenShadow);
    
    MoveToEx(hDC, 8+offset, 3+offset, NULL);
    LineTo(hDC, 8+offset,  8+offset);
    MoveToEx(hDC, 4+offset, 8+offset, NULL);
    LineTo(hDC, 8+offset,  8+offset);
    MoveToEx(hDC, 5+offset, 7+offset, NULL);
    LineTo(hDC, 5+offset,  10+offset);
    MoveToEx(hDC, 7+offset, 4+offset, NULL);
    LineTo(hDC, 10+offset,  4+offset);
    
	SetPixel(hDC, 7+offset, 7+offset, crColor);

	SelectObject(hDC, hPenOld);
    DeleteObject (hPenShadow);

	if (bIsDisabled)
	{
		HPEN hPenLight = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DHILIGHT));
		hPenOld = (HPEN)SelectObject(hDC, hPenLight);

		MoveToEx(hDC, 6+offset, 10+offset, NULL);
		LineTo(hDC, 6+offset, 9+offset);
		LineTo(hDC, 8+offset, 9+offset);
		LineTo(hDC, 8+offset, 8+offset);
		LineTo(hDC, 9+offset, 8+offset);
		LineTo(hDC, 9+offset, 5+offset);
		LineTo(hDC, 11+offset,5+offset);
    

		SelectObject(hDC, hPenOld);
		DeleteObject (hPenLight);

	}

}