#include "priv.h"
#include <commctrl.h>

#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"


// external declaration
extern INITSTRUCT gParam;



/* ************************************************************************ */
BOOL CDlgID3MassTag::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		Init();
		LoadParam();
		InitFileList();
		return FALSE;
	
	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_DESTROY:
		SaveParam();
		return 0;

	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;
		
		case IDC_TAG_LIST_FILE:	    
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;

		case IDC_TAG_CHK_ARTIST:
		case IDC_TAG_CHK_TITLE:
		case IDC_TAG_CHK_ALBUM:
		case IDC_TAG_CHK_YEAR:
		case IDC_TAG_CHK_TRACK:	
		case IDC_TAG_CHK_GENRE:
		case IDC_TAG_CHK_COMMENT:
			if (HIWORD(wParam) == BN_CLICKED)
				CheckTag2Edit(LOWORD(wParam));
			return TRUE;

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		case IDC_TAG_BTN_GO:
			BtnGO();
			return TRUE;
		}
	}
	return FALSE;
}



/* ************************************************************************ */
void CDlgID3MassTag::Init()
{
	CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 

	// Load Genre in Combobox
	HWND hWndCbo = GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE2);
	SendMessage(hWndCbo, CB_RESETCONTENT, 0, 0);

	for (LPTSTR *genre=ID3_Genres; *genre; genre++)
		SendMessage(hWndCbo, CB_ADDSTRING, 0, (LPARAM)*genre);	
}

/* ************************************************************** */
void CDlgID3MassTag::LoadParam()
{
	int i;

	for (i = 0; i < 7; i++)
	{	CheckDlgButton(m_hDlg,IDC_TAG_CHK_TITLE+i,(gParam.nMassCheckTAG & (1<<i))?BST_CHECKED:BST_UNCHECKED);	
		CheckTag2Edit(IDC_TAG_CHK_TITLE+i);
	}

	for (i = 0; i < 2; i++)
		CheckDlgButton(m_hDlg,IDC_TAG_CHK_ID3V1+i,(gParam.nMassTagID3 & (1<<i))?BST_CHECKED:BST_UNCHECKED);	
	
}


/* ************************************************************** */
void CDlgID3MassTag::SaveParam()
{
	int i;

	gParam.nMassCheckTAG = 0;
	gParam.nMassTagID3 = 0;

	for (i = 0; i < 7; i++)
		if (IsDlgButtonChecked(m_hDlg,IDC_TAG_CHK_TITLE+i))
			gParam.nMassCheckTAG |= 1<<i;
	
	for (i = 0; i < 2; i++)
		if (IsDlgButtonChecked(m_hDlg,IDC_TAG_CHK_ID3V1+i))
			gParam.nMassTagID3 |= 1<<i;

}



/* ************************************************************************ */
void CDlgID3MassTag::CheckTag2Edit(int nID)
{
	BOOL bEnabled;
	
	nID -= IDC_TAG_CHK_TITLE;

	bEnabled =  (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_TITLE + nID) == BST_CHECKED);	
	EnableWindow(GetDlgItem(m_hDlg, IDC_TAG_EDIT_TITLE2 + nID), bEnabled);

	EnableGOButton();
}

/* ************************************************************************ */
void CDlgID3MassTag::EnableGOButton()
{
	BOOL bEnableGO = FALSE;

	// if at least one item checked Enable button Go
	for (int i = 0; i <7; i++)
	{	bEnableGO =  (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_TITLE + i) == BST_CHECKED);	
		if (bEnableGO)
			break;

	}
	EnableWindow(GetDlgItem(m_hDlg, IDC_TAG_BTN_GO), bEnableGO);
}


/* ************************************************************************ */
void CDlgID3MassTag::BtnGO()
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected, bLog;
	const TCHAR *Msg_label[] = {TEXT_FAILED, TEXT_SUCCESS};
	TCHAR szMsg[1024];
	
	SaveParam();


	if( gParam.nMassTagID3 == 0)
	{	MessageBox(m_hDlg, TEXT_CHECKID3, TEXT_INFORMATION, MB_OK | MB_ICONINFORMATION);
		return;
	}
	
	

	bSelected  = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ONLYSELECTED)== BST_CHECKED);
	
	if (bSelected == FALSE)
	{	// All Items
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETCOUNT, 0, 0);
	}
	else
	{	// Only selected item 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELCOUNT, 0, 0);
		if (nCount == LB_ERR || nCount == 0)
		{	MessageBox(m_hDlg, TEXT_MSG_SELECT_FILE, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return; 
		}
	
		pArray = (int*)malloc(nCount*sizeof(int));
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELITEMS, (WPARAM)nCount, (LPARAM)pArray);
	}
	

	// Msgbox confirm before execute ?
	if (MsgConfirm(nCount, pArray, 0) == FALSE)
	{
		if (bSelected == TRUE)
			free(pArray);
		return;
	}



	bLog = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_LOG)== BST_CHECKED);
	
	if (bLog == TRUE)
	{
		TCHAR szFileLog[MAX_PATH];
		szFileLog[0] = '\0';
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG), (LPTSTR)szFileLog, sizeof(szFileLog));
		
		if (_tcslen(szFileLog) == 0)
		{	MessageBox(m_hDlg, TEXT_MSG_CHOOSELOG, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return;
		}
		if ((filelog = _tfopen(szFileLog, _T("w+t"))) == NULL)
		{	MessageBox(m_hDlg, TEXT_MSG_ERRLOG, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return;
		}

		_ftprintf(filelog, TEXT_MSG_MASSTAG_LOG, ID3TagName[gParam.nMassTagID3], nCount);
		_ftprintf(filelog, TEXT_LOG_DATE, GetFormattedDateTime());
	}


	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), FALSE);


	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_MASSTAG_STATUS);
	ShowWindow(hProgress, SW_SHOW);
	ShowWindow(hStatus, SW_SHOW);
	SendMessage(hProgress, PBM_SETPOS, 1, 0);	// these two are required to display the empty bar
	SendMessage(hProgress, PBM_SETPOS, 0, 0);
	SendMessage(hProgress, PBM_SETRANGE, 0, MAKELPARAM(0, nCount)); 
	SendMessage(hProgress, PBM_SETSTEP, (WPARAM) 1, 0);


	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETITEMDATA, (WPARAM)(bSelected ? pArray[i] : i), 0);
		
		bOK = MassTAG(mp3i);

		if (bLog == TRUE)
			_ftprintf(filelog, TEXT_LOG_FILE, i+1, Msg_label[bOK], mp3i->GetCFilename());	
		
		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, TEXT_STATUS_WORKING, i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);

	}
	
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), TRUE);
	SetCursor(hcurOld);

	if (bSelected == TRUE)
		free(pArray);

	if (bLog == TRUE)
		fclose(filelog);

	// Show resume action in MsgBox
	_stprintf(szMsg, TEXT_MSG_MASSTAG_RESULT, nCount);
	MessageBox(m_hDlg, szMsg, TEXT_INFORMATION, MB_OK | MB_ICONINFORMATION);

	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}


/* ************************************************************************ */
BOOL CDlgID3MassTag::MassTAG(CMP3Info *mp3i)
{
	if (!mp3i) 
		return FALSE;

	CMP3Info *mp3iSave = new CMP3Info(mp3i->GetFilename());

	if (!mp3iSave)
		return FALSE;		


	// /////////////
	// ID3V1 Section
	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ID3V1)== BST_CHECKED)
	{
		struct ID3V1MP3Info id3i;
		TCHAR szTemp[1024];

		if (mp3iSave->HaveID3v1()==TRUE)
			memcpy(&id3i, mp3iSave->GetID3v1(), sizeof(struct ID3V1MP3Info));
		else
			memset(&id3i, 0, sizeof(struct ID3V1MP3Info));
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_TITLE)== BST_CHECKED)
			GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_TITLE2), (LPSTR)&id3i.SongName, sizeof(id3i.SongName));
			
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ARTIST)== BST_CHECKED)
			GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_ARTIST2), (LPSTR)&id3i.Artist, sizeof(id3i.Artist));
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ALBUM)== BST_CHECKED)
			GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_ALBUM2), (LPSTR)&id3i.Album, sizeof(id3i.Album));
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_YEAR)== BST_CHECKED)
			GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_YEAR2), (LPSTR)&id3i.Year, sizeof(id3i.Year));
	
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_COMMENT)== BST_CHECKED)
			GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_COMMENT2), (LPSTR)&id3i.Comment, sizeof(id3i.Comment));
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_TRACK)== BST_CHECKED)
		{
			GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_TRACK2), (LPSTR)szTemp, 4);
			id3i.Track = _ttoi(szTemp);
		}
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_GENRE)== BST_CHECKED)
		{
			int len = GetWindowTextLength(GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE2));
			GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_GENRE2), szTemp, len+1);
			id3i.Genre = GenreStr2Int(szTemp);
		}

		mp3iSave->SetID3v1(&id3i);

	}


	// /////////////
	// ID3V2 Section
	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ID3V2)== BST_CHECKED)
	{
		if (mp3iSave->HaveID3v2()==FALSE)
			mp3iSave->SetTagVersion(VERSION_ID3V2 +  mp3iSave->GetTagVersion());
		
		// Update Frame if it checked	
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_TITLE)== BST_CHECKED)
			GetTextMP3info(ID3TAG_TIT2, mp3iSave);
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ARTIST)== BST_CHECKED)
			GetTextMP3info(ID3TAG_TPE1, mp3iSave);
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ALBUM)== BST_CHECKED)
			GetTextMP3info(ID3TAG_TALB, mp3iSave);
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_YEAR)== BST_CHECKED)
			GetTextMP3info(ID3TAG_TYER, mp3iSave);
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_TRACK)== BST_CHECKED)
			GetTextMP3info(ID3TAG_TRCK, mp3iSave);
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_COMMENT)== BST_CHECKED)
			GetTextMP3info(ID3TAG_COMM, mp3iSave);
		
		if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_GENRE)== BST_CHECKED)
			GetTextMP3info(ID3TAG_TCON, mp3iSave);

	}

		
	if (mp3iSave->SaveTag() != ID3V2_ERR_OK)
		return FALSE;


	mp3i->CopyAll(mp3iSave);
	delete mp3iSave;

	return TRUE;
}

