#include "priv.h"
#include <commctrl.h>
#include <winbase.h>		// MoveFile: Rename file
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Dlg.h"

// external declaration ShBrowseFolder.cpp
int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData);

// external declaration
extern INITSTRUCT gParam;


/* ************************************************************************ */
BOOL CDlgOrganize::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg)
	{
	case WM_INITDIALOG:
		LoadParam();
		InitFileList(FALSE); // Full name mode
		return FALSE;

	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_DESTROY:
		SaveParam();
		return 0;

	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;

		case IDC_TAG_LIST_FILE:	    
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;

		case IDC_ORGANIZE_BTN_DEST:
			ChooseDirectory();
			return TRUE;

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		case IDC_TAG_BTN_GO:
			BtnGO();
			return TRUE;

		}
	}
	return FALSE;
}





/* ************************************************************** */
void CDlgOrganize::LoadParam()
{
	const TCHAR *szID3v[] = {_T("ID3v1"), _T("ID3v2"), _T("ID3v2 / ID3v1")};
	TCHAR szTmp[80];

	CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 

	// Directory name from field of ID3Tag
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_RESETCONTENT, 0, 0);
	_stprintf(szTmp, _T("..\\%s"), TEXT_ARTIST);
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)szTmp);	
	_stprintf(szTmp, _T("..\\%s"), TEXT_ALBUM);
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)szTmp);	
	_stprintf(szTmp, _T("..\\%s"), TEXT_GENRE);
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)szTmp);	
	_stprintf(szTmp, _T("..\\%s\\%s"), TEXT_ARTIST, TEXT_ALBUM);
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)szTmp);	

	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_SETCURSEL, gParam.nOrganizeField, 0);
	
	// Show option information from ID3v1 or ID3v2
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szID3v[0]);	
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szID3v[1]);	
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_SETCURSEL, gParam.nOrganizeFromID3, 0);
}


/* ************************************************************** */
void CDlgOrganize::SaveParam()
{
	gParam.nOrganizeField  = SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_GETCURSEL, 0, 0);
	gParam.nOrganizeFromID3 = SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_GETCURSEL, 0, 0);
}



/* ************************************************************************ */
void CDlgOrganize::BtnGO()
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected, bLog;
	const TCHAR *Msg_label[] = {TEXT_FAILED, TEXT_SUCCESS};
	TCHAR szMsg[1024];
	TCHAR szPathDest[MAX_PATH];
	
	SaveParam();


	// Path target exist ?	
	szPathDest[0] = '\0';
	GetWindowText(GetDlgItem(m_hDlg, IDC_ORGANIZE_EDIT_DEST), (LPTSTR)szPathDest, sizeof(szPathDest));
	
	if (_tcslen(szPathDest) == 0)
	{	MessageBox(m_hDlg, TEXT_MSG_CHOOSEPATHDEST, TEXT_ERROR, MB_OK | MB_ICONSTOP);
		return;
	}


	// Only selected items Options
	bSelected  = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ONLYSELECTED)== BST_CHECKED);

	if (!bSelected)
	{	// All Items 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETCOUNT, 0, 0);
	}
	else
	{	// Only selected item 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELCOUNT, 0, 0);
		if (nCount == LB_ERR || nCount == 0)
		{	MessageBox(m_hDlg, TEXT_MSG_SELECT_FILE, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return; 
		}
	
		pArray = (int*)malloc(nCount*sizeof(int));
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELITEMS, (WPARAM)nCount, (LPARAM)pArray);
	}
	

	// Msgbox confirm before execute ?
	if (MsgConfirm(nCount, pArray, 8) == FALSE)
	{
		if (bSelected == TRUE)
			free(pArray);
		return;
	}



	// Trace LOG 
	bLog = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_LOG)== BST_CHECKED);
	
	if (bLog)
	{
		TCHAR szFileLog[MAX_PATH];
		szFileLog[0] = '\0';
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG), (LPTSTR)szFileLog, sizeof(szFileLog));
		
		if (_tcslen(szFileLog) == 0)
		{	MessageBox(m_hDlg, TEXT_MSG_CHOOSELOG, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return;
		}
		if ((filelog = _tfopen(szFileLog, _T("w+t"))) == NULL)
		{	MessageBox(m_hDlg, TEXT_MSG_ERRLOG, TEXT_ERROR, MB_OK | MB_ICONSTOP);
			return;
		}

		_ftprintf(filelog, TEXT_MSG_ORGANIZEFILE_LOG, nCount);
		_ftprintf(filelog, TEXT_LOG_DATE, GetFormattedDateTime());
	}

	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), FALSE);

	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_ORGANIZE_STATUS);
	ShowWindow(hProgress, SW_SHOW);
	ShowWindow(hStatus, SW_SHOW);
	SendMessage(hProgress, PBM_SETPOS, 1, 0);	// these two are required to display the empty bar
	SendMessage(hProgress, PBM_SETPOS, 0, 0);
	SendMessage(hProgress, PBM_SETRANGE, 0, MAKELPARAM(0, nCount)); 
	SendMessage(hProgress, PBM_SETSTEP, (WPARAM) 1, 0);

	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		TCHAR szOldFilename[MAX_PATH];

		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETITEMDATA, (WPARAM)(bSelected ? pArray[i] : i), 0);

		_tcscpy(szOldFilename, mp3i->GetCFilename());
		bOK =  OrganizeFile(mp3i, szPathDest);
		
		if (bLog)
			_ftprintf(filelog, TEXT_LOG_ORGANIZE_FILE, i+1, Msg_label[bOK], szOldFilename, mp3i->GetCFilename());	

		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, TEXT_STATUS_WORKING, i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);

	}
	
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), TRUE);
	SetCursor(hcurOld);

	if (bSelected)
		free(pArray);

	if (bLog)
		fclose(filelog);

	// Show resume action in MsgBox
	_stprintf(szMsg, TEXT_MSG_ORGANIZE_RESULT, nCount, szPathDest);
	MessageBox(m_hDlg, szMsg, TEXT_INFORMATION, MB_OK | MB_ICONINFORMATION);

	
	// Refresh list
	InitFileList(FALSE);		// Full name mode
	
	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}


/* ************************************************************************ */
BOOL CDlgOrganize::OrganizeFile(CMP3Info *mp3i, TCHAR *szPathDest)
{
	int nLen, i;
	TCHAR szFile[MAX_PATH];
	TCHAR szFileDest[MAX_PATH];
	TCHAR szDir[MAX_PATH];
	TCHAR szFullPathDest[MAX_PATH];
	TCHAR szTmp[MAX_PATH];
	TCHAR *p;
	
	szFile[0] = '\0';
	szFileDest[0] = '\0';
	szFullPathDest[0] = '\0';
	szDir[0] = '\0';

	GetField2Dir(mp3i, szDir);
	
	// No field for target directory !!!
	nLen = _tcslen(szDir);
	if (nLen < 1)
		return FALSE;


	// Construct target path
	_stprintf(szFullPathDest, _T("%s\\%s"), szPathDest, szDir);
	
	// no directory  --> exit
	if (*szFullPathDest == 0)	
		return FALSE; 
	
	// create directory if no exist
	if ( MakeFullDir(szFullPathDest)== FALSE)
		return FALSE;

	


	// construct the future full file name (Target path + short file name)
	p = _tcsrchr(mp3i->GetCFilename(), '\\');
	if (p)
		_tcscpy(szFile, p+1);
	else
		_tcscpy(szFile, mp3i->GetCFilename());

	_stprintf(szTmp, _T("%s\\%s"), szFullPathDest, szFile);
	

	nLen = _tcslen(szTmp);

	// Bad file name too short
	if (nLen<4)
		return FALSE;	
	
	// Remove  at the end ".mp3"
	p = szTmp + nLen - 4;
	*p= '\0';			
	
	i = 1;

	while (i<100)
	{
		if (i>1)
			_stprintf(szFileDest, _T("%s(%d).mp3"), szTmp, i);
		else
			_stprintf(szFileDest, _T("%s.mp3"), szTmp);

		
		if (FileExists(szFileDest) == FALSE)
		{		
			TCHAR szDrive[MAX_PATH];
			BOOL bRet=0;
			
			_tcscpy(szDrive, mp3i->GetCFilename());

			if (GetDriveType((LPTSTR)szDrive) == DRIVE_CDROM)
				bRet = CopyFile((LPTSTR)mp3i->GetCFilename(), (LPTSTR)szFileDest, TRUE);
			else	
				bRet = MoveFile((LPTSTR)mp3i->GetCFilename(), (LPTSTR)szFileDest);
			

			if (bRet)
			{
				// OK change in memory the new filename
				mp3i->SetFilename(szFileDest);
				return TRUE;
			}
			else
				return FALSE;
		}
		

		i++;

	}

	if (i > 100)
		return FALSE;

	return TRUE;
}


/* ****************************************************************** */
void CDlgOrganize::GetField2Dir(CMP3Info *mp3i, TCHAR *szDir)
{
	if (mp3i->HaveTag() == FALSE)
		return;

	if (gParam.nOrganizeFromID3 == 0)
	{
		// From ID3v1 TAG
		struct ID3V1MP3Info *id3i;
		
		id3i = mp3i->GetID3v1();
		
		if(id3i == NULL)
			return;

		switch(gParam.nOrganizeField)
		{
			default:
			case 0: // Artist
				if (_tcslen((const char*)&id3i->Artist)==0)
					_tcscpy(szDir, TEXT_UNKOWNARTIST);
				else	
					_tcscpy(szDir, (const char*)&id3i->Artist);
				break;
			
			case 1: // Album
				if (_tcslen((const char*)&id3i->Album)==0)
					_tcscpy(szDir,  TEXT_UNKOWNALBUM);
				else	
					_tcscpy(szDir,  (const char*)&id3i->Album);
				break;
			
			case 2: // Genre
				_tcscpy(szDir, (const char*)GenreInt2Str(id3i->Genre)); break;
			
			case 3: // Artist\Album
				if (_tcslen((const char*)&id3i->Artist)==0)
					_tcscpy(szDir, TEXT_UNKOWNARTIST);
				else	
					_tcscpy(szDir, (const char*)&id3i->Artist);
				
				_tcscat(szDir,  _T("\\"));

				if (_tcslen((const char*)&id3i->Album)==0)
					_tcscat(szDir,  TEXT_UNKOWNALBUM);
				else	
					_tcscat(szDir,  (const char*)&id3i->Album);
				break;

		}

	}


	else
	{
		// From ID3v2 Tag
		tstring str = _T("");

		switch(gParam.nOrganizeField)
		{
			default:
			case 0: // ..\Artist
				GetFrameInfo(mp3i, ID3TAG_TPE1, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szDir, TEXT_UNKOWNARTIST);
				else	
					_tcscpy(szDir, str.c_str());
				break;

			case 1: // ..\Album
				GetFrameInfo(mp3i, ID3TAG_TALB, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szDir, TEXT_UNKOWNALBUM);
				else	
					_tcscpy(szDir, str.c_str());
				break;

			case 2: // ..\Genre
				GetFrameInfo(mp3i, ID3TAG_TCON, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szDir, TEXT_UNKOWNGENRE);
				else	
					_tcscpy(szDir, str.c_str());
				break;

			case 3: // ..\Artist\Album
				GetFrameInfo(mp3i, ID3TAG_TPE1, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szDir, TEXT_UNKOWNARTIST);
				else	
					_tcscpy(szDir, str.c_str());

				_tcscat(szDir,  _T("\\"));

				GetFrameInfo(mp3i, ID3TAG_TALB, str);
				if (_tcslen(str.c_str())==0)
					_tcscat(szDir, TEXT_UNKOWNALBUM);
				else	
					_tcscat(szDir, str.c_str());
				break;


		}

	}

	return;
}




/* ************************************************************************ */
BOOL CDlgOrganize::DirectoryExists(TCHAR *szName) 
{ 
	// Check if directory exists 
	DWORD Code = GetFileAttributes(szName); 
	return (Code != -1) && (Code & FILE_ATTRIBUTE_DIRECTORY ); 
} 

/* ************************************************************************ */
BOOL CDlgOrganize::FileExists(TCHAR *szName) 
{ 
	// Check if file exists 
	HANDLE hFile;
	
	hFile = CreateFile((LPCTSTR)szName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	
	if( hFile == INVALID_HANDLE_VALUE )
		return FALSE;
	
	DWORD dwSizeOfFile = ::GetFileSize( hFile, NULL );
	if (dwSizeOfFile == 0xFFFFFFFF)
		return FALSE;
	
	CloseHandle(hFile);

	return TRUE;
	
} 

/* ************************************************************************ */
BOOL CDlgOrganize::MakeFullDir(TCHAR *szDir)
	
{
	// Create full directory, if it doesn't already exist
	if( !DirectoryExists(szDir) ) 
	{
		// First make sure parent dir exists
		TCHAR *p, *path = _tcsdup(szDir);

		if( (p=_tcsrchr(path,'\\')) || (p=_tcsrchr(path,'/')) ) 
		{
			// The parent is a dir, not a drive
			*p = '\0';

			if( !MakeFullDir(path) ) 
			{ 	// Can't create parent
				free(path);
				return FALSE;
			}
		}
		free(path);

		if( !::CreateDirectory(szDir,NULL) ) 
		{
			// Cannot create directory 
			return FALSE;
		}
	}

	return(TRUE);
}


/* ************************************************************************ */
void CDlgOrganize::ChooseDirectory()
{
	static TCHAR szDir[MAX_PATH]= {0};
	LPMALLOC pMalloc; 

	// Get's the Shell's default allocator 
	if (SHGetMalloc(&pMalloc) == NOERROR) 
	{ 
		BROWSEINFO bi; 
		LPITEMIDLIST pidl; 
		
		if (szDir[0] == '\0')
			GetCurrentDirectory(MAX_PATH, szDir);

		// Zero the BROWSEINFO structure to start off with 
		ZeroMemory( &bi, sizeof(bi) ); 
		
		bi.hwndOwner      = m_hDlg;
		bi.pidlRoot       = NULL;
		bi.pszDisplayName = szDir;
		bi.ulFlags        = BIF_RETURNONLYFSDIRS; //BIF_STATUSTEXT|BIF_RETURNONLYFSDIRS; 
		bi.lpfn           = BrowseCallbackProc;		// Set Callback function 
		bi.lParam         = (LPARAM)szDir;
		bi.iImage         = 0;

		// This next call issues the dialog box 
		if ((pidl = ::SHBrowseForFolder(&bi)) != NULL) 
		{ 
			if (::SHGetPathFromIDList(pidl, szDir)) 
			{ 
				// At this point szBuffer contains the selected path 
				TCHAR *p;
				TCHAR szDirSelect[MAX_PATH];
			
				_tcscpy(szDirSelect, szDir);
				p = szDirSelect + _tcslen(szDirSelect) - 1;
				if (*p == '\\')	*p = '\0';
				
				SetWindowText(GetDlgItem(m_hDlg, IDC_ORGANIZE_EDIT_DEST), szDirSelect);
				
			} 
			
			// Free the PIDL allocated by SHBrowseForFolder 
			pMalloc->Free(pidl); 
		} 
		
		// Release the shell's allocator 
		pMalloc->Release(); 
	} 
	
}

