#include "priv.h"

#include <winuser.h>   // GetClassLong
#include <commctrl.h>

#include <wingdi.h>		// Header proc
#include "gen.h"
#include "util.h"
#include "definition.h"
#include "Dlg.h"
#include "ID3V1.h"
#include "Volume.h"
#include "resource.h"

#define MAX_ITEM_SIZ 512  // Taille maxi pour l'affichage d'une colonne pour le listview

// external declaration main.cpp
BOOL CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);
// external declaration util.cpp
void SetHookAccelMain(BOOL bActive, int idAccel, HWND hDlg);
// external declaration ShBrowseFolder.cpp
int CALLBACK BrowseRecurseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData);
// external declaration
extern winampGeneralPurposePlugin plugin;
extern TCHAR szAppName[];
extern HWND ghWndMain;


// for listview header
static LRESULT CALLBACK Header_WndProc(HWND, UINT, WPARAM, LPARAM);
static void             Header_SetSortInfo(HWND hwndList, int nCol, BOOL bAsc);
static void             Header_Initialize(HWND hwndList);
static BOOL             Header_ListCtrlOnErase(HWND hWnd, HDC hDC);

/* sort columns in reverse order */
static BOOL reverse_sort = FALSE;
static UINT m_uHeaderSortCol = 0;
static BOOL m_fHeaderSortAsc = TRUE;


#ifdef STRICT 
	static WNDPROC g_lpHeaderWndProc = NULL;
#else 
	static FARPROC  g_lpHeaderWndProc = NULL;
#endif 


BOOL gbCheckRecurseDir;		// Recursive subdirectories


STATGENRESTRUCT gStatGenre[ID3TAG_MAX_GENRE+2];

COLUMNSSTRUCT gDefault_column[COLUMN_MAX]=
{	
	//  Name         Order  Shown  Width,    Alignement    Is ID3 ? 
	{ TEXT_ARTIST,     0,    1,     120,     LVCFMT_LEFT  ,1},  
	{ TEXT_TITLE,      1,    1,     160,     LVCFMT_LEFT  ,1},  
	{ TEXT_GENRE,      2,    1,      85,     LVCFMT_LEFT  ,1},  
	{ TEXT_YEAR,       3,    1,      45,     LVCFMT_LEFT  ,1},  
	{ TEXT_ALBUM,      4,    1,     160,     LVCFMT_LEFT  ,1},  
	{ TEXT_COMMENT,    5,    1,     160,     LVCFMT_LEFT  ,1},  
	{ TEXT_MODE,       6,    1,      75,     LVCFMT_LEFT  ,0},  
	{ TEXT_BITRATE,    7,    1,      90,     LVCFMT_RIGHT ,0},  
	{ TEXT_FREQ,       8,    1,      70,     LVCFMT_LEFT  ,0},  
	{ TEXT_LAYER,      9,    1,     100,     LVCFMT_LEFT  ,0},  
	{ TEXT_TIME,      10,    1,      50,     LVCFMT_RIGHT ,0},  
	{ TEXT_SIZE,      11,    1,      80,     LVCFMT_RIGHT ,0},  
	{ TEXT_FILENAME,  12,    1,     400,     LVCFMT_LEFT  ,0},  
	{ TEXT_TRACK,     13,    1,      60,     LVCFMT_RIGHT ,1},  
	{ TEXT_ID3TAG,    14,    1,      80,     LVCFMT_LEFT  ,0},   
	{ TEXT_BPM,       15,    1,      60,     LVCFMT_LEFT  ,1} 
};

TEMPLATESTRUCT gaTemplateDefault[MAX_TEMPLATE]=
{
	// Standard 
	{0x00000000, 0x00FFFFFF, 1,0,1,	// CSS1:Text color, background color, Font name, FontStyle, FontSize
		0x00000000, 0x00FFFFFF, 1,1,6,
		0x00000000, 0x00C0C0C0, 1,1,2,	// Attention au codage de la couleur
		0x00000000, 0x00FFFFFF, 1,0,1,  // RGB stockee --> 0x00BBGGRR (inversee)
		0x00FF0000},
	// Winamp
	{0x0000BFFF, 0x00400000, 0,0,1,
		0x0000BFFF, 0x00400000, 0,1,5,
		0x00400000, 0x0000BFFF, 0,1,3, 
		0x00FFFFFF, 0x00400000, 0,0,0, 
		0x00FF9F40},
	// BigBlue
	{0x00505050, 0x00C0C0C0, 1,1,1,
		0x00000000, 0x00B09890, 1,1,1,
		0x00FFFFFF, 0x00806450, 1,1,1, 
		0x00000000, 0x00B09880, 1,1,1, 
		0x00A00000},
	// Desert
	{0x0098CCE0, 0x00000040, 2,0,1,
		0x0098CCE0, 0x00000040, 2,1,5,
		0x00FFFFFF, 0x00000080, 2,0,1, 
		0x00000080, 0x0098CCE0, 2,0,1, 
		0x000000FF},
	// Rain Forest
	{0x00000000, 0x00737973, 0,0,1,
		0x00B5AA9C, 0x00000000, 0,1,5,
		0x009FA49F, 0x00655956, 0,1,3, 
		0x00000000, 0x009FA499, 0,0,0, 
		0x00732800},
	// Rain Forest
	{0x008F97A0,0x003F273F, 0,0,1,
		0x00FFFFFF, 0x00505F70, 0,1,5,
		0x00FFFFFF, 0x005F3F40, 0,1,3, 
		0x00000000, 0x008F97A0, 0,0,0, 
		0x00C0CFD0},
	// 
	{0x00FFFFFF, 0x00000000, 3,1,1,
		0x00000000, 0x00707070, 3,1,1,
		0x00FFFFFF, 0x003F3F3F, 3,1,1, 
		0x00B08F3F, 0x00000000, 3,1,1, 
		0x00B08F3F},
	// 
	{0x003F1F1F, 0x006F776F, 3,1,1,
		0x006F776F, 0x002F0800, 3,1,4,
		0x00000000, 0x00BFBFBF, 3,1,3, 
		0x00303F40, 0x006F776F, 3,1,1, 
		0x00402F30}

};		



INITSTRUCT gParam;




/* ************************************************************************ */
// Constructor initialze member variables
CDlgPlaylist::CDlgPlaylist() 
{
	m_hwndList = NULL;
	m_hwndStat = NULL;
	m_bShowDuplicate = FALSE;
	m_mp3iSelArray = NULL; 
	m_nSelCount = 0;	

	m_nSelLength = 0;
	m_filterArtist.assign(_T(""));
	m_filterAlbum.assign(_T(""));
	m_filterComment.assign(_T(""));
	m_nFilterBy = NO_FILTER;

	m_bRefreshSelLength = TRUE;
	m_last_sort = 0;
}


/* ************************************************************************ */
BOOL CDlgPlaylist::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:
			m_hwndList = GetDlgItem(m_hDlg, IDC_LVW_PLAYLIST);
			
			ghWndMain = m_hDlg;

			// HOWTO: Set the Title Bar Icon in a Dialog Box
			// Last reviewed: December 30, 1998 - Article ID: Q179582 
			m_hIcon = (HICON)LoadImage(plugin.hDllInstance, MAKEINTRESOURCE(IDI_ICON0), IMAGE_ICON, 
                           GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), 0);
			if(m_hIcon)
               SendMessage(m_hDlg, WM_SETICON, ICON_SMALL, (LPARAM)m_hIcon);

			LoadParam();
			Init();
			SetForegroundWindow(m_hDlg);
			return TRUE;
		
/*		case WM_GETMINMAXINFO: 
			// limit minimum size of the main window 
			((MINMAXINFO FAR*)lParam)->ptMinTrackSize.x =  450;
			((MINMAXINFO FAR*)lParam)->ptMinTrackSize.y = 180; 
			return TRUE;
*/
		case WM_CLOSE:
			LV_ColumnSaveSetting();
			DestroyWindow ();
	        return 0;

		case WM_ACTIVATE :
			if ((LOWORD(wParam) == WA_ACTIVE)  || (LOWORD(wParam) == WA_CLICKACTIVE))          
				SetHookAccelMain(TRUE, IDR_ACCELERATOR1, m_hDlg);
			else
				SetHookAccelMain(FALSE, IDR_ACCELERATOR1, NULL);
			return TRUE;

		case WM_DESTROY :
			ghWndMain = NULL;
			if (m_mp3iSelArray != NULL)
				free(m_mp3iSelArray);
			SaveParam();
			LV_Playlist(COMDLG_SAVE);
			ListView_DeleteAllItems(m_hwndList); 
			
			//PostQuitMessage (0);	// MODELESS
			//EndDialog(m_hDlg,0);	// MODAL
			return TRUE;

		case WM_DRAWITEM:
	        OwnerDrawBtn((LPDRAWITEMSTRUCT)lParam);		
			return TRUE;

	   case WM_CONTEXTMENU: 
			if(doContextMenu(wParam, lParam)) 
				return FALSE; 
			break; 

	   case WM_INITMENU:
		   CheckMenuItem((HMENU) wParam, ID_OPTION_VIEW_ID3V1, 
			   (gParam.nShowID3v1 == 0) ? MF_BYCOMMAND | MF_CHECKED : MF_BYCOMMAND | MF_UNCHECKED);
		   CheckMenuItem((HMENU) wParam, ID_OPTION_VIEW_ID3V2, 
			   (gParam.nShowID3v1 == 1) ? MF_BYCOMMAND | MF_CHECKED : MF_BYCOMMAND | MF_UNCHECKED);
		   CheckMenuItem((HMENU) wParam, ID_OPTION_VIEW_ID3V2_ID3V1, 
			   (gParam.nShowID3v1 == 2) ? MF_BYCOMMAND | MF_CHECKED : MF_BYCOMMAND | MF_UNCHECKED);
		   return TRUE;

		case WM_SIZE:
			Resize(lParam);
			return TRUE;
		
		case WM_HSCROLL:
			VolumeSlider(m_hDlg, (HWND)lParam, TRUE);
			return TRUE;

		case WM_DROPFILES:  
			LV_DropFiles((HDROP)wParam);
			return TRUE;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
		
			case ID_FILE_QUIT:
			case IDK_QUIT:
				DestroyWindow ();
				return TRUE;

			case ID_FILE_PROPERTY:
			case IDK_PROPERTY:
				{
					CDlgProperty dlg(this, TRUE);
					dlg.DoModal(IDD_DLGPROPERTY, m_hDlg);
				}
				return TRUE;

			case ID_PLAYLIST_PROPERTY:
			case IDK_PROPERTYLIST:
				{	// Selection properties
					CDlgProperty dlg(this, FALSE);
					dlg.DoModal(IDD_DLGPROPERTY, m_hDlg);
				}
				return TRUE;

			case ID_OPTION_COLUMN:
			case IDK_COLUMN:
				{
					CDlgColumn dlg(this);
					if(dlg.DoModal(IDD_DLGCOLUMN, m_hDlg)==TRUE)
					{
						LV_ColumnResetDisplay(FALSE);
						gParam.NbCol = LV_ColumnGetNum();
					}
				}
				return TRUE;

			case ID_OPTION_RESETTODEFAULT:
				{
					CDlgReset dlg;
					if (dlg.DoModal(IDD_RESET, m_hDlg) == TRUE)
					{
						SaveParam();
						LV_ColumnResetDisplay(FALSE);
						gParam.NbCol = LV_ColumnGetNum();
					}
					
				}
				return TRUE;

			case ID_OPTION_MATCHTIME:
			case IDK_MATCHTIME:
				{
					CDlgMatchTime dlg;
					int nLength = 0;
					
					nLength = dlg.DoModal(IDD_MATCHTIME, m_hDlg);
					if (nLength != 0)
						LV_MatchTime(nLength);
						
				}
				return TRUE;

			case ID_OPTION_VIEW_ID3V1:
			case IDK_VIEW_ID3V1:
				gParam.nShowID3v1 = 0;
				LV_StatGenre();
				GetMP3Selected();
				LV_UpdatePlaylist();
				SetMP3Selected();
				return TRUE;
			case ID_OPTION_VIEW_ID3V2:
			case IDK_VIEW_ID3V2:
				gParam.nShowID3v1 = 1;
				LV_StatGenre();
				GetMP3Selected();
				LV_UpdatePlaylist();
				SetMP3Selected();
				return TRUE;
			case ID_OPTION_VIEW_ID3V2_ID3V1:
			case IDK_VIEW_ID3V2_ID3V1:
				gParam.nShowID3v1 = 2;
				LV_StatGenre();
				GetMP3Selected();
				LV_UpdatePlaylist();
				SetMP3Selected();
				return TRUE;

			case ID_OPTION_OPTION:
			case IDK_OPTION:
				{
					CDlgOption dlg;
					dlg.DoModal(IDD_DLGOPTION, m_hDlg);
				}
				return TRUE;

			case ID_OPTION_HTMLCSSCLASSEDITOR:
			case IDK_HTMLCSSEDITOR:
				{
					CDlgStyleHTML dlg;
					dlg.DoModal(IDD_DLGSTYLEHTML, m_hDlg);
				}
				return TRUE;

			case ID_OPTION_CONFIGURE:
			case IDK_CONFIGURE:
				DialogBox(plugin.hDllInstance,MAKEINTRESOURCE(IDD_DLGCONFIGURE),m_hDlg,(DLGPROC)::ConfigProc);
				return TRUE;
			
			case ID_OPTION_FILTER:
			case IDK_FILTER:
				{
					CDlgFilter dlg(this);
					if(dlg.DoModal(IDD_DLGFILTER, m_hDlg)==TRUE)
					{
						LV_DetermineFilter();
						LV_TitleWindowFilter();
						LV_UpdatePlaylist();
					}

				}
				return TRUE;
			
			case ID_PLAYLIST_EDITTAG:
				SendMessage(m_hDlg,WM_COMMAND, (WPARAM)ID_ID3TAG_EDITID3TAG, NULL);
				return TRUE;

			case ID_ID3TAG_EDITID3TAG:
			case ID_ID3TAG_MASSSETID3:
			case ID_ID3TAG_RENAMEFILEFROMID3:
			case ID_ID3TAG_WRITEID3FROMFILE:
			case ID_ID3TAG_CONVERSION:
			case ID_ID3TAG_ORGANIZE:
			case IDK_EDITTAG:
				{
					int nPage;
					
					if (LOWORD(wParam) == IDK_EDITTAG)
						nPage = 0;
					else
						nPage = (LOWORD(wParam) - ID_ID3TAG_EDITID3TAG);
					
					
					GetMP3Selected();
								
					CDlgEditMain dlg(this, nPage);
					dlg.DoModal(IDD_DLGEDITMAIN, m_hDlg);
					
					LV_StatGenre();
					LV_UpdatePlaylist();
					LV_ColumnSort(m_uHeaderSortCol, FALSE, FALSE); 
					SetMP3Selected();
					
					
				}
				return TRUE;

			case ID__ABOUT:
			case IDK_ABOUTHELP:
				{
					CDlgAbout dlg;
					dlg.DoModal(IDD_DLGABOUT, m_hDlg);
				}
				return TRUE;

			case IDC_BUTTON_FILE:
			case IDC_BUTTON_ADD:
			case IDC_BUTTON_OPTION:
				MenuButton(LOWORD(wParam));				
				return FALSE;

			case ID_SEARCH_ARTIST:
			case ID_SEARCH_TITLE:
			case ID_SEARCH_ALBUM:
			case ID_SEARCH_COMMENT:
			case ID_SEARCH_FILENAME:
			case ID_SEARCH_MATCHCASEMODE:
				MenuCheckItem(LOWORD(wParam)-ID_SEARCH_ARTIST);				
				LV_UpdatePlaylist();
				return FALSE;

			case ID_SEARCH_SELECTALL:
				gParam.nOptionSearch = 31;
				LV_UpdatePlaylist();
				return FALSE;
			case ID_SEARCH_SELECTNONE:
				gParam.nOptionSearch = 0;
				LV_UpdatePlaylist();
				return FALSE;
			case IDK_FIND:
				SetFocus(GetDlgItem(m_hDlg, IDC_EDIT_SEARCH));
				return TRUE;

			case IDC_EDIT_SEARCH:
				if (HIWORD(wParam) == EN_UPDATE)
					LV_UpdatePlaylist();
				return TRUE;

			case IDK_SEARCH_ARTIST:
			case IDK_SEARCH_TITLE:
			case IDK_SEARCH_ALBUM:
			case IDK_SEARCH_COMMENT:
			case IDK_SEARCH_FILENAME:
				MenuCheckItem(LOWORD(wParam)-IDK_SEARCH_ARTIST);				
				return FALSE;

			case ID_FILENEW:
			case IDK_NEW:
				LV_New();
				return FALSE;

			case ID_FILEOPEN:
			case IDK_OPEN:
				LV_ComDlg(COMDLG_OPEN);
				return FALSE;
			
			case ID_FILESAVE:
			case ID_FILESAVEAS:
			case IDK_SAVE:
				LV_ComDlg(COMDLG_SAVE);
				return FALSE;
			
			case ID_FILEIMPORT:
				LV_Import();
				return FALSE;

			case ID_FILEEXPORT:
				LV_Export();
				return FALSE;

			case ID_ADD_ADDFILES:
			case IDK_ADDFILES:
				LV_AddFiles();
				return FALSE;

			case ID_ADD_ADDDIRECTORY:
			case IDK_ADDDIRECTORY:
				LV_AddDirectory();
				return FALSE;

			case ID_PLAYLIST_PLAY:
			case IDK_PLAY:
				winamp_enqueue_to_playlist(TRUE);
				return FALSE;

			case ID_PLAYLIST_PLAYALL:
				winamp_enqueue_all_to_playlist(TRUE);
				return FALSE;

			case ID_PLAYLIST_ENQUEUE:
				winamp_enqueue_to_playlist(FALSE);
				return FALSE;

			case ID_PLAYLIST_ENQUEUEALL:
				winamp_enqueue_all_to_playlist(FALSE);
				return FALSE;

			case ID_PLAYLIST_SHOWHIDEPLAYLIST:
				SendMessage(plugin.hwndParent, WM_COMMAND, (WPARAM)40040,0);
				return FALSE;

			case ID_PLAYLIST_CLEARCURRENTPLAYLIST:
				SendMessage(plugin.hwndParent, WM_USER, (WPARAM)NULL, 101);
				return FALSE;

			case ID_PLAYLIST_SELECTALL:
			case IDK_SELECTALL:
				LV_SelectAll(TRUE);
				ST_UpdateLengthSelected();
				return FALSE;

			case ID_PLAYLIST_SELECTNONE:
				LV_SelectAll(FALSE);
				ST_UpdateLengthSelected();
				return FALSE;

			case ID_PLAYLIST_SELECTTOGGLE:
			case IDK_SELECTTOGGLE:
				LV_SelectToggle();
				return FALSE;

			case ID_PLAYLIST_SELECTWITHSAME_ARTIST: 
			case ID_PLAYLIST_SELECTWITHSAME_TITLE:  
			case ID_PLAYLIST_SELECTWITHSAME_GENRE:  
			case ID_PLAYLIST_SELECTWITHSAME_YEAR:   
			case ID_PLAYLIST_SELECTWITHSAME_ALBUM:  
			case ID_PLAYLIST_SELECTWITHSAME_COMMENT:
			case ID_PLAYLIST_SELECTWITHSAME_MODE:   
			case ID_PLAYLIST_SELECTWITHSAME_BITRATE:
			case ID_PLAYLIST_SELECTWITHSAME_FREQSAMPLING:
			case ID_PLAYLIST_SELECTWITHSAME_LAYER:
			case ID_PLAYLIST_SELECTWITHSAME_TIME:
			case ID_PLAYLIST_SELECTWITHSAME_FILESIZE:
			case ID_PLAYLIST_SELECTWITHSAME_FILENAME:
				{
				int nCol = LOWORD(wParam)-ID_PLAYLIST_SELECTWITHSAME_ARTIST;
				LV_SelectWithSame(nCol);
				}
				return FALSE;

			case ID_PLAYLIST_SELECTCROP:
			case IDK_CROP:
				LV_SelectCrop();
				return FALSE;

			case ID_PLAYLIST_DEL:
			case IDK_DEL:
				LV_SelectDel();
				return FALSE;

			case ID_PLAYLIST_REMOVEFILE:
				LV_SelectRemoveFile();
				return FALSE;


			case ID_PLAYLIST_SELECTALLDEADFILES:
				LV_DeadFiles();
				return FALSE;

			case ID_PLAYLIST_SHOWDUPLICATE:
				m_bShowDuplicate = !m_bShowDuplicate; 
				if (m_bShowDuplicate)
					LV_ShowDuplicate();
				else
					LV_UpdatePlaylist();
				return FALSE;

			default:
				return FALSE;
			}
			
		case WM_NOTIFY:
			NotifyHandler(uMsg, wParam, lParam, plugin.hDllInstance);
			break;

	}
	return FALSE;
}




/****************************************************************************
* 
*    FUNCTION: EZDlg_NotifyHandler(HWND, UINT, UINT, LONG)
*
*    PURPOSE: This function is the handler for the WM_NOTIFY that is 
*    sent to the parent of the list view window.
*
****************************************************************************/
LRESULT CDlgPlaylist::NotifyHandler(UINT uMsg, WPARAM wParam, LPARAM lParam,
	HINSTANCE hInst)
{
	LV_DISPINFO FAR * pnmv = (LV_DISPINFO FAR *) lParam;
	NM_LISTVIEW *pNm = (NM_LISTVIEW *)lParam;
	
	switch(((LPNMHDR) lParam)->code)
	{

		// listview
        case NM_DBLCLK: 
		case NM_RETURN: 
			switch (gParam.nDblClic)
			{
				case 0:	winamp_enqueue_to_playlist(TRUE); break;
				case 1:	winamp_enqueue_to_playlist(FALSE); break;
				case 2: SendMessage(m_hDlg,WM_COMMAND, (WPARAM)ID_ID3TAG_EDITID3TAG, NULL); break;
			}

			return(TRUE); 
		
		case LVN_ITEMCHANGED :
			{
				if (m_bRefreshSelLength == FALSE)
					return TRUE;
				
				
				NM_LISTVIEW *pnmv = (LPNMLISTVIEW) lParam; 
					
				if ((pnmv->uOldState & LVIS_SELECTED) && !(pnmv->uNewState & LVIS_SELECTED))
				{
					LVITEM lvi; 
					ZeroMemory(&lvi, sizeof(lvi)); 
					lvi.mask  = LVIF_PARAM; 
					lvi.iItem = pnmv->iItem;
					ListView_GetItem(m_hwndList, &lvi);
				
					CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

					m_nSelLength -= mp3i->GetPlayLength();
					
					TCHAR szTimeFrmt[100];
					TCHAR szTemp[100];
					Time2Str(m_nSelLength, &szTimeFrmt[0], TRUE);
					_stprintf(szTemp, TEXT_STATUS_SELLENGTH, szTimeFrmt);
					ST_ChangeText((LPTSTR)szTemp, 4);
					return TRUE;
				}
				
				if (!(pnmv->uOldState & LVIS_SELECTED) && (pnmv->uNewState & LVIS_SELECTED))
				{
					LVITEM lvi; 
					ZeroMemory(&lvi, sizeof(lvi)); 
					lvi.mask  = LVIF_PARAM; 
					lvi.iItem = pnmv->iItem;
					ListView_GetItem(m_hwndList, &lvi);
					
					CMP3Info *mp3i = (CMP3Info *)lvi.lParam;
					
					m_nSelLength += mp3i->GetPlayLength();
					
					TCHAR szTimeFrmt[100];
					TCHAR szTemp[100];
					Time2Str(m_nSelLength, &szTimeFrmt[0], TRUE);
					_stprintf(szTemp, TEXT_STATUS_SELLENGTH, szTimeFrmt);
					ST_ChangeText((LPTSTR)szTemp, 4);
					return TRUE;
				}

			}
			return TRUE;

		case LVN_GETDISPINFO:
			// Provide the item or subitem's text, if requested. 
		    if (pnmv->item.mask & LVIF_TEXT) 
			{ 
				TCHAR szCol[MAX_ITEM_SIZ];
				LV_GetInfoCol((CMP3Info*)pnmv->item.lParam, pnmv->item.iSubItem, szCol, pnmv->item.iItem + 1) ;
				_tcscpy(pnmv->item.pszText, szCol);
			}
			break;


		case LVN_COLUMNCLICK:
			LV_ColumnSort(pNm->iSubItem, FALSE);
			break;

		default:
			break;
	}
	return 0L;
}


/**************************************************************************** */
int CALLBACK CDlgPlaylist::ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, int nSubItem)
{   
	int iCmp; 
	tstring str1 = _T("");
	tstring str2 = _T("");
	

	CMP3Info *mi1 = (CMP3Info *)lParam1;
	CMP3Info *mi2 = (CMP3Info *)lParam2;
	

	// Compare the specified column. 
	switch(nSubItem)
	{
		case 0: // Artist
			GetFrameInfo(mi1, ID3TAG_TPE1, str1);
			GetFrameInfo(mi2, ID3TAG_TPE1, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
		case 1:
			// Title songname
			GetFrameInfo(mi1, ID3TAG_TIT2, str1);
			GetFrameInfo(mi2, ID3TAG_TIT2, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;

		case 2:  // Genre
			GetFrameInfo(mi1, ID3TAG_TCON, str1);
			GetFrameInfo(mi2, ID3TAG_TCON, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
		
		case 3: // Year
			GetFrameInfo(mi1, ID3TAG_TYER, str1);
			GetFrameInfo(mi2, ID3TAG_TYER, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
		
		case 4: // Album
			GetFrameInfo(mi1, ID3TAG_TALB, str1);
			GetFrameInfo(mi2, ID3TAG_TALB, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;

		case 5: // Comment
			GetFrameInfo(mi1, ID3TAG_COMM, str1);
			GetFrameInfo(mi2, ID3TAG_COMM, str2);
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
			
		case 6: // Mode
			str1 = ModeName[mi1->GetMode()];;
			str2 = ModeName[mi2->GetMode()];;
			iCmp = _tcsicmp(str1.c_str(), str2.c_str());
			break;
		
		case 7: // Bitrate
			if (mi1->GetBitrate() == mi2->GetBitrate())
				iCmp = 0;
			else if (mi1->GetBitrate() > mi2->GetBitrate())
				iCmp = 1;
			else
				iCmp = -1;
			break;
		
		case 8: // Frequency
			if (mi1->GetFrequency() == mi2->GetFrequency())
				iCmp = 0;
			else if (mi1->GetFrequency() > mi2->GetFrequency())
				iCmp = 1;
			else
				iCmp = -1;
		
		case 9: // Layer
			{
				if (mi1->GetLayer() == mi2->GetLayer())
					iCmp = 0;
				else if (mi1->GetLayer() > mi2->GetLayer())
					iCmp = 1;
				else
					iCmp = -1;
				
				break;
			}
			
		case 10:  // Length Time
			if (mi1->GetPlayLength() == mi2->GetPlayLength())
					iCmp = 0;
				else if (mi1->GetPlayLength() > mi2->GetPlayLength())
					iCmp = 1;
				else
					iCmp = -1;
			break;

		case 11: // File size
			if (mi1->GetMP3Size() == mi2->GetMP3Size())
					iCmp = 0;
				else if (mi1->GetMP3Size() > mi2->GetMP3Size())
					iCmp = 1;
				else
					iCmp = -1;
			break;

		case 12:  // File name
			iCmp = _tcsicmp(mi1->GetCFilename(), mi2->GetCFilename());
			break;
		
		case 13: // Track #
			{
				GetFrameInfo(mi1, ID3TAG_TRCK, str1);
				GetFrameInfo(mi2, ID3TAG_TRCK, str2);
				int track1 = _ttoi(str1.c_str());
				int track2 = _ttoi(str2.c_str());

				if (track1 == track2)
					iCmp = 0;
				else if (track1 > track2)
					iCmp = 1;
				else
					iCmp = -1;
			}
				
			break;

		case 14: // ID3 Tag version
			if (mi1->GetTagVersion() == mi2->GetTagVersion())
					iCmp = 0;
				else if (mi1->GetTagVersion() > mi2->GetTagVersion())
					iCmp = 1;
				else
					iCmp = -1;
			break;


		case 15: // B.P.M.
			{
				GetFrameInfo(mi1, ID3TAG_TBPM, str1);
				GetFrameInfo(mi2, ID3TAG_TBPM, str2);
				int track1 = _ttoi(str1.c_str());
				int track2 = _ttoi(str2.c_str());

				if (track1 == track2)
					iCmp = 0;
				else if (track1 > track2)
					iCmp = 1;
				else
					iCmp = -1;
			}
				
			break;
	}

	if (reverse_sort == TRUE)
		iCmp *= -1;
	
	return (iCmp); 

	
}

/* ************************************************************ */
void CDlgPlaylist::LV_GetInfoCol(CMP3Info *mp3i, int col, TCHAR *szCol, int iItem) 
{
	
	tstring str = _T(" ");
	
	if (!mp3i) 
	{
		_tcscpy(szCol, _T(" "));
		return;
	}


	switch(m_realColumn[col])
	{
	case 0:  // Artist
		GetFrameInfo(mp3i, ID3TAG_TPE1, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 1: // Title
		GetFrameInfo(mp3i, ID3TAG_TIT2, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 2: // Genre
		GetFrameInfo(mp3i, ID3TAG_TCON, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 3: // Year
		GetFrameInfo(mp3i, ID3TAG_TYER, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 4: // Album
		GetFrameInfo(mp3i, ID3TAG_TALB, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 5: // Comment
		GetFrameInfo(mp3i, ID3TAG_COMM, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	case 6: // Mode Stereo...
		str = ModeName[mp3i->GetMode()];
		_tcscpy(szCol, (LPTSTR)str.c_str());
		break;
	case 7: // Bitrate
		if (mp3i->isVBR())
			_stprintf(szCol, _T("%dkb/s (VBR)"), mp3i->GetBitrate());
		else
			_stprintf(szCol, _T("%dkb/s"), mp3i->GetBitrate());
		break;
	case 8: // Frequency
		_stprintf(szCol, _T("%dHz"), mp3i->GetFrequency());
		break;
	case 9: // Version & Mode
		{
			TCHAR mpeglayer[100];
			mp3i->GetMPEGVersionLayer(mpeglayer);
			_tcscpy(szCol, (LPTSTR)mpeglayer);
		}
		break;
	case 10: // Length Time
		Time2Str(mp3i->GetPlayLength(), szCol);
		break;
	case 11: // File size
		_stprintf(szCol, _T("%u "), mp3i->GetMP3Size());
		break;
	case 12: // File name
		str = mp3i->GetCFilename();
		_tcscpy(szCol, (LPTSTR)str.c_str());
		break;
	case 13: // Track #
		GetFrameInfo(mp3i, ID3TAG_TRCK, str);
		_tcscpy(szCol, (LPTSTR)str.c_str());
		break;
	case 14: // ID3Tag
		str = ID3TagName[mp3i->GetTagVersion()];
		_tcscpy(szCol, (LPTSTR)str.c_str());
		break;
	case 15: // B.P.M.
		GetFrameInfo(mp3i, ID3TAG_TBPM, str);
		_tcsncpy(szCol, (LPTSTR)str.c_str(), MAX_ITEM_SIZ);
		szCol[MAX_ITEM_SIZ] = 0; 
		break;
	default:
		_tcscpy(szCol, TEXT_NOTHING);
		break;
	}

	// Show number for 1st column
	if (iItem != -1)
	{
		if ((gParam.bShowNumber == 1) && (col == 0))
		{	TCHAR szTemp[MAX_ITEM_SIZ];
			_stprintf(szTemp, _T("%d. %s"), iItem, szCol);       
			_tcscpy(szCol, szTemp);
		}
	}

}


/* ****************************************************************** */
void CDlgPlaylist::GetFrameInfo(CMP3Info *mp3i, int fid, tstring &str)
{
//	nShowID3v1 == 0  ID3V1
//	nShowID3v1 == 1  ID3V2
//	nShowID3v1 == 2  ID3V2 if null --> ID3V1

	// ///////////////////////////////
	// Retreive information from ID3v2
	// ///////////////////////////////
	if (gParam.nShowID3v1 == 1 || gParam.nShowID3v1 == 2)
	{
		// only supported frames allowed
		if (!ID3Frames[fid].Supported) 
			return; 
		if (ID3Frames[fid].Type != ID3ENC_STRING)	
			return;
		
		CMP3Frame *mp3f = mp3i->GetFrame(fid);
			
		if (mp3f)
			str = ((CMP3StringFrame *)mp3f)->GetString();	// handle only the first String


		
		if (gParam.nShowID3v1 == 2)
		{
			if (str.compare(_T("")) == 0 || str.compare(_T(" ")) == 0)
			{
				str.erase();
				goto showID3v1;
			}

		}
		return;
	}
	
showID3v1:
	
	// ///////////////////////////////
	// Retreive information from ID3v1
	// ///////////////////////////////
	{
		struct ID3V1MP3Info *id3i = mp3i->GetID3v1();

		if (!id3i)
			return;

		switch(fid)
		{
		
		case ID3TAG_COMM: // read the Comment
			if (!IsStringEmpty((LPSTR)id3i->Comment)) 
				a2tcpy(str, (LPSTR)id3i->Comment);
			break;
	
		case ID3TAG_TYER: // read the Year
			if (!IsStringEmpty((LPSTR)id3i->Year)) 
				a2tcpy(str, (LPSTR)id3i->Year);
			break;

		case ID3TAG_TALB: // read the Album
			if (!IsStringEmpty((LPSTR)id3i->Album)) 
				a2tcpy(str, (LPSTR)id3i->Album);
			break;
	
		case ID3TAG_TPE1: // read the Artist
			if (!IsStringEmpty((LPSTR)id3i->Artist)) 
				a2tcpy(str, (LPSTR)id3i->Artist);
			break;
	
		case ID3TAG_TIT2:  // read the Title:Song name
			if (!IsStringEmpty((LPSTR)id3i->SongName)) 
				a2tcpy(str, (LPSTR)id3i->SongName);
			break;
	
		case ID3TAG_TCON: // read the genre
			a2tcpy(str, (LPSTR)GenreInt2Str(id3i->Genre));
			break;

		case ID3TAG_TRCK: // read the Track
			if (id3i->Track != 0) 
			{
				TCHAR szTrack[4];
				_stprintf(szTrack, _T("%d"), id3i->Track);
				a2tcpy(str, (LPSTR)szTrack);
			}

		}
	
	}
}

/* ****************************************************************** */
void CDlgPlaylist::Resize(LPARAM lParam)
{
	RECT rc;
	WORD wWidth  = LOWORD(lParam);  
	WORD wHeight = HIWORD(lParam); 
	int hStatus;
	int hEdit;
	HWND hwndEdit;
	POINT pt;
	
	hwndEdit = GetDlgItem(m_hDlg, IDC_EDIT_SEARCH);
	GetClientRect(hwndEdit, &rc);
	hEdit  = (rc.bottom - rc.top) + 20;

	GetClientRect(m_hwndStat, &rc);
	hStatus = rc.bottom - rc.top;
	
	MoveWindow(GetDlgItem(m_hDlg, IDC_LVW_PLAYLIST), 0, hEdit, wWidth , wHeight - hEdit - hStatus, TRUE); 
	MoveWindow(m_hwndStat, 0, wHeight - hStatus, wWidth, hStatus, TRUE);

	hwndEdit = GetDlgItem(m_hDlg, IDC_SLIDER_VOLUME);
	GetWindowRect(hwndEdit, &rc);
	
	pt.x = rc.left;
	pt.y = rc.top;
	ScreenToClient(m_hDlg, &pt);
	wWidth = wWidth -  pt.x  - 4;

	if (wWidth > 0)
		MoveWindow(hwndEdit, pt.x, pt.y, wWidth, rc.bottom - rc.top, TRUE);
}

/* ************************************************************************ */
void CDlgPlaylist::Init()
{
	LPARAM lParam;
	RECT rect;	

	HCURSOR hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));

	
	// Initialize Volume 
	VolumeInit(m_hDlg, GetDlgItem(m_hDlg, IDC_SLIDER_VOLUME));

	SendDlgItemMessage(m_hDlg, IDC_EDIT_SEARCH, EM_SETLIMITTEXT , 30L, 0L);

	ST_CreateStatus(0);
	
	// Initialize listview
	Header_Initialize(m_hwndList);
	LV_InitIcon();
	LV_ColumnResetDisplay(TRUE);
	GetColumnOrder(m_realColumn);


	if (gParam.nOnStartup == 0)
		LV_Playlist(COMDLG_OPEN);
	else
		LV_PlaylistWinamp(COMDLG_OPEN);

	
	GetClientRect(m_hDlg, &rect);
	lParam = rect.right - rect.left;  
	lParam |= (rect.bottom - rect.top) << 16; 
	SendMessage(m_hDlg, WM_SIZE , (WPARAM)SIZE_RESTORED, (LPARAM)lParam);


	// Selectionne le 1er de la liste
	ListView_SetItemState(m_hwndList, 0, LVIS_SELECTED , LVIS_SELECTED);

	SetCursor(hcurOld);
}

/* ************************************************************ */
void CDlgPlaylist::OwnerDrawBtn (LPDRAWITEMSTRUCT lpdis)
{                                                    
    HDC hDC;
    RECT rect;
	HFONT hFontButton, hOldFont;
	LOGFONT lf;	
	HRGN hRgn;
	POINT pt[4];

    hDC = lpdis->hDC;
    CopyRect (&rect, &(lpdis->rcItem));    

	if (lpdis->CtlID == IDC_BUTTON_OPTION)
	{
		UINT uState = DFCS_SCROLLRIGHT;
		
		switch (lpdis->itemAction) 
		{	
			case ODA_DRAWENTIRE:  
			case ODA_FOCUS:
				break;
			case ODA_SELECT:
				if (lpdis->itemState & ODS_SELECTED)
					uState = DFCS_SCROLLRIGHT | DFCS_PUSHED;
				break;
			default:
				return;
		}

		DrawFrameControl(hDC, &rect, DFC_SCROLL, uState);
		return;
	}
	

	if (lpdis->CtlID == IDC_BUTTON_FILE || lpdis->CtlID == IDC_BUTTON_ADD)
	{
		switch (lpdis->itemAction) 
		{	
			case ODA_DRAWENTIRE:  
			case ODA_FOCUS:
				DrawFrameControl(hDC, &rect, DFC_BUTTON, DFCS_BUTTONPUSH);
				InflateRect(&rect, -2, -2);
				rect.left += 8;
				rect.top  += 1;
				break;

			case ODA_SELECT:
				if (lpdis->itemState & ODS_SELECTED)
				{
					DrawFrameControl(hDC, &rect, DFC_BUTTON, DFCS_BUTTONPUSH|DFCS_PUSHED);
					InflateRect(&rect, -2, -2);
					rect.left += 9;
					rect.top  += 2;
				}
				else
				{
					DrawFrameControl(hDC, &rect, DFC_BUTTON, DFCS_BUTTONPUSH);
					InflateRect(&rect, -2, -2);
					rect.left += 8;
					rect.top  += 1;
				}
				
				break;

			default:
				return;

		}

		ZeroMemory(&lf,sizeof(lf));
		_tcscpy (lf.lfFaceName, _T("MS Sans Serif"));
		lf.lfHeight = 5;
		lf.lfWidth  = 5;
		lf.lfUnderline = FALSE;
		lf.lfOutPrecision = OUT_TT_PRECIS;
		hFontButton = CreateFontIndirect ((LPLOGFONT) &lf);
		if (hFontButton!=NULL) 
		{
			TCHAR szTitle[20];

			if (lpdis->CtlID == IDC_BUTTON_FILE)
				_tcscpy(szTitle, TEXT_BTN_FILE);
			else
				_tcscpy(szTitle, TEXT_BTN_ADD);

			hOldFont = (HFONT)SelectObject(hDC, hFontButton);
			SetBkMode(hDC,TRANSPARENT);
			SetTextAlign(hDC,TA_LEFT);
			TextOut(hDC,rect.left,rect.top ,szTitle, _tcslen(szTitle));

			SelectObject(hDC, hOldFont);
			DeleteObject (hFontButton); 
		}

		pt[0].x  = (lpdis->rcItem).right - 16;
		pt[0].y  = (lpdis->rcItem).top + 8;
		pt[1].x = pt[0].x + 7;
		pt[1].y = pt[0].y;
		pt[2].x = pt[0].x + 3;
		pt[2].y = pt[0].y + 4;
		pt[3].x = pt[0].x + 3;
		pt[3].y = pt[0].y + 3;
		hRgn = CreatePolygonRgn((CONST POINT*) &pt,4,WINDING);
		FillRgn(hDC, hRgn, (HBRUSH)GetStockObject(BLACK_BRUSH));
		DeleteObject(hRgn);
	}	

}                                                    



/* ************************************************************************ */
void CDlgPlaylist::LoadParam()
{
	TCHAR ini_file[MAX_PATH], *p;
	TCHAR szBuffer[80];
	int i,j, nfilled=0;
	

	GetModuleFileName(plugin.hDllInstance,ini_file,sizeof(ini_file));
	p = ini_file + _tcslen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	_tcscat(ini_file, _T("plugin.ini"));

	// //////////////////////////////////
	// Columns Playlist
	// //////////////////////////////////

	GetPrivateProfileString(szAppName, 
		_T("Columns_Order"), _T("?"), szBuffer,sizeof(szBuffer),ini_file);
	
	nfilled = _stscanf(szBuffer, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d"),
		&gParam.column_order[0],  &gParam.column_order[1],   &gParam.column_order[2], 
		&gParam.column_order[3],  &gParam.column_order[4],   &gParam.column_order[5], 
		&gParam.column_order[6],  &gParam.column_order[7],   &gParam.column_order[8],
		&gParam.column_order[9],  &gParam.column_order[10],  &gParam.column_order[11], 
		&gParam.column_order[12], &gParam.column_order[13],  &gParam.column_order[14],
		&gParam.column_order[15]);

	// Si pb de recuperation initialisation de toutes les colonnes
	// dans l'ordre
	if (nfilled < COLUMN_MAX)
		for (i = 0; i < COLUMN_MAX; i++) 
			gParam.column_order[i] = gDefault_column[i].nOrder;

	GetPrivateProfileString(szAppName, 
		_T("Columns_Width"),	_T("?"), szBuffer,sizeof(szBuffer),ini_file);
	
	nfilled = _stscanf(szBuffer, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d"),
		&gParam.column_width[0],  &gParam.column_width[1],   &gParam.column_width[2], 
		&gParam.column_width[3],  &gParam.column_width[4],   &gParam.column_width[5], 
		&gParam.column_width[6],  &gParam.column_width[7],   &gParam.column_width[8],
		&gParam.column_width[9],  &gParam.column_width[10],  &gParam.column_width[11], 
		&gParam.column_width[12], &gParam.column_width[13],  &gParam.column_width[14],
		&gParam.column_width[15]);

	// Si pb de recuperation initialisation de toutes les colonnes
	// avec les largeurs par defaut
	if (nfilled < COLUMN_MAX)
		for (i = 0; i < COLUMN_MAX; i++) 
			gParam.column_width[i] = gDefault_column[i].nWidth;


	GetPrivateProfileString(szAppName, 
		_T("Columns_Shown"),	_T("?"), szBuffer,sizeof(szBuffer),ini_file);
	
	nfilled = _stscanf(szBuffer, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d"),
		&gParam.column_shown[0],  &gParam.column_shown[1],   &gParam.column_shown[2], 
		&gParam.column_shown[3],  &gParam.column_shown[4],   &gParam.column_shown[5], 
		&gParam.column_shown[6],  &gParam.column_shown[7],   &gParam.column_shown[8],
		&gParam.column_shown[9],  &gParam.column_shown[10],  &gParam.column_shown[11], 
		&gParam.column_shown[12], &gParam.column_shown[13],  &gParam.column_shown[14],
		&gParam.column_shown[15]);

	// Si pb de recuperation initialisation de toutes les colonnes
	// avec les largeurs par defaut
	if (nfilled < COLUMN_MAX)
		for (i = 0; i < COLUMN_MAX; i++) 
			gParam.column_shown[i] = gDefault_column[i].nShown;


	gParam.sort_column  = (int)GetPrivateProfileInt(szAppName,  _T("sort_column"),0, ini_file);
	gParam.sort_reverse = (int)GetPrivateProfileInt(szAppName,  _T("sort_reverse"),0, ini_file);

	gParam.NbCol = 0;
	for (i = 0; i < COLUMN_MAX; i++)
	{
		if (gParam.column_shown[i] == 1) 
				gParam.NbCol++;
	}

	// //////////////////////////////////////////////////
	// Load in array Format Tag rename filename from TAG
	// //////////////////////////////////////////////////
	for (i = 0; i<MAX_COL_FRMT; i++) 
		gParam.FrmtCol[i] = -1;

	GetPrivateProfileString(szAppName, 
		_T("FrmtTag"), 
		_T("0,1,-1,-1,-1,-1") ,
		szBuffer,sizeof(szBuffer),ini_file);

	nfilled = _stscanf(szBuffer, _T("%d,%d,%d,%d,%d,%d"),
		&gParam.FrmtCol[0],	&gParam.FrmtCol[1], &gParam.FrmtCol[2], 
		&gParam.FrmtCol[3],	&gParam.FrmtCol[4], &gParam.FrmtCol[5]);
	
	// Si pb de recuperation initialisation de toutes les colonnes Frmt
	// dans l'ordre
	if (nfilled < MAX_COL_FRMT)
		for (i = 0; i <MAX_COL_FRMT; i++)	
			gParam.FrmtCol[i] = i;
	
	// //////////////////////////////////////////////////
	// Other parameters
	// //////////////////////////////////////////////////
	gParam.bExportAllItems	  = (BOOL)GetPrivateProfileInt(szAppName, _T("ExportAllItems"),0, ini_file);
	gParam.bExportHTML_Format =	(BOOL)GetPrivateProfileInt(szAppName, _T("ExportHTMLFormat"),0, ini_file);
	gParam.bExportHTML_Link   = (BOOL)GetPrivateProfileInt(szAppName, _T("ExportHTMLLink"),0, ini_file);
	gParam.bExportBothID3     = (BOOL)GetPrivateProfileInt(szAppName, _T("ExportBothID3"), 0, ini_file);
	gParam.bShowNumber        = (BOOL)GetPrivateProfileInt(szAppName, _T("ShowNumber"),0, ini_file);
	gParam.bShowGrid          = (BOOL)GetPrivateProfileInt(szAppName, _T("ShowGrid"),0, ini_file);
	gParam.nOptionSearch      = (int)GetPrivateProfileInt(szAppName,  _T("OptionSearch"),31, ini_file);
	gParam.nShowID3v1         = (int)GetPrivateProfileInt(szAppName,  _T("ShowID3v1"),0, ini_file);
	gParam.nDblClic           = (int)GetPrivateProfileInt(szAppName,  _T("DblClic"),0, ini_file);
	gParam.nOnStartup         = (BOOL)GetPrivateProfileInt(szAppName, _T("OnStartup"),0, ini_file);
	gParam.nMassCheckTAG      = (int)GetPrivateProfileInt(szAppName,  _T("MassTAG"),0, ini_file);
	gParam.nMassTagID3        = (int)GetPrivateProfileInt(szAppName,  _T("MassTagID3"), 3, ini_file);
	gParam.nWriteTagfromFile  = (int)GetPrivateProfileInt(szAppName,  _T("WriteTagfromFile"), 3, ini_file);
	gParam.nRenameFileFromTagWithID3v1 = (int)GetPrivateProfileInt(szAppName,  _T("RenameFileFromTagWithID3v1"), 1, ini_file);
	gParam.nMatchTime         = (int)GetPrivateProfileInt(szAppName,  _T("MatchTime"), 0, ini_file);
	gParam.nOptionCapitalize  = (int)GetPrivateProfileInt(szAppName,  _T("OptionCapitalize"), 0, ini_file);
	gParam.nOptionConvert     = (int)GetPrivateProfileInt(szAppName,  _T("OptionConvert"), 0, ini_file);
	gParam.nOrganizeFromID3   = (int)GetPrivateProfileInt(szAppName,  _T("OrganizeFromID3"), 0, ini_file);
	gParam.nOrganizeField     = (int)GetPrivateProfileInt(szAppName,  _T("OrganizeField"), 0, ini_file);



	{
		TCHAR szSep[6];
		int nLen;
		GetPrivateProfileString(szAppName, _T("Separator"),  _T("# - #") ,szSep,sizeof(szSep),ini_file);
		
		nLen = _tcslen(szSep);

		if (nLen < 2)
			_tcscpy(gParam.szSep, _T(" - "));
		else
		{
			TCHAR *p = &szSep[1];
			_tcsncpy(gParam.szSep, p, nLen -2);
		}

	}

	// //////////////////////////////////////////////////
	// WEB Template / HTML
	// //////////////////////////////////////////////////
	for (i=0; i<MAX_TEMPLATE; i++)
	{
		
		TCHAR szTempWeb[255];
		TCHAR szDefaultValue[80];
		COLORREF crColor;
		int r,g,b;

		// Recupere la palette de color pour le style CSS
		for (j=0;j<MAX_CSS;j++)
		{
			// Text color
			 crColor = gaTemplateDefault[i].css[j].cColor;
			_stprintf(szDefaultValue,"%d %d %d", GetRValue(crColor), GetGValue(crColor), GetBValue(crColor));
			_stprintf(szTempWeb, _T("HTML%d_CSS%d_Color"), i, j);
			GetPrivateProfileString(szAppName, szTempWeb, szDefaultValue,
				szBuffer,sizeof(szBuffer),ini_file);
			_stscanf(szBuffer, _T("%d %d %d"), &r, &g, &b);
			gParam.Template[i].css[j].cColor = RGB(r,g,b);

			// Background color
			crColor = gaTemplateDefault[i].css[j].cColorBack;
			_stprintf(szDefaultValue,"%d %d %d", GetRValue(crColor), GetGValue(crColor), GetBValue(crColor));
			_stprintf(szTempWeb, _T("HTML%d_CSS%d_ColorBack"), i, j);
			GetPrivateProfileString(szAppName, szTempWeb, szDefaultValue,
				szBuffer,sizeof(szBuffer),ini_file);
			_stscanf(szBuffer, _T("%d %d %d"), &r, &g, &b);
			gParam.Template[i].css[j].cColorBack = RGB(r,g,b);

			// Recupere les proprietes Fonts
			_stprintf(szDefaultValue, _T("%d,%d,%d"), 
				gaTemplateDefault[i].css[j].nFontName, 
				gaTemplateDefault[i].css[j].nFontStyle,
				gaTemplateDefault[i].css[j].nFontSize);

			_stprintf(szTempWeb, _T("HTML%d_CSS%d_Font"), i, j);

			GetPrivateProfileString(szAppName, szTempWeb, szDefaultValue,
				szBuffer,sizeof(szBuffer),ini_file);

			_stscanf(szBuffer, _T("%d,%d,%d"), 
				&gParam.Template[i].css[j].nFontName, 
				&gParam.Template[i].css[j].nFontStyle,
				&gParam.Template[i].css[j].nFontSize);
		}

		// Hyperlink page color
		crColor = gaTemplateDefault[i].cLinkColor;
		_stprintf(szDefaultValue,"%d %d %d", GetRValue(crColor), GetGValue(crColor), GetBValue(crColor));
		_stprintf(szTempWeb, _T("HTML%d_LinkColor"), i);
		GetPrivateProfileString(szAppName, szTempWeb, szDefaultValue,
				szBuffer,sizeof(szBuffer),ini_file);
		_stscanf(szBuffer, _T("%d %d %d"), &r, &g, &b);
		gParam.Template[i].cLinkColor = RGB(r,g,b);

	}


	// //////////////////////////////////////////////////
	// Window Position / dimension
	// //////////////////////////////////////////////////
	gParam.x      = (int)GetPrivateProfileInt(szAppName, _T("x"),0, ini_file);
	gParam.y      = (int)GetPrivateProfileInt(szAppName, _T("y"),0, ini_file);
	gParam.width  = (int)GetPrivateProfileInt(szAppName, _T("width"),0, ini_file);
	gParam.height = (int)GetPrivateProfileInt(szAppName, _T("height"),0, ini_file);

	if (gParam.width * gParam.height != 0)
		SetWindowPos(m_hDlg, 0, gParam.x, gParam.y, gParam.width, gParam.height, SWP_NOZORDER);

}

/* ************************************************************************ */
void CDlgPlaylist::SaveParam()
{
	TCHAR ini_file[MAX_PATH], *p;
	TCHAR szBuffer[80];
	int i,j, nfilled=0;
	GetModuleFileName(plugin.hDllInstance,ini_file,sizeof(ini_file));
	p = ini_file + _tcslen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	_tcscat(ini_file, _T("plugin.ini"));

// Column settings
	_stprintf(szBuffer, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d"),
		gParam.column_order[0],	gParam.column_order[1], gParam.column_order[2], 
		gParam.column_order[3],	gParam.column_order[4],	gParam.column_order[5], 
		gParam.column_order[6], gParam.column_order[7], gParam.column_order[8],
		gParam.column_order[9], gParam.column_order[10],gParam.column_order[11], 
		gParam.column_order[12],gParam.column_order[13],gParam.column_order[14],
		gParam.column_order[15]);
	WritePrivateProfileString(szAppName, _T("Columns_Order"), szBuffer,ini_file);

	_stprintf(szBuffer, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d"),
		gParam.column_width[0], gParam.column_width[1],  gParam.column_width[2], 
		gParam.column_width[3], gParam.column_width[4],  gParam.column_width[5], 
		gParam.column_width[6], gParam.column_width[7],  gParam.column_width[8],
		gParam.column_width[9], gParam.column_width[10], gParam.column_width[11], 
		gParam.column_width[12],gParam.column_width[13], gParam.column_width[14],
		gParam.column_width[15]);
	WritePrivateProfileString(szAppName, _T("Columns_Width"), szBuffer,ini_file);

	_stprintf(szBuffer, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d"),
		gParam.column_shown[0], gParam.column_shown[1],  gParam.column_shown[2], 
		gParam.column_shown[3], gParam.column_shown[4],  gParam.column_shown[5], 
		gParam.column_shown[6], gParam.column_shown[7],  gParam.column_shown[8],
		gParam.column_shown[9], gParam.column_shown[10], gParam.column_shown[11], 
		gParam.column_shown[12],gParam.column_shown[13], gParam.column_shown[14],
		gParam.column_shown[15]);
	WritePrivateProfileString(szAppName, _T("Columns_Shown"), szBuffer,ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.sort_column);
	WritePrivateProfileString(szAppName,_T("sort_column"),
		szBuffer, (LPTSTR)ini_file);
	_stprintf(szBuffer,_T("%d"),gParam.sort_reverse);
	WritePrivateProfileString(szAppName,_T("sort_reverse"),
		szBuffer, (LPTSTR)ini_file);


	_stprintf(szBuffer, _T("%d,%d,%d,%d,%d,%d"),
		gParam.FrmtCol[0],	gParam.FrmtCol[1], 	gParam.FrmtCol[2], 
		gParam.FrmtCol[3],	gParam.FrmtCol[4],  gParam.FrmtCol[5]);
	WritePrivateProfileString(szAppName, _T("FrmtTAG"), szBuffer,ini_file);

	WritePrivateProfileString(szAppName, _T("ExportAllItems"),
		gParam.bExportAllItems ? _T("1"): _T("0"), (LPTSTR)ini_file);

	WritePrivateProfileString(szAppName, _T("ExportHTMLFormat"),
		gParam.bExportHTML_Format ? _T("1"):_T("0"), (LPTSTR)ini_file);

	WritePrivateProfileString(szAppName, _T("ExportHTMLLink"),
		gParam.bExportHTML_Link ? _T("1"):_T("0"), (LPTSTR)ini_file);

	WritePrivateProfileString(szAppName, _T("ExportBothID3"),
		gParam.bExportBothID3 ? _T("1"):_T("0"), (LPTSTR)ini_file);

	WritePrivateProfileString(szAppName, _T("ShowNumber"),
		gParam.bShowNumber ? _T("1"): _T("0"), (LPTSTR)ini_file);

	WritePrivateProfileString(szAppName, _T("ShowGrid"),
		gParam.bShowGrid ? _T("1"): _T("0"), (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nOptionSearch);
	WritePrivateProfileString(szAppName,_T("OptionSearch"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nShowID3v1);
	WritePrivateProfileString(szAppName,_T("ShowID3v1"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer, _T("%d"),gParam.nDblClic);
	WritePrivateProfileString(szAppName,_T("DblClic"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nOnStartup);
	WritePrivateProfileString(szAppName,_T("OnStartup"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nMassCheckTAG);
	WritePrivateProfileString(szAppName,_T("MassTAG"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nMassTagID3);
	WritePrivateProfileString(szAppName,_T("MassTagID3"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nWriteTagfromFile);
	WritePrivateProfileString(szAppName,_T("WriteTagfromFile"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nRenameFileFromTagWithID3v1);
	WritePrivateProfileString(szAppName,_T("RenameFileFromTagWithID3v1"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nMatchTime);
	WritePrivateProfileString(szAppName,_T("MatchTime"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nOptionCapitalize);
	WritePrivateProfileString(szAppName,_T("OptionCapitalize"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nOptionConvert);
	WritePrivateProfileString(szAppName,_T("OptionConvert"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nOrganizeFromID3);
	WritePrivateProfileString(szAppName,_T("OrganizeFromID3"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("%d"),gParam.nOrganizeField);
	WritePrivateProfileString(szAppName,_T("OrganizeField"),
		szBuffer, (LPTSTR)ini_file);

	_stprintf(szBuffer,_T("#%s#"),gParam.szSep);
	WritePrivateProfileString(szAppName, _T("Separator"), szBuffer, ini_file);

	// //////////////////////////////////////////////////
	// WEB Style / HTML
	// //////////////////////////////////////////////////
	for (i=0; i<MAX_TEMPLATE; i++)
	{
		TCHAR szTempWeb[255];
		COLORREF crColor;

		// Sauvearge la palette de color pour le style(i)
		for (j=0;j<MAX_CSS;j++)
		{
			// Text color
			crColor = gParam.Template[i].css[j].cColor;
			_stprintf(szBuffer,"%d %d %d", GetRValue(crColor), GetGValue(crColor), GetBValue(crColor));
			_stprintf(szTempWeb, _T("HTML%d_CSS%d_Color"), i, j);
			WritePrivateProfileString(szAppName, szTempWeb, szBuffer, (LPTSTR)ini_file);
		
			// background color
			crColor = gParam.Template[i].css[j].cColorBack;
			_stprintf(szBuffer,"%d %d %d", GetRValue(crColor), GetGValue(crColor), GetBValue(crColor));
			_stprintf(szTempWeb, _T("HTML%d_CSS%d_ColorBack"), i, j);
			WritePrivateProfileString(szAppName, szTempWeb, szBuffer, (LPTSTR)ini_file);

			// Recupere les proprietes Fonts
			_stprintf(szBuffer,"%d,%d,%d", 
				gParam.Template[i].css[j].nFontName, 
				gParam.Template[i].css[j].nFontStyle,
				gParam.Template[i].css[j].nFontSize);
			_stprintf(szTempWeb, _T("HTML%d_CSS%d_Font"), i, j);
			WritePrivateProfileString(szAppName, szTempWeb, szBuffer, (LPTSTR)ini_file);
		}

		// Hyperlink page color
		crColor = gParam.Template[i].cLinkColor;
		_stprintf(szBuffer,"%d %d %d", GetRValue(crColor), GetGValue(crColor), GetBValue(crColor));
		_stprintf(szTempWeb, _T("HTML%d_LinkColor"), i);
		WritePrivateProfileString(szAppName, szTempWeb, szBuffer, (LPTSTR)ini_file);

	}

	// Window Position 
	RECT rect;
	GetWindowRect(m_hDlg, &rect);
	gParam.x = rect.left;
	gParam.y = rect.top;
	gParam.width  = rect.right - rect.left;
	gParam.height = rect.bottom - rect.top;
	
	int offX = gParam.x + gParam.width;
    int offY = gParam.y + gParam.height;
	int maxX =  GetSystemMetrics(SM_CXSCREEN);     // Screen Width
    int maxY =  GetSystemMetrics(SM_CYSCREEN);     // Screen Height

    if (offX > maxX)
    {   offX = maxX;
        gParam.x = (offX - gParam.width > 0) ? (offX - gParam.width) : 0;
    }
    if (offY > maxY)
    {   offY = maxY;
        gParam.y = (offY - gParam.height > 0) ? (offY - gParam.height) : 0;
    }
	_stprintf(szBuffer,_T("%d"),gParam.x);
	WritePrivateProfileString(szAppName,_T("x"),szBuffer, (LPTSTR)ini_file);
	_stprintf(szBuffer,_T("%d"),gParam.y);
	WritePrivateProfileString(szAppName,_T("y"),szBuffer, (LPTSTR)ini_file);
	_stprintf(szBuffer,_T("%d"),gParam.width);
	WritePrivateProfileString(szAppName,_T("width"),szBuffer, (LPTSTR)ini_file);
	_stprintf(szBuffer,_T("%d"),gParam.height);
	WritePrivateProfileString(szAppName,_T("height"),szBuffer, (LPTSTR)ini_file);

}


/* ************************************************************************ */
void CDlgPlaylist::LV_InitIcon()
{
	HIMAGELIST hSmall;
	HICON hIcon[4];
	int i;

	// initialise le ListView / Colonnes
	ListView_SetExtendedListViewStyle(m_hwndList, LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

	// Create Imagelist / small Icon
	// Associate the image lists with the list view control.
	hSmall = ImageList_Create (16, 16,	 ILC_COLOR16 | ILC_MASK, 3, 0);

	for (i=0; i<NUMELEMENTS(hIcon);i++)
	{	
		hIcon[i] = LoadIcon (plugin.hDllInstance, MAKEINTRESOURCE (IDI_ICON_ID3_0+i));
		ImageList_AddIcon (hSmall, hIcon[i]);
	}
	
	ListView_SetImageList (m_hwndList, hSmall, LVSIL_SMALL);

}


/* ************************************************************************ */
void CDlgPlaylist::LV_ColumnsGrid()
{
	DWORD dwExStyle = ListView_GetExtendedListViewStyle(m_hwndList);

	if (gParam.bShowGrid)
		dwExStyle |= LVS_EX_GRIDLINES;
	else
		dwExStyle &= ~LVS_EX_GRIDLINES;

	ListView_SetExtendedListViewStyle(m_hwndList, dwExStyle);
}

/* ************************************************************************ */
void CDlgPlaylist::LV_New()
{
	LV_Redraw(FALSE);

	ListView_DeleteAllItems(m_hwndList); 
	m_minfo.ResetList ();

	m_nTotalLength = 0;
	m_nAvgLength   = 0;
	m_nSelLength   = 0;
	UpdateStatus();
	LV_StatGenre();
	
	LV_Redraw(TRUE);
}

/* ************************************************************************ */
void CDlgPlaylist::LV_ComDlg(int nMode)
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256];
	TCHAR szFile[256];
	TCHAR szFileTitle[256]; 
	TCHAR szFilter[] = _T("EZ-Playlist (*.ez3)|*.ez3|");
    
    SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), szDirName);
	*szFile = '\0'; 

	memset(&ofn, 0, sizeof(OPENFILENAME)); 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 

	if (nMode == COMDLG_OPEN)
	{
		ofn.lpstrTitle = (LPCTSTR)_T("Open EZ-Playlist"); 
		if (GetOpenFileName(&ofn) == 0) 
			return;
		LV_LoadEZ3(szFile);
	}
	else
	{
		ofn.lpstrTitle = (LPCTSTR)_T("Save EZ-Playlist"); 
		if (GetSaveFileName(&ofn)) 
		{
			AddFileExtension(szFile, _T(".ez3")); 
			LV_WriteEZ3(szFile);
		}
		
	}

}

/* ************************************************************************ */
void CDlgPlaylist::LV_Playlist(int nOpen)
{
	TCHAR filename[255];
	TCHAR  *p;
	
	GetModuleFileName(plugin.hDllInstance,filename,sizeof(filename));
	p = filename + _tcslen(filename);
	while (p >= filename && *p != '.') p--;
	p++;
	*p++ = 'E';
	*p++ = 'Z';
	*p++ = '3';
	*p = '\0';
	
	
	if (nOpen == COMDLG_OPEN)
		LV_LoadEZ3(filename);
	else
		LV_WriteEZ3(filename);
}

/* ************************************************************************ */
void CDlgPlaylist::LV_PlaylistWinamp(int nOpen)
{
	TCHAR filename[255];
	HMODULE hModuleWinamp;
	TCHAR *p;
	
	hModuleWinamp = (HMODULE)GetClassLong(plugin.hwndParent, GCL_HMODULE);
	GetModuleFileName(hModuleWinamp,filename,sizeof(filename));
	p = filename + _tcslen(filename);
	while (p >= filename && *p != '.') p--;
	p++;
	*p++ = 'm';
	*p++ = '3';
	*p++ = 'u';
	*p = '\0';
	
	if (nOpen == COMDLG_OPEN)
	{
		// Writes out the current playlist to Winampdir \winamp.m3u
		SendMessage(plugin.hwndParent, WM_USER, 0,120);
		LV_LoadM3U(filename);
	}
	else
	{
		COPYDATASTRUCT cds;
		
		if (LV_WriteM3U(filename) == FALSE)
			return;

		// Efface l'ancienne playlist de WinAMP et ajoute la nvelle
		SendMessage(plugin.hwndParent, WM_USER, 0,101);

		cds.dwData = 100;
		cds.lpData = (void*)filename;
		cds.cbData = _tcslen((TCHAR*)cds.lpData)+1;
		SendMessage(plugin.hwndParent, WM_COPYDATA, (WPARAM)NULL,(LPARAM)&cds);
	}

}


/* ************************************************************************ */
void CDlgPlaylist::LV_LoadEZ3(TCHAR *filename)
{
	FILE *file;	
	EZ3HEADER header;
	LV_ITEM lvi;

	if ( (file = _tfopen(filename, _T("rb"))) == NULL)
		return;

	if (fread(&header, 1, sizeof(header), file) != sizeof(header))
	{	fclose(file);
		MessageBox(m_hDlg, TEXT_ERR_FILEPLAYLIST, TEXT_ERROR, MB_OK|MB_ICONSTOP);
		return;
	}

	if (_tcsnccmp(header.tag, _T("EZ3DATA1"), 8) != 0)
	{	fclose(file);
		MessageBox(m_hDlg, TEXT_ERR_BADFILE, TEXT_ERROR, MB_OK|MB_ICONSTOP);
		return;
	}


	//Turn the listview's redrawing off 
	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 
	m_minfo.ResetList();

	
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK; 

	m_nTotalLength = 0;
	m_nSelLength   = 0;

	
	// Scan EZPlaylist 
	for (int iItem = 0; iItem < header.nCountFile; iItem++)
	{
		EZ3HEADERMP3INFO headerfile;
		CMP3Info *mp3i = NULL;
		
		if (fread(&headerfile, 1, sizeof(headerfile), file) != sizeof(headerfile))
			goto err_loadEZ3;

		TCHAR *szFilename = (TCHAR*)malloc(headerfile.nLenFilename+1);

		if (szFilename == NULL)
			goto err_loadEZ3;

		*szFilename = '\0';
		fread(szFilename, sizeof(TCHAR), headerfile.nLenFilename, file);
		szFilename[headerfile.nLenFilename] = '\0';

		tstring strfile(szFilename);
		mp3i = new CMP3Info(strfile, FALSE);

		if (mp3i == NULL)
			goto err_loadEZ3;

		// Set CoreMP3, init MP3Info
		mp3i->SetFileEZ3(&headerfile);

		// Read ID3v1 if exist
		if (mp3i->HaveID3v1())
		{
			struct ID3V1MP3Info id3i;
			
			if (fread(&id3i, 1, sizeof(id3i), file) != sizeof(id3i))
			{	delete mp3i;
				goto err_loadEZ3;
			}
			if (mp3i->SetID3v1(&id3i) == FALSE)
			{	delete mp3i;
				goto err_loadEZ3;
			}
			
		}

		// Read ID3v2 frames
		for (int i = 0; i<headerfile.nCountFrame; i++)
		{
			EZ3HEADERFRAME headerframe;
			
			if (fread(&headerframe, 1, sizeof(headerframe), file) != sizeof(headerframe))
			{	delete mp3i;
				goto err_loadEZ3;
			}

			
			LPTSTR str = (LPTSTR)malloc((headerframe.size+1)*sizeof(TCHAR));
			if (str)
			{
				fread(str, sizeof(TCHAR), headerframe.size, file);
				str[headerframe.size] = '\0';

				CMP3StringFrame *sf;
				int fid = headerframe.fid;

				if (fid == ID3TAG_NUMTAGS || !ID3Frames[fid].Supported)	// unknown TAG or unsupported
				{	delete mp3i;
					goto err_loadEZ3;
				}

				sf = new CMP3StringFrame(fid, str);
				
				if (sf == NULL)
				{	free(str);
					delete sf;
					delete mp3i;

					goto err_loadEZ3;
				}
				mp3i->AddFrame(sf, FALSE);

				//free(str); car nettoyage lors du destructor
			}



		}
		
		free(szFilename);
		
		int pos = m_minfo.AddInfo(mp3i);
		int nImage =  mp3i->GetTagVersion();

		// Initialize item-specific LVITEM members.         
		// Add the item.         
		lvi.iItem = iItem; 
		lvi.iImage = nImage;
		lvi.iSubItem = 0;         
		lvi.lParam = (LPARAM)mp3i;    // item data  
		ListView_InsertItem(m_hwndList, &lvi);  
		
		m_nTotalLength += mp3i->GetPlayLength();

	}

	fclose(file);


	UpdateStatus();
	LV_Redraw(TRUE);
	LV_StatGenre();

	return;

err_loadEZ3:
	fclose(file);
	MessageBox(m_hDlg, TEXT_ERR_CORRUPTFILE, TEXT_ERROR, MB_OK|MB_ICONSTOP);
	return;
	
}


/* ************************************************************************ */
void CDlgPlaylist::LV_LoadM3U(TCHAR *filename)
{
	TCHAR szTemp[4096];
	FILE *filePlaylist;		
	BOOL bExtM3U;
	TCHAR *p;
	int iItem, nImage = 0;
	LV_ITEM lvi;
	int index;
	
	if ( (filePlaylist = _tfopen(filename, _T("r+t") )) == NULL)
		return;
	
	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 
	m_minfo.ResetList();
	
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	m_nTotalLength = 0;
	m_nSelLength   = 0;

	
	// Scan playlist Winamp
	for (iItem = 0; 
		_fgetts(szTemp, NUMELEMENTS(szTemp), filePlaylist);
		iItem++)
	{

		// 1st detect is extented playlist  ?
		if (iItem == 0)
		{	bExtM3U = (_tcsncmp(szTemp, _T("#EXTM3U"), 7) == 0 ? TRUE :FALSE);
			continue;
		}

		// Skip extended information if exist
		if (bExtM3U)
			if(_tcsncmp(szTemp, _T("#EXTINF"), 7)==0)
				_fgetts(szTemp, NUMELEMENTS(szTemp), filePlaylist);

		// Read info media filename	 
		p = szTemp + _tcslen(szTemp) -1;
		*p = '\0';
		
		tstring filename(szTemp);

		index = InsertFile(filename);

		if (index >= 0)		
		{
			CMP3Info *mp3i = m_minfo.GetInfo(index);

			nImage = mp3i->GetTagVersion();
			// Initialize item-specific LVITEM members.         
			// Add the item.         
			lvi.iItem = iItem; 
			lvi.iImage = nImage;
			lvi.iSubItem = 0;         
			lvi.lParam = (LPARAM)mp3i;    // item data  
			ListView_InsertItem(m_hwndList, &lvi);  
			
			m_nTotalLength += mp3i->GetPlayLength();
		}
	}

	fclose(filePlaylist);
	

	UpdateStatus();
	LV_Redraw(TRUE);
	LV_StatGenre();
}

/* ************************************************************************ */
void CDlgPlaylist::LV_LoadPLS(TCHAR *filename)
{
	TCHAR szTemp[4096];
	FILE *filePlaylist;		
	TCHAR *p, *szFile;
	int iItem, nImage=0;
	LV_ITEM lvi;
	int index;
	
	if ( (filePlaylist = _tfopen(filename, _T("r+t"))) == NULL)
		return;
	
	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 
	m_minfo.ResetList();
	
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	m_nTotalLength = 0;
	m_nSelLength   = 0;

	
	// Scan playlist Winamp
	for (iItem = 0; 
		_fgetts(szTemp, NUMELEMENTS(szTemp), filePlaylist);
		iItem++)
	{

		// 1st detect is extented playlist  ?
		if (iItem == 0)
		{	if(_tcsnccmp(szTemp, _T("[playlist]"), 10) != 0)
			{
				MessageBox(m_hDlg, TEXT_ERR_BADFILEPLS, TEXT_ERROR, MB_OK|MB_ICONSTOP);
				return;
			}
		}

		// Skip extended information if exist
		if(_tcsnccmp(szTemp, _T("File"), 4) !=0)
			continue;

		p = _tcschr(szTemp, '=');
		if (p==0)
			continue;
		else
			p++;	
		
		szFile = p;

		// Read info media filename	 
		p = szFile + _tcslen(szFile) -1;
		*p = '\0';
		
		tstring filename(szFile);

		index = InsertFile(filename);
		
		if (index >= 0)		
		{
			CMP3Info *mp3i = m_minfo.GetInfo(index);
			
			nImage = mp3i->GetTagVersion();
			// Initialize item-specific LVITEM members.         
			// Add the item.         
			lvi.iItem = iItem; 
			lvi.iImage = nImage;
			lvi.iSubItem = 0;         
			lvi.lParam = (LPARAM)mp3i;    // item data  
			ListView_InsertItem(m_hwndList, &lvi);  
			
			m_nTotalLength += mp3i->GetPlayLength();
		}
	}

	fclose(filePlaylist);


	UpdateStatus();
	LV_Redraw(TRUE);
	LV_StatGenre();
}


/* ************************************************************************ */
int CDlgPlaylist::InsertFile(tstring& file)
{
	// Verif if file is *.MP3 format extension 
	// check the extensions
	if (CheckExtension(file))
	{
		CMP3Info *mp3i;
		mp3i = new CMP3Info(file);
		
		if (mp3i)
		{
			int pos = m_minfo.AddInfo(mp3i);
			
			return (pos);
		}
	}
	
	return (-1);
}

/* ************************************************************************ */
void CDlgPlaylist::LV_WriteEZ3(TCHAR* filename)
{
	FILE *file;		
	EZ3HEADER header;
	int i;
	int nCount = m_minfo.GetCount();
	
	if ( (file = _tfopen(filename, _T("wb"))) == NULL)
		return;

	_tcscpy(header.tag, _T("EZ3DATA1"));
	header.nCountFile = nCount;
	fwrite (&header, sizeof(header), 1, file);

	
	for (i = 0; i< nCount; i++)
	{
		CMP3Info *mp3i = m_minfo.GetInfo(i);
	
		if (mp3i)
		{
			EZ3HEADERMP3INFO hmi;

			mp3i->GetFileEZ3(&hmi);	

			// Save file info header
			fwrite (&hmi, sizeof(hmi), 1, file);	
			fwrite (mp3i->GetCFilename(), sizeof(TCHAR), hmi.nLenFilename, file);

			// Save ID3v1 Tag
			if (mp3i->HaveID3v1())
			{
				struct ID3V1MP3Info *id3i = mp3i->GetID3v1();
				if (id3i)
					fwrite (id3i, sizeof(struct ID3V1MP3Info), 1, file);	
			}
			
			// Save ID3v2 Frames
			for (int j=0; j<mp3i->GetFrameCount(); j++)
			{
				CMP3Frame *mp3f = mp3i->GetIndexFrame(j);

				if (mp3f)
				{
					// Ne sauvegarde dans la plyalist EZ3 que les frame de type STRING
					// Compte uniquement les frames de type STRING uniquement
					int nFrameType = mp3f->GetType();

					if(nFrameType == ID3ENC_STRING)
					{
						tstring str = _T("");
						EZ3HEADERFRAME hfr;						

						str = ((CMP3StringFrame *)mp3f)->GetString();
						
						hfr.fid = mp3f->GetID();
						hfr.size = str.length();

						// Save frame header
						fwrite (&hfr, sizeof(hfr), 1, file);	
						// Save frame
						fwrite (str.c_str(), sizeof(TCHAR), hfr.size, file);
					}	
					
				}
				
			}

		
		}
	}

	fclose(file);
	
}

/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteM3U(TCHAR *filename)
{
	FILE *file;		// winamp.m3u playlist
	int i, nCount;
	LV_ITEM lvi;
	tstring s1, s2;
	
	if ( (file = _tfopen(filename, _T("w+t"))) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 
	 
	_ftprintf(file, _T("#EXTM3U\n"));

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;

		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		GetFrameInfo(mp3i, ID3TAG_TPE1, s1);	// Artist
		GetFrameInfo(mp3i, ID3TAG_TIT2, s2);	// Title
		
		_ftprintf(file, _T("#EXTINF,%d,%s - %s\n"),
			mp3i->GetPlayLength(),
			s1.c_str(),
			s2.c_str());

		s1 = mp3i->GetCFilename();
		_ftprintf(file, _T("%s\n"), s1.c_str());
	
	}
	fclose(file);

	return TRUE;
}


/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WritePLS(TCHAR *filePLS)
{
	FILE *file;		// winamp.m3u playlist
	int i, nCount;
	LV_ITEM lvi;
	tstring filename, artist, title;
	
	if ( (file = _tfopen(filePLS, _T("w+t") )) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 
	
	_ftprintf(file, _T("[playlist]\n") );

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) 
			continue;

		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		GetFrameInfo(mp3i, ID3TAG_TPE1, artist);	// Artist
		GetFrameInfo(mp3i, ID3TAG_TIT2, title);		// Title
		filename = mp3i->GetCFilename();			// filename
		
		_ftprintf(file, _T("File%d=%s\nTitle%d=%s - %s\nLength%d=%d\n"), 
			i+1, 
			filename.c_str(),
			i+1,
			artist.c_str(), 
			title.c_str(),
			i+1,
			mp3i->GetPlayLength());
			
		
	
	}
	_ftprintf(file, _T("NumberOfEntries=%i\nVersion=2"), nCount);
	fclose(file);

	return TRUE;
}

/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteHTML_Table(TCHAR* filename)
{
	FILE *file;		// playlist HTML format
	int i,k,nCount,nCountSelect;
	int iColArtist;
	LV_ITEM lvi;
	TCHAR *title, *p; 
	TCHAR szCol[256];
	unsigned long uTotalSize = 0;
	long nTotalLength = 0;
	int nOldShowID3v1;

	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;


	if (gParam.bExportBothID3)
		nOldShowID3v1 = gParam.nShowID3v1;


	title = _tcschr(filename, '\\');
	if (!title)
		title = filename;
	else
		title++;

	p = _tcschr(title, '.');
	if (p) 	*p = '\0';

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED;  

	// Detect Artist column for add hyperlink
	if (gParam.bExportHTML_Link==1) 
	{	for (i = 0; i < gParam.NbCol; i++)
		{	if(gParam.column_order[i]==0)	
			{	iColArtist = i;
				break;
			}
		}	
	}		
	
	// Header / Body / CSS
	LV_WriteHTML_CSS(file, title);
	
	
	// Initialize Table format
	_ftprintf(file, _T("<table border=1 cellpadding=5 cellspacing=0>\n"));
	_ftprintf(file, _T("<tr class=\"ColumnHeader\">\n"));
	
	// Table header , columns header
	_ftprintf(file, _T("<th>N</th>\n"));	
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];		//$ UseCol
		
		if ((gParam.bExportBothID3) && (gDefault_column[k].nID3 == 1))
		{
			_ftprintf(file, _T("<th>%s(id3v1)</th>\n"), gDefault_column[k].szName);
			_ftprintf(file, _T("<th>%s(id3v2)</th>\n"), gDefault_column[k].szName);
		}
		else
			_ftprintf(file, _T("<th>%s</th>\n"), gDefault_column[k].szName);
	}
	_ftprintf(file, _T("</tr>\n"));

	nCountSelect = 0;

	for (i = 0; i< nCount; i++)
	{
		// New begin row
		_ftprintf(file, _T("<tr class=\"ColumnText\">\n"));

		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;
		
		if (!mp3i)
			continue;

		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		nCountSelect++;
		uTotalSize   += (mp3i->GetFileSize() / 1024 / 1024);
		nTotalLength += mp3i->GetPlayLength();
		
		_ftprintf(file, _T("<td>%i</td>\n"), i+1);

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			int h = gParam.column_order[k];

			if ((gParam.bExportBothID3) && (gDefault_column[h].nID3 == 1))
			{
				//recupere info ID3v1
				gParam.nShowID3v1 = 0;  
				LV_WriteHTML_TableColumn(file, mp3i, k, iColArtist);

				//recupere info ID3v2
				gParam.nShowID3v1 = 1;  
				LV_WriteHTML_TableColumn(file, mp3i, k, iColArtist);
			}
			else
			{
				LV_WriteHTML_TableColumn(file, mp3i, k, iColArtist);
			}
				

		}

		// End begin row
		_ftprintf(file, _T("</tr>\n") );
	
	}


	// End table
	_ftprintf(file, _T("</table>\n<BR>\n"));
	
	
	// Pied de page resume / copyright
	LV_WriteHTML_Footer(file, szCol, uTotalSize, nTotalLength, nCountSelect);

	fclose(file);

	if (gParam.bExportBothID3)
		gParam.nShowID3v1 = nOldShowID3v1;


	return TRUE;
}

/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteHTML_Listing(TCHAR* filename)
{
	FILE *file;		// playlist HTML format
	int i,k,nCount;
	LV_ITEM lvi;
	TCHAR *title, *p; 
	TCHAR szCol[1024];
	unsigned long uTotalSize = 0;
	long nTotalLength = 0;
	int nCountSelect = 0;
	int nOldShowID3v1;
	
	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;

	if (gParam.bExportBothID3)
		nOldShowID3v1 = gParam.nShowID3v1;
	

	title = _tcsrchr(filename, '\\');
	if (!title)
		title = filename;
	else
		title++;

	p = _tcsrchr(title, '.');
	if (p) 	*p = '\0';

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED;  


	// Header / Body / CSS
	LV_WriteHTML_CSS(file, title);


	// Contenu en ligne	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;

		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;
		
		nCountSelect++;
		uTotalSize   += (mp3i->GetFileSize() / 1024 / 1024); 
		nTotalLength += mp3i->GetPlayLength();

		if (gParam.bExportBothID3)
		{
			tstring artist = _T("");
			tstring title  = _T("");

			//recupere info ID3v1
			gParam.nShowID3v1 = 0;  
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);	// Artist ID3v1
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);		// Title ID3v1

			// Artist & title
			if (gParam.bExportHTML_Link == 1) 
			{	_ftprintf(file, _T("<a href =\"%s\">(id3v1) %s - %s</a><BR>\n"),
					mp3i->GetCFilename(), 
					artist.c_str(), 
					title.c_str()); 
			}
			else
			{
				_ftprintf(file, _T("<B>(id3v1) %s - %s</B><BR>\n"),
					artist.c_str(), 
					title.c_str()); 
			}

			//recupere info ID3v2
			gParam.nShowID3v1 = 1;  
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);	// Artist ID3v1
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);		// Title ID3v1

			// Artist & title
			if (gParam.bExportHTML_Link == 1) 
			{	_ftprintf(file, _T("<a href =\"%s\">(id3v2) %s - %s</a><BR>\n"),
					mp3i->GetCFilename(), 
					artist.c_str(), 
					title.c_str()); 
			}
			else
			{
				_ftprintf(file, _T("<B>(id3v2) %s - %s</B><BR>\n"),
					artist.c_str(), 
					title.c_str()); 
			}

		}
		else
		{
			tstring artist = _T("");
			tstring title  = _T("");
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);	// Artist
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);		// Title

			// Artist & title
			if (gParam.bExportHTML_Link == 1) 
			{
				_ftprintf(file, _T("<a href =\"%s\">%s - %s</a><BR>\n"),
					mp3i->GetCFilename(), 
					artist.c_str(), 
					title.c_str()); 
			}
			else
			{
				_ftprintf(file, _T("<B>%s - %s</B><BR>\n"),
					artist.c_str(), 
					title.c_str()); 
			}
		}


		// Scan all columns except Artist and title
		for (k = 0; k < gParam.NbCol; k++)
		{
			int h  = gParam.column_order[k];
			
			if (h > 1)	
			{
				if ((gParam.bExportBothID3) && (gDefault_column[h].nID3 == 1))
				{
					//recupere info ID3v1
					gParam.nShowID3v1 = 0;  
					LV_GetInfoCol(mp3i, k, szCol); 	
					_ftprintf(file, _T("- %s (id3v1): %s<BR>\n"), gDefault_column[h].szName,szCol); 

					//recupere info ID3v2
					gParam.nShowID3v1 = 1;  
					LV_GetInfoCol(mp3i, k, szCol); 	
					_ftprintf(file, _T("- %s (id3v2): %s<BR>\n"), gDefault_column[h].szName,szCol); 
				}
				else
				{
					LV_GetInfoCol(mp3i, k, szCol); 	
					_ftprintf(file, _T("- %s: %s<BR>\n"), gDefault_column[h].szName,szCol); 
				}
			}
		}
		
		_ftprintf(file, _T("<BR>\n") );		// seperate line between songs
			
	}

	_ftprintf(file, _T("<BR>\n") );		// seperate line between songs

	// Pied de page resume / copyright
	LV_WriteHTML_Footer(file, szCol, uTotalSize, nTotalLength, nCountSelect);

	fclose(file);

	if (gParam.bExportBothID3)
		gParam.nShowID3v1 = nOldShowID3v1;
		
	return TRUE;
}


/* ************************************************************************* */
void CDlgPlaylist::LV_WriteHTML_TableColumn(FILE *file, CMP3Info *mp3i, int iCol, int iColArtist)
{
	TCHAR szTemp[1024];
	
	LV_GetInfoCol(mp3i, iCol, szTemp); 	
	
	int retVide = (_tcscmp(szTemp, _T(" "))==0) ||  (_tcslen(szTemp) == 0);
	
	if (gParam.bExportHTML_Link == 1 && iCol == iColArtist) 
	{
		tstring filename(_T(""));
		filename = mp3i->GetCFilename();
		
		if (!retVide)
			//  nowrap
			_ftprintf(file, _T("<td nowrap><a href =\"%s\">%s</a></td>\n"),
				filename.c_str(), szTemp);
		else
			_ftprintf(file, _T("<td><a href =\"%s\">&nbsp;</a></td>\n"), 
				filename.c_str());
	}
	else
	{
		if (!retVide)
			//  nowrap
			_ftprintf(file, _T("<td nowrap>%s</td>\n"), szTemp);
		else
			_ftprintf(file, _T("<td>&nbsp;</td>\n") );
	}


}




/* ************************************************************************* */
void CDlgPlaylist::LV_WriteHTML_CSS(FILE *file, TCHAR *title)
{
	TEMPLATESTRUCT *tpl;
	int nCSSStyle[4][2]    =  {{0,0},{0,1},{1,0},{1,1}};
	char *CSSText[MAX_CSS] =  { _T("TextPage"), _T("TitlePage"), _T("ColumnHeader"), _T("ColumnText")};

	
	// Header /Body structure
	_ftprintf(file, _T("<html>\n<head>\n<title>%s</title>\n"), title);
	_ftprintf(file, _T("<style type=\"text/css\">\n"));
	
	tpl = &gParam.Template[gParam.nCurrentTemplate];

	// Insert Style CSS
	for (int i = 0; i< MAX_CSS;i++)
	{
		int nStyle  = nCSSStyle[ tpl->css[i].nFontStyle ][0];
		int nWeight = nCSSStyle[ tpl->css[i].nFontStyle ][1];
	
		_ftprintf(file, _T(".%s {font-family:%s; font-size:%s; font-style:%s; font-weight:%s; color:#%02X%02X%02X; background-color:#%02X%02X%02X }\n"),
			CSSText[i],
			CSSFontName[tpl->css[i].nFontName],
			CSSFontSize[tpl->css[i].nFontSize],
			CSSFontStyle[nStyle],
			CSSFontWeight[nWeight],
			GetRValue(tpl->css[i].cColor),
			GetGValue(tpl->css[i].cColor),
			GetBValue(tpl->css[i].cColor),
			GetRValue(tpl->css[i].cColorBack),
			GetGValue(tpl->css[i].cColorBack),
			GetBValue(tpl->css[i].cColorBack));
			
	}
	
	_ftprintf(file, _T("</style>\n</head>\n<body class=\"TextPage\" link=#%02X%02X%02X>\n"),
		GetRValue(tpl->cLinkColor),
		GetGValue(tpl->cLinkColor),
		GetBValue(tpl->cLinkColor));
		
	_ftprintf(file, _T("<center><h1 class=\"TitlePage\">Playlist page:%s</h1></center>\n"), title);

}




/* ************************************************************************* */
void CDlgPlaylist::LV_WriteHTML_Footer(FILE *file, TCHAR *szCol, unsigned long uTotalSize, long nTotalLength, int nCountSelect)
{
	long nAvgLength = 0;
	
	// Description 
	_ftprintf(file, TEXT_HTML_TOTALFILE, nCountSelect);
	_ftprintf(file, TEXT_HTML_TOTALSIZE, uTotalSize);
	
	Time2Str(nTotalLength, szCol, FALSE);
	_ftprintf(file, TEXT_HTML_TOTALLENGTH, szCol);

	// Avoid Divide per zero
	if (nCountSelect == 0) nCountSelect =1;

	nAvgLength = nTotalLength / nCountSelect;
	Time2Str(nAvgLength, szCol, TRUE);
	_ftprintf(file, TEXT_HTML_AVGLENGTH, szCol);

	_ftprintf(file, TEXT_HTML_CREATEBY );
	_ftprintf(file, TEXT_HTML_VERSION );
	_ftprintf(file, _T("<a href=\"mailto:boyerjl@club-internet.fr\">") );
	_ftprintf(file, TEXT_HTML_JLB );

	_ftprintf(file, TEXT_HTML_DATE, GetFormattedDateTime());

	_ftprintf(file, _T("</body></html>\n"));
	
}



/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteASCII(TCHAR* filename)
{
	FILE *file;		// playlist ASCII text format
	int i,k,nCount;
	LV_ITEM lvi;
	TCHAR szCol[256];

	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];		// UseCol
		if (i != gParam.NbCol -1)	
			_ftprintf(file, _T("%s\t"), gDefault_column[k].szName);
		else
			_ftprintf(file, _T("%s\n"), gDefault_column[k].szName);
	}

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;
		
		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			LV_GetInfoCol(mp3i, k, szCol); 	
			if (k != gParam.NbCol -1)
				_ftprintf(file, _T("%s\t"),szCol);
			else
				_ftprintf(file, _T("%s\n"), szCol);

		}
	
	}
	
	fclose(file);
	return TRUE;
}

/* ************************************************************************ */
BOOL CDlgPlaylist::LV_WriteASCIIFixed(TCHAR* filename)
{
	FILE *file;		// playlist ASCII text format
	int i,k,nCount;
	LV_ITEM lvi;
	TCHAR szCol[256];
	TCHAR szFormat[256];
	int *nColMaxArray;
	int nLen;
	
	nCount = ListView_GetItemCount(m_hwndList);

	nColMaxArray = (int*)calloc(gParam.NbCol, sizeof(int));

	if (nColMaxArray == NULL)
		return FALSE;

	
	// Determine les largeurs max
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;
		
		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			LV_GetInfoCol(mp3i, k, szCol); 	
			
			nLen = _tcslen(szCol);
			
			if (nLen > nColMaxArray[k])
				nColMaxArray[k] =  nLen;
	
		}
	
	}
	
	// Si lles titres de colonnes sont plus grand que les informations
	for (i = 0; i < gParam.NbCol; i++)
	{
		k = gParam.column_order[i];	
		
		nLen = _tcslen(gDefault_column[k].szName);
		
		if (nLen > nColMaxArray[i])
			nColMaxArray[k] =  nLen;
	}


	
	if ( (file = _tfopen(filename, _T("w+t") )) == NULL)
		return FALSE;

	nCount = ListView_GetItemCount(m_hwndList);
	

	// Header
	nLen = 0;
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];	

		_stprintf(szFormat, _T("%%-%ds  | "), nColMaxArray[k]);
		_stprintf(szCol, szFormat, gDefault_column[k].szName);

		_ftprintf(file, szCol);

		if (i == gParam.NbCol -1)	
			_ftprintf(file, _T("\n"));

		nLen += _tcslen(szCol);
	}

	// Ligne separation header
	for (i = 0; i < gParam.NbCol; i++)
	{	k = gParam.column_order[i];	

		_stprintf(szFormat, _T("%%-%ds"), nColMaxArray[k]);
		_stprintf(szCol, szFormat, gDefault_column[k].szName);
		_tcsset(szCol, '-');
		_ftprintf(file,  _T("%s--+-"), szCol);

		if (i == gParam.NbCol -1)	
			_ftprintf(file, _T("\n"));

		nLen += _tcslen(szCol);
	}


	// Sauvegarde des colonnes
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i)
			continue;
		
		if (gParam.bExportAllItems == 1 && (lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		// Scan all columns
		for (k = 0; k < gParam.NbCol; k++)
		{
			LV_GetInfoCol(mp3i, k, szCol); 	
			
			_stprintf(szFormat, _T("%%-%ds  | "), nColMaxArray[k]);
			_ftprintf(file, szFormat, szCol);

			if (k == gParam.NbCol -1)	
				_ftprintf(file, _T("\n"));

		}
	
	}
	
	fclose(file);
	return TRUE;
}



/* ************************************************************************ */
void CDlgPlaylist::LV_AddFiles()
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256];
	TCHAR szFile[16384];
	TCHAR szFileTitle[256]; 
	TCHAR szFilter[] = TEXT_FILTERADDFILE;

	SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), szDirName);
	*szFile = '\0'; 	
	
	memset(&ofn, 0, sizeof(OPENFILENAME)); 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = NULL; //m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.lpstrTitle = (LPCTSTR) TEXT_ADDFILE_TITLE; 
	ofn.Flags = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_ALLOWMULTISELECT | OFN_EXPLORER; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 	
	
	if (GetOpenFileName(&ofn) == 0) 
	{
		if (CommDlgExtendedError() == FNERR_BUFFERTOOSMALL)
			MessageBox(m_hDlg, TEXT_ADDFILE_TOOMANYFILE, TEXT_ADDFILE_ERROR, MB_OK|MB_ICONSTOP); 
		return;
	}

	else
	{
		TCHAR szCurDirName[MAX_PATH];
		TCHAR szPath[MAX_PATH];
		size_t size;
		int iItem, nImage=0;
		LV_ITEM lvi;
			
		// Initialize LVITEM members that are common to all items. 
		lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
		lvi.state = 0;     
		lvi.stateMask = 0; 
		lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

		GetCurrentDirectory(sizeof(szCurDirName), szCurDirName );
		size = _tcslen(szFile);

		if (_tcslen(szCurDirName) != size)
		{
			//Only one file selected
			
			tstring filename(szFile);

			int index = InsertFile(filename);
			
			if (index >= 0)		
			{	
				CMP3Info *mp3i = m_minfo.GetInfo(index);

				nImage = mp3i->GetTagVersion();

				iItem = ListView_GetItemCount(m_hwndList);
	
				lvi.iItem = iItem; 
				lvi.iImage = nImage;
				lvi.iSubItem = 0;         
				lvi.lParam = (LPARAM)mp3i;    // item data  
				ListView_InsertItem(m_hwndList, &lvi);  
					
				m_nTotalLength += mp3i->GetPlayLength();
			}
		}
		else
		{
			// multiselected files
			BOOL bExit = FALSE;
			TCHAR *p;

			_tcscpy(szPath, szFile);
			p = szPath +  size - 1;	
			if ( *p != '\\')
				_tcscat(szPath, _T("\\"));

			while (!bExit)
			{
				
				TCHAR *pFile = szFile + size + 1;
				int filesize = _tcslen(pFile);
				
				if (!filesize)
					bExit = TRUE;
				else
				{
					TCHAR szTemp[1024];
					
					int index;

					size += (filesize + 1);

					_tcscpy(szTemp, szPath);
					_tcscat(szTemp, pFile);
	
					tstring filename(szTemp); 

					index = InsertFile(filename);
				
					if (index >= 0)		
					{	
						CMP3Info *mp3i = m_minfo.GetInfo(index);

						nImage = mp3i->GetTagVersion();

						iItem = ListView_GetItemCount(m_hwndList);
			
						lvi.iItem = iItem; 
						lvi.iImage = nImage;
						lvi.iSubItem = 0;         
						lvi.lParam = (LPARAM)mp3i;    // item data  
						ListView_InsertItem(m_hwndList, &lvi);  
							
						m_nTotalLength += mp3i->GetPlayLength();
					}
	
				}
			
			
			}
	
		}

		UpdateStatus();
		LV_StatGenre();
	}

}



/* ************************************************************************ */
void CDlgPlaylist::LV_AddDirectory()
{
	static TCHAR szDir[MAX_PATH]= {0};
	LPMALLOC pMalloc; 

	// Get's the Shell's default allocator 
	if (SHGetMalloc(&pMalloc) == NOERROR) 
	{ 
		BROWSEINFO bi; 
		LPITEMIDLIST pidl; 
		
		if (szDir[0] == '\0')
			GetCurrentDirectory(MAX_PATH, szDir);

		// Zero the BROWSEINFO structure to start off with 
		ZeroMemory( &bi, sizeof(bi) ); 
		
		bi.hwndOwner      = m_hDlg;
		bi.pidlRoot       = NULL;
		bi.pszDisplayName = szDir;
		bi.ulFlags        = BIF_RETURNONLYFSDIRS; //BIF_STATUSTEXT|BIF_RETURNONLYFSDIRS; 
		bi.lpfn           = BrowseRecurseCallbackProc;		// Set Callback function 
		bi.lParam         = (LPARAM)szDir;
		bi.iImage         = 0;

		// This next call issues the dialog box 
		if ((pidl = ::SHBrowseForFolder(&bi)) != NULL) 
		{ 
			if (::SHGetPathFromIDList(pidl, szDir)) 
			{ 
				// At this point szBuffer contains the selected path 
				TCHAR *p;
				TCHAR szDirSelect[MAX_PATH];
				HCURSOR hcurOld;
			
				_tcscpy(szDirSelect, szDir);
				p = szDirSelect + _tcslen(szDirSelect) - 1;
				if (*p == '\\')	*p = '\0';

				LV_Redraw(FALSE);
				
				ST_CreateStatus(2);
				ST_ChangeText((LPTSTR)_T("Scanning..."), 0);
				ST_ChangeText((LPTSTR)_T(""), 1);
				
				
				hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));

				LV_AddDir2List(szDirSelect);

				SetCursor(hcurOld);
			
				LV_Redraw(TRUE);
				LV_StatGenre();
				UpdateStatus();	
			} 
			
			// Free the PIDL allocated by SHBrowseForFolder 
			pMalloc->Free(pidl); 
		} 
		
		// Release the shell's allocator 
		pMalloc->Release(); 
	} 

	ST_CreateStatus(1);
	UpdateStatus();
}

/* ************************************************************************ */
void CDlgPlaylist::LV_DropFiles(HDROP hFilesInfo)
{    
	// Handle to file info structure when files
	// are dropped on our application
	int nTotalFiles; // Number of files dropped on our application
	int nIndex;		// Counter of files
	TCHAR szFileName[MAX_PATH];  // Buffer for filename
	
	// get number of files dropped
	nTotalFiles = DragQueryFile(hFilesInfo, -1, NULL, 0);
	
	// No files quit stop drag
	if (nTotalFiles == 0)
	{	DragFinish(hFilesInfo);
		return;
	}
	
	LV_Redraw(FALSE);
	ST_ChangeText((LPTSTR)_T("Scanning..."), 0);
	ST_ChangeText((LPTSTR)_T(""), 1);
	HCURSOR hcurOld = SetCursor(LoadCursor(plugin.hDllInstance,	(nTotalFiles > 1) ?	MAKEINTRESOURCE(IDC_DRPFIL_MULTIPLE) : MAKEINTRESOURCE(IDC_DRPFIL_SINGLE)));
	
	gbCheckRecurseDir = TRUE;

	// add the file names to the listbox
	for (nIndex = 0; nIndex < nTotalFiles; nIndex++)
	{
		// get the next file name
		DragQueryFile(hFilesInfo, nIndex, (LPSTR)szFileName, MAX_PATH);

		DWORD attr = GetFileAttributes(szFileName);

		if (attr & FILE_ATTRIBUTE_DIRECTORY)
		{
			// add directory name to the playlist
			if (_tcscmp(_T("."), szFileName) != 0 && _tcscmp(_T(".."), szFileName) != 0) 
				LV_AddDir2List(szFileName);
		}
		else
		{	// add mp3 file name to the playlist
			TCHAR *p = NULL;
			TCHAR szFilenamelowercase[MAX_PATH];
			_tcscpy(szFilenamelowercase, szFileName);
			CharLower(szFilenamelowercase);
            
			p = _tcsstr(szFilenamelowercase, _T(".mp3"));
            if (p) 
			{	tstring filename(szFileName);

				int index = InsertFile(filename);
				
				if (index >= 0)		
				{	
					LV_ITEM lvi;
					int iItem, nImage=0;

					CMP3Info *mp3i = m_minfo.GetInfo(index);

					nImage = mp3i->GetTagVersion();
					iItem = ListView_GetItemCount(m_hwndList);
					lvi.iItem = iItem; 
					lvi.iImage = nImage;
					lvi.iSubItem = 0;         
					lvi.lParam = (LPARAM)mp3i;    // item data  
					ListView_InsertItem(m_hwndList, &lvi);  
						
					m_nTotalLength += mp3i->GetPlayLength();
					ST_ChangeText((LPTSTR)szFileName, 1);
				}
			}
		}

	}

	// release memory Windows allocated for transferring 
	// filenames to app
	DragFinish(hFilesInfo);

	SetCursor(hcurOld);

	LV_UpdatePlaylist(); 
	LV_Redraw(TRUE);
	LV_StatGenre();
	
	ST_CreateStatus(1);
	UpdateStatus();	
}

/* ************************************************************************ */
void CDlgPlaylist::LV_AddDir2List(TCHAR* dir)
{    
	WIN32_FIND_DATA data;
	HANDLE hFile;
    TCHAR* buffer;
	
	buffer = (TCHAR*)malloc(MAX_PATH);    
	_stprintf(buffer, _T("%s\\*"), dir);
    
	hFile = FindFirstFile(buffer, &data);

    if(hFile != INVALID_HANDLE_VALUE) 
	{        
		do 
		{
            if (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) 
			{
				if (gbCheckRecurseDir == FALSE)
					continue;

                if (_tcscmp(_T("."), data.cFileName) != 0 && _tcscmp(_T(".."), data.cFileName) != 0) 
				{
                    TCHAR* subDir;
					
					subDir = (TCHAR*)malloc(MAX_PATH);
                    _stprintf(subDir, _T("%s\\%s"), dir, data.cFileName);
                    
					LV_AddDir2List(subDir);
                    
					free(subDir);
                }
            } 
			else 
			{
				TCHAR *p = NULL;
				
				TCHAR szFilenamelowercase[MAX_PATH];

				_tcscpy(szFilenamelowercase, data.cFileName);
				CharLower(szFilenamelowercase);
                
				p = _tcsstr(szFilenamelowercase, _T(".mp3"));
                if (p) 
				{
					LV_ITEM lvi;
					int iItem, nImage=0;
					TCHAR szTemp[256];
					int index;

					// Initialize LVITEM members that are common to all items. 
					lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
					lvi.state = 0;     
					lvi.stateMask = 0; 
					lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 
					
					_stprintf(szTemp, _T("%s\\%s"), dir, data.cFileName);

					tstring filename(szTemp); 

					index = InsertFile(filename);
				
					if (index >= 0)		
					{	
						CMP3Info *mp3i = m_minfo.GetInfo(index);
						
						nImage = mp3i->GetTagVersion();			
					
						iItem = ListView_GetItemCount(m_hwndList);

						lvi.iItem = iItem; 
						lvi.iImage = nImage;
						lvi.iSubItem = 0;         
						lvi.lParam = (LPARAM)mp3i;    // item data  
						ListView_InsertItem(m_hwndList, &lvi);  
							
						m_nTotalLength +=  mp3i->GetPlayLength();

						ST_ChangeText((LPTSTR)data.cFileName, 1);
					}
                }            
			}

		}while(FindNextFile(hFile, &data)); 

        FindClose(hFile);    
	}    
	
	free(buffer);
}





/* ************************************************************ */
void CDlgPlaylist::LV_Import()
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256]; 
	TCHAR szFile[256];
	TCHAR szFileTitle[256]; 
    TCHAR szFilter[] = TEXT_FILTEROPENDIALOG_IMPORT;
    
	SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), (LPTSTR)szDirName);
	*szFile = '\0'; 

	memset(&ofn, 0, sizeof(OPENFILENAME)); 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.lpstrTitle = (LPCTSTR) TEXT_IMPORTPLAYLIST; 
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 

	if (GetOpenFileName(&ofn)) 
	{ 
		switch (ofn.nFilterIndex) 
		{
			case 1:
				LV_LoadM3U(szFile);
				break;
			case 2:
				LV_LoadPLS(szFile);
				break;
		}
	} 
}


/* ************************************************************ */
void CDlgPlaylist::LV_Export()
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256]; 
	TCHAR szFile[256];
	TCHAR szFileTitle[256]; 
	TCHAR szFilter[] = 	TEXT_FILTEROPENDIALOG_EXPORT;
		
   
	SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), szDirName);
	*szFile = '\0'; 

	memset(&ofn, 0, sizeof(OPENFILENAME)); 

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = szFile; 
	ofn.nMaxFile = sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.lpstrTitle = (LPCTSTR) TEXT_EXPORTPLAYLIST; 
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 

	if (GetSaveFileName(&ofn)) 
	{ 
		switch (ofn.nFilterIndex) 
		{
		case 1:
			AddFileExtension(szFile, _T(".html")); 
			if (gParam.bExportHTML_Format == 0)
				LV_WriteHTML_Table(szFile);
			else
				LV_WriteHTML_Listing(szFile);
			break;
		case 2:
			AddFileExtension(szFile, _T(".m3u")); 
			LV_WriteM3U(szFile);
			break;
		case 3:
			AddFileExtension(szFile, _T(".pls")); 
			LV_WritePLS(szFile);
			break;
		case 4:
			AddFileExtension(szFile, _T(".txt")); 			
			LV_WriteASCII(szFile);
			break;
		case 5:
			AddFileExtension(szFile, _T(".txt")); 			
			LV_WriteASCIIFixed(szFile);
			break;
		}
	} 
}


/* ************************************************************ */
void CDlgPlaylist::AddFileExtension(TCHAR *szFile, TCHAR *szExt) 
{ 
	int nLen = _tcslen(szFile);
	int nLenExt  = _tcslen(szExt);

	if (nLen > nLenExt)
	{
		TCHAR *szFileExt = NULL;
		TCHAR *p = &szFile[nLen-nLenExt];

		// nLenExt+1: terminate string by '\0'
		szFileExt = (TCHAR*)malloc(nLenExt+1);		
		if (szFileExt == NULL)
		{	_tcscat(szFile, szExt);
			return;
		}
		_tcscpy(szFileExt, p);
		_tcslwr(szFileExt);

		// there is no extension so add it
		if (_tcscmp(szFileExt,szExt) != 0)
			_tcscat(szFile, szExt);

		free(szFileExt);
	}
	else
		_tcscat(szFile, szExt);

}

/* ************************************************************ */
BOOL CDlgPlaylist::doContextMenu(WPARAM wParam, LPARAM lParam) 
{ 
	HWND  hwndListView = (HWND)wParam; 
	HMENU hMenuLoad, hMenu; 

	if(hwndListView != GetDlgItem(m_hDlg, IDC_LVW_PLAYLIST)) 
		return FALSE; 

	hMenuLoad = LoadMenu(plugin.hDllInstance, MAKEINTRESOURCE(IDR_MENU1)); 
	
	// subMenu WinAMP
	hMenu = GetSubMenu(hMenuLoad, 1); 

	UpdateMenu(hMenu); 

	TrackPopupMenu(hMenu, 
				  TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
				  LOWORD(lParam), 
				  HIWORD(lParam), 
				  0, 
				  m_hDlg, 
				  NULL); 

	DestroyMenu(hMenuLoad); 

	return TRUE; 
} 

/* ************************************************************ */
void CDlgPlaylist::MenuButton(int nButton) 
{ 
	HMENU hMenuLoad, hMenu; 
	HWND hwndButton;
	RECT rect;
	POINT pt;

	hMenuLoad = LoadMenu(plugin.hDllInstance, MAKEINTRESOURCE(IDR_MENU1)); 
	
	// subMenu File
	switch (nButton)
	{ 
	case IDC_BUTTON_FILE:
		hMenu = GetSubMenu(hMenuLoad, 0); 
		
		break;
	case IDC_BUTTON_ADD:
		hMenu = GetSubMenu(hMenuLoad, 3); 
		break;
	case IDC_BUTTON_OPTION:
		{
			int i;

			hMenu = GetSubMenu(hMenuLoad, 4); 
			
			for (i=0; i<6; i++)
			{
				CheckMenuItem(hMenu, ID_SEARCH_ARTIST + i,  
					(gParam.nOptionSearch & (1<<i) )? 
					MF_BYCOMMAND | MF_CHECKED :
					MF_BYCOMMAND | MF_UNCHECKED);
				
			}
			
		}
		break;
	default:
		return;
	}

	hwndButton = GetDlgItem(m_hDlg, nButton); 
	GetClientRect(hwndButton, &rect);
	pt.x  = rect.left;
	pt.y  = rect.bottom;
	ClientToScreen(hwndButton, &pt);

	TrackPopupMenu(hMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
		pt.x, pt.y, 0, m_hDlg, NULL); 

	DestroyMenu(hMenuLoad); 

} 

/* ************************************************************ */
void CDlgPlaylist::MenuCheckItem(int nItem)
{
	gParam.nOptionSearch ^=  1<<nItem;
}

/* ************************************************************ */
void CDlgPlaylist::UpdateMenu(HMENU hMenu) 
{ 
	CheckMenuItem(hMenu, ID_PLAYLIST_SHOWDUPLICATE,  (m_bShowDuplicate == TRUE ? MF_BYCOMMAND | MF_CHECKED: MF_BYCOMMAND | MF_UNCHECKED));
} 

/* ************************************************************ */
void CDlgPlaylist::winamp_enqueue_to_playlist(BOOL bPlay)
{
	LVITEM lvi; 
	int i,nCount;
	char *file;
	COPYDATASTRUCT cds;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) continue;

		if ((lvi.state & LVIS_SELECTED)== LVIS_SELECTED)
		{
			if (bPlay)
				SendMessage(plugin.hwndParent, WM_USER, (WPARAM)NULL, 101);

			// Ajoute a la playlist la playlist Test.m3u identique pour un MP3 Titi.mp3
			
			file = (char*)mp3i->GetCFilename();

			cds.dwData = 100;
			cds.lpData = (void*)file;
			cds.cbData = _tcslen((TCHAR*)cds.lpData)+1;
			SendMessage(plugin.hwndParent, WM_COPYDATA, (WPARAM)NULL,(LPARAM)&cds);

			if (bPlay)
			{
				// Set the 1st song and play
				SendMessage(plugin.hwndParent, WM_USER, (WPARAM)0, 121);
				SendMessage(plugin.hwndParent, WM_USER, (WPARAM)NULL, 102);
			}
		}
		
	}

}

/* ************************************************************ */
void CDlgPlaylist::winamp_enqueue_all_to_playlist(BOOL bPlay)
{
	int i, nCount;
	LV_ITEM lvi;
	COPYDATASTRUCT cds;
	int iItem;

	iItem = ListView_GetNextItem(m_hwndList, -1, LVNI_ALL | LVNI_SELECTED); 

	nCount = ListView_GetItemCount(m_hwndList);

	lvi.mask =  LVIF_PARAM; 

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) continue;

		cds.dwData = 100;
		cds.lpData = (void*)mp3i->GetCFilename();
		cds.cbData = _tcslen((TCHAR*)cds.lpData)+1;
		SendMessage(plugin.hwndParent, WM_COPYDATA, (WPARAM)NULL,(LPARAM)&cds);
	}

	if (bPlay)
	{
		SendMessage(plugin.hwndParent, WM_USER, (WPARAM)0, 121);
		SendMessage(plugin.hwndParent, WM_USER, (WPARAM)NULL, 102);
	}


}


/* ************************************************************ */
void CDlgPlaylist::LV_SelectAll(BOOL bSelect)
{
	// See ST_UpdateLengthSelected
	m_bRefreshSelLength = FALSE;

	ListView_SetItemState(m_hwndList, -1, (bSelect ? LVIS_SELECTED : 0) , LVIS_SELECTED);  

	m_bRefreshSelLength = TRUE;;
}

/* ************************************************************ */
void CDlgPlaylist::LV_SelectToggle(void)
{
	LVITEM lvi; 
	int i,nCount;

	m_bRefreshSelLength = FALSE;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_STATE;
	lvi.stateMask = LVIS_SELECTED; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		ListView_SetItemState(m_hwndList, i, (lvi.state & LVIS_SELECTED)? 0: LVIS_SELECTED , LVIS_SELECTED);
	}

	m_bRefreshSelLength = TRUE;
	ST_UpdateLengthSelected(); 

}

/* ************************************************************ */
void CDlgPlaylist::LV_SelectWithSame(int nCol)
{
	LV_ITEM lvi;
	int i, nCount, ret;
	BOOL bSame;
	CMP3Info *mp3i1, *mp3i2;
	tstring str1, str2;
	
	// retreive information for the item selected
    i = ListView_GetNextItem(m_hwndList, -1, LVNI_SELECTED); 
    if (i < 0) 
		return;

	lvi.iItem = i; 
    lvi.iSubItem = 0; 
    lvi.mask = LVIF_PARAM; 

    ret = ListView_GetItem(m_hwndList, &lvi); 
	mp3i1 = (CMP3Info *)lvi.lParam;

	if (!mp3i1) return;
	
	// Unselect all item before
	LV_SelectAll(FALSE);
	m_bRefreshSelLength = FALSE;

	// Scan all item in the listview
	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		mp3i2 = (CMP3Info *)lvi.lParam;
		

		// Check the good column ?
		bSame = FALSE;
		switch(nCol)
		{
		case 0: // Artist
			GetFrameInfo(mp3i1, ID3TAG_TPE1, str1);
			GetFrameInfo(mp3i2, ID3TAG_TPE1, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;
		case 1: // Title
			GetFrameInfo(mp3i1, ID3TAG_TIT2, str1);
			GetFrameInfo(mp3i2, ID3TAG_TIT2, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;

		case 2: // Genre
			GetFrameInfo(mp3i1, ID3TAG_TCON, str1);
			GetFrameInfo(mp3i2, ID3TAG_TCON, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;

		case 3: // Year
			GetFrameInfo(mp3i1, ID3TAG_TYER, str1);
			GetFrameInfo(mp3i2, ID3TAG_TYER, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;
		
		case 4: // Album
			GetFrameInfo(mp3i1, ID3TAG_TALB, str1);
			GetFrameInfo(mp3i2, ID3TAG_TALB, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;
		
		case 5: // Comment
			GetFrameInfo(mp3i1, ID3TAG_COMM, str1);
			GetFrameInfo(mp3i2, ID3TAG_COMM, str2);
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;

		case 6: // Mode
			bSame = (mp3i1->GetMode() == mp3i2->GetMode()); 
			break;
		
		case 7: // Bitrate
			bSame = (mp3i1->GetBitrate() == mp3i2->GetBitrate()); 
			break;
		
		case 8: // Frequency
			bSame = (mp3i1->GetFrequency() == mp3i2->GetFrequency());
			break;
		
		case 9: // MPEG Layer
			{
				TCHAR mpeglayer1[100], mpeglayer2[100];
				mp3i1->GetMPEGVersionLayer(mpeglayer1);
				mp3i2->GetMPEGVersionLayer(mpeglayer2);
				ret = _tcscmp(mpeglayer1, mpeglayer2); 
				bSame = (ret == 0);
		
			}
			break;
		
		case 10: // Length Time
			bSame = (mp3i1->GetPlayLength() == mp3i2->GetPlayLength());
			break;
		
		case 11: // File size
			bSame = (mp3i1->GetMP3Size() == mp3i2->GetMP3Size());
			break;
		
		case 12: // File name
			str1= mp3i1->GetCFilename();
			str2= mp3i2->GetCFilename();
			ret = _tcscmp(str1.c_str(), str2.c_str()); 
			bSame = (ret == 0);
			break;
		
		default:
			bSame = FALSE;
			break;
		}

		
		ListView_SetItemState(m_hwndList, i, ((bSame)?LVIS_SELECTED:0), LVIS_SELECTED);
	}


	m_bRefreshSelLength = TRUE;
	ST_UpdateLengthSelected(); 
}

/* ************************************************************ */
void CDlgPlaylist::LV_SelectCrop(void)
{
	LVITEM lvi; 
	int i,nCount, nSelCount;
	
	nSelCount = ListView_GetSelectedCount(m_hwndList);
	if (nSelCount < 1)
		return;

	LV_Redraw(FALSE);

	nCount = ListView_GetItemCount(m_hwndList);
	
	i = 0;
	while (nCount > nSelCount)
	{
		ZeroMemory(&lvi, sizeof(lvi)); 
		lvi.mask = LVIF_STATE | LVIF_PARAM; 
		lvi.stateMask = LVIS_SELECTED; 
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		if ((lvi.state & LVIS_SELECTED)== LVIS_SELECTED)
			// skip select item
			i++;		
		else
		{
			CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

			if (mp3i) 
			{
				// delete and clear data for unselect item
				int pos = m_minfo.FindInfo(mp3i);
				
				if (pos >=0)
				{
					ListView_DeleteItem(m_hwndList, i);
					m_minfo.DeleteInfo(pos);
					

					nCount--;
				}
			}	
		}
	}

	
	LV_Redraw(TRUE);
	
	ST_UpdateLengthSelected();
	LV_UpdateStatus();
	LV_StatGenre();
}

/* ************************************************************ */
void CDlgPlaylist::LV_SelectDel(void)
{
	LVITEM lvi; 
	int i, nSelCount;
	
	nSelCount = ListView_GetSelectedCount(m_hwndList);
	if (nSelCount < 1)
		return;

	LV_Redraw(FALSE);

	i = 0;
	while (nSelCount > 0)
	{
		ZeroMemory(&lvi, sizeof(lvi)); 
		lvi.mask = LVIF_STATE | LVIF_PARAM; 
		lvi.stateMask = LVIS_SELECTED; 
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		if ((lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
		{
			// skip unselect item
			i++;
		}
		else
		{
			CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

			if (mp3i) 
			{
				// delete and clear data for unselect item
				int pos  = m_minfo.FindInfo(mp3i);
				
				if (pos >=0)
				{
					ListView_DeleteItem(m_hwndList, i);
					m_minfo.DeleteInfo(pos);
				

					nSelCount--;
				}
			}	
		}
	}

	// Selected the next item
	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_STATE; 
	lvi.stateMask = LVIS_SELECTED; 
	lvi.iItem  = i;
	ListView_SetItemState(m_hwndList, i, LVIS_SELECTED , LVIS_SELECTED);

	LV_Redraw(TRUE);

	ST_UpdateLengthSelected();
	LV_UpdateStatus();
	LV_StatGenre();
}



/* ************************************************************ */
void CDlgPlaylist::LV_SelectRemoveFile(void)
{
	LVITEM lvi; 
	int i, nSelCount;
	
	nSelCount = ListView_GetSelectedCount(m_hwndList);
	if (nSelCount < 1)
		return;


	if (MessageBox( m_hDlg, TEXT_CONFIRM_REMOVEFILE, _T("Warning"), MB_YESNO | MB_ICONWARNING ) == IDNO)
		return;


	LV_Redraw(FALSE);

	i = 0;
	while (nSelCount > 0)
	{
		ZeroMemory(&lvi, sizeof(lvi)); 
		lvi.mask = LVIF_STATE | LVIF_PARAM; 
		lvi.stateMask = LVIS_SELECTED; 
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		if ((lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
		{
			// skip unselect item
			i++;
		}
		else
		{
			nSelCount--;
			
			CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

			if (mp3i) 
			{
				LPCTSTR lpFileName;

				lpFileName = mp3i->GetCFilename();
					 
				if (DeleteFile(lpFileName) == 0)
				{
					// Delete file failed, show message error
					LPVOID lpMsgBuf;
					TCHAR szMsgErr[1024];

					FormatMessage( 
						FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
						(LPTSTR) &lpMsgBuf,
						0,
						NULL);
					
					_stprintf(szMsgErr, TEXT_ERROR_REMOVEFILE, lpFileName, lpMsgBuf);

					// Display the string.
					MessageBox( m_hDlg, (LPCTSTR)szMsgErr, TEXT_ERROR, MB_OK | MB_ICONINFORMATION );
					
					// Free the buffer.
					LocalFree( lpMsgBuf );

					i++;	// va au suivant
				}
				else
				{
					// delete and clear data for unselect item
					int pos  = m_minfo.FindInfo(mp3i);
					
					if (pos >=0)
					{
						m_minfo.DeleteInfo(pos);
						ListView_DeleteItem(m_hwndList, i);
					}
				}	

			}


		}
	}


	LV_Redraw(TRUE);

	ST_UpdateLengthSelected();
	LV_UpdateStatus();
	LV_StatGenre();
}




/* ************************************************************ */
void CDlgPlaylist::LV_UpdateStatus(void)
{
	LVITEM lvi; 
	int i,nCount;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_PARAM; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	m_nTotalLength = 0;

	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);
		
		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) continue;

		m_nTotalLength += mp3i->GetPlayLength();
	}

	UpdateStatus();

}

/* ************************************************************ */
void CDlgPlaylist::LV_StatGenre(void)
{
	int i, idGenre;
	CMP3Info *mp3i;
	tstring genre;

	for (i = 0; i < ID3TAG_MAX_GENRE+2; i++ )
	{
		gStatGenre[i].bCheck = FALSE;
		gStatGenre[i].nCount = 0;
	}
	
	// Scan all playlist  from memory 
	for (i=0; i < m_minfo.GetCount(); i++)
	{
		mp3i = m_minfo.GetInfo(i);
		
		genre.assign(_T(""));
		GetFrameInfo(mp3i, ID3TAG_TCON, genre);
		
		idGenre = GenreStr2Int(genre.c_str());
		
		if (idGenre >= 0 && idGenre < ID3TAG_MAX_GENRE+2)
		{
			gStatGenre[idGenre].bCheck = TRUE;
			gStatGenre[idGenre].nCount++;
		}

	}


	LV_DetermineFilter();
	LV_TitleWindowFilter();
}

/* ************************************************************ */
void CDlgPlaylist::LV_UpdatePlaylist()
{
	TCHAR szSearch[40];
	LVITEM lvi; 
	int i, iItem;
	BOOL bFind;

	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 
		
    // Initialize LVITEM members that are common to all items. 
    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state = 0;     
	lvi.stateMask = 0; 
    lvi.pszText = LPSTR_TEXTCALLBACK;   // app. maintains text 

	m_nTotalLength = 0;
	
	// Scan all playlist  from memory 
	GetDlgItemText(m_hDlg, IDC_EDIT_SEARCH, (LPTSTR)szSearch, 40);


	if (_tcslen(szSearch) != 0)
	{
		// /////////////////////////
		// Filter with search string 
		// /////////////////////////

		// Match case checked
		if (!(gParam.nOptionSearch & 32))
			_tcslwr(szSearch);

		iItem = 0;

		for (i=0; i < m_minfo.GetCount(); i++)
		{
			tstring artist =_T("");
			tstring title =_T(""); 
			tstring album =_T(""); 
			tstring comment =_T(""); 
			tstring filename =_T("");
			tstring genre= _T("");
						

			CMP3Info *mp3i = m_minfo.GetInfo(i);
			
			filename = mp3i->GetCFilename();
			GetFrameInfo(mp3i, ID3TAG_TPE1, artist);
			GetFrameInfo(mp3i, ID3TAG_TIT2, title);
			GetFrameInfo(mp3i, ID3TAG_TALB, album);
			GetFrameInfo(mp3i, ID3TAG_COMM, comment);
			GetFrameInfo(mp3i, ID3TAG_TCON, genre);
	
			bFind = FALSE;

			
			tstring artistlwr(artist.c_str());
			tstring titlelwr(title.c_str()); 
			tstring albumlwr(album.c_str()); 
			tstring commentlwr(comment.c_str()); 
			tstring filenamelwr(filename.c_str());
			

			// compare item in lowercase  
			if (!(gParam.nOptionSearch & 32))
			{
				_tcslwr((TCHAR*)artistlwr.c_str());
				_tcslwr((TCHAR*)titlelwr.c_str());
				_tcslwr((TCHAR*)albumlwr.c_str());
				_tcslwr((TCHAR*)commentlwr.c_str());
				_tcslwr((TCHAR*)filenamelwr.c_str());
			}

			if( (gParam.nOptionSearch & 1)  && _tcsstr(artistlwr.c_str(), szSearch)  != 0 ||
				(gParam.nOptionSearch & 2)  && _tcsstr(titlelwr.c_str(), szSearch)   != 0 ||
				(gParam.nOptionSearch & 4)  && _tcsstr(albumlwr.c_str(), szSearch) != 0 ||
				(gParam.nOptionSearch & 8)  && _tcsstr(commentlwr.c_str(), szSearch) != 0 ||
				(gParam.nOptionSearch & 16) && _tcsstr(filenamelwr.c_str(), szSearch) != 0 )
					bFind = TRUE;
			
			if (LV_Filter(mp3i) && bFind)
			{
			
				int nImage = mp3i->GetTagVersion();

				// Initialize item-specific LVITEM members.         
				// Add the item.         
				lvi.iItem = iItem; 
				lvi.iSubItem = 0;         
				lvi.iImage = nImage; 
				lvi.lParam = (LPARAM)mp3i;    // item data  
				ListView_InsertItem(m_hwndList, &lvi);  
				
				m_nTotalLength += mp3i->GetPlayLength();
				iItem++;
			}

		}

	}
	else
	{
		// /////////////////////////
		// No Filter search
		// /////////////////////////

		
		iItem = 0;
		for (i=0; i < m_minfo.GetCount(); i++)
		{
			
			CMP3Info *mp3i = m_minfo.GetInfo(i);

			if (LV_Filter(mp3i))
			{
				int nImage = mp3i->GetTagVersion();
				
				// Initialize item-specific LVITEM members.         
				// Add the item.         
				lvi.iItem = iItem; 
				lvi.iSubItem = 0;         
				lvi.iImage = nImage; 
				lvi.lParam = (LPARAM)mp3i;    // item data  
				ListView_InsertItem(m_hwndList, &lvi);  
				
				m_nTotalLength += mp3i->GetPlayLength();
				iItem++;
			}
			else
				iItem = iItem;

			
		}

	}


	LV_UpdateStatus();
	LV_Redraw(TRUE);
}


/* ************************************************************ */
void CDlgPlaylist::LV_DetermineFilter()
{
	m_nFilterBy = NO_FILTER;

	// Filter by genre ?
	int idGenre;
	BOOL bFilterGenre = FALSE;
	
	for (idGenre=0 ;idGenre<ID3TAG_MAX_GENRE+2; idGenre++)
	{
		if (gStatGenre[idGenre].bCheck == FALSE && gStatGenre[idGenre].nCount != 0)
		{	bFilterGenre = TRUE;
			break;
		}
	}
	
	if (bFilterGenre == TRUE)
		m_nFilterBy = FILTER_GENRE;

	// Filter by Artist
	if (!m_filterArtist.empty())
		m_nFilterBy |= FILTER_ARTIST;

	// Filter by Album
	if (!m_filterAlbum.empty())
		m_nFilterBy |= FILTER_ALBUM;

	// Filter by Comment
	if (!m_filterComment.empty())
		m_nFilterBy |= FILTER_COMMENT;

}


/* ************************************************************ */
BOOL CDlgPlaylist::LV_Filter(CMP3Info *mp3i)
{
	if (m_nFilterBy == NO_FILTER)
		return (TRUE);

	BOOL bFiltered = FALSE;
	
	// Genre OK ?
	if (m_nFilterBy & FILTER_GENRE)
	{
		tstring genre;
		GetFrameInfo(mp3i, ID3TAG_TCON, genre);
		int idGenre = GenreStr2Int(genre.c_str());
	
		if (gStatGenre[idGenre].bCheck == FALSE)
			return(FALSE);
	}
	// Artist OK ?
	if (m_nFilterBy & FILTER_ARTIST)
	{
		tstring artist;
		GetFrameInfo(mp3i, ID3TAG_TPE1, artist);
	
		if (artist.empty())
			return(FALSE);
		if (m_filterArtist.compare(artist))
			return(FALSE);
	}

	// Album OK ?
	if (m_nFilterBy & FILTER_ALBUM)
	{
		tstring album;
		GetFrameInfo(mp3i, ID3TAG_TALB, album);
	
		if (album.empty())
			return(FALSE);
		if (m_filterAlbum.compare(album))
			return(FALSE);
	}

	// Comment OK ?
	if (m_nFilterBy & FILTER_COMMENT)
	{
		tstring comment;
		GetFrameInfo(mp3i, ID3TAG_COMM, comment);
	
		if (comment.empty())
			return(FALSE);
		if (m_filterComment.compare(comment))
			return(FALSE);
	}

	return(TRUE);
}


/* ************************************************************ */
void CDlgPlaylist::LV_TitleWindowFilter()
{
	TCHAR szTitle[MAX_PATH] = {0};
	
	_tcscat(szTitle, szAppName);
	
	if (m_nFilterBy == NO_FILTER)
	{	SetWindowText(m_hDlg,(LPCTSTR)szTitle);
		return;
	}

	_tcscat(szTitle, _T(" ("));
	
	if (m_nFilterBy & FILTER_GENRE)
	{
		_tcscat(szTitle, TEXT_GENRE);
		_tcscat(szTitle, _T(", "));
	}
	
	if (m_nFilterBy & FILTER_ARTIST)
	{
		_tcscat(szTitle, TEXT_ARTIST);
		_tcscat(szTitle, _T(", "));
	}

	
	if (m_nFilterBy & FILTER_ALBUM)
	{
		_tcscat(szTitle, TEXT_ALBUM);
		_tcscat(szTitle, _T(", "));
	}

	// Comment OK ?
	if (m_nFilterBy & FILTER_COMMENT)
	{
		_tcscat(szTitle, TEXT_COMMENT);
		_tcscat(szTitle, _T(", "));
	}

	int nLen = _tcslen(szTitle);
	TCHAR *p =  &szTitle[nLen-2];
	*p++ = ')';
	*p = '\0';

	SetWindowText(m_hDlg,(LPCTSTR)szTitle);
}


/* ************************************************************ */
void CDlgPlaylist::LV_ShowDuplicate(void)
{
	LVITEM lvi; 
	int iItem = 0;
	int i,j;
	CMP3Info *mp3i1, *mp3i2;
	int *pDuplicates = NULL;
	int nCount = m_minfo.GetCount();
	
	if (nCount < 1)
		return;

	pDuplicates = (int*)calloc(nCount, sizeof(int));

	if (pDuplicates == NULL)
		return;
	
	HCURSOR hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));

	// /////////////////
	// Detect duplicates
	for (i=0; i<nCount; i++)
	{
		mp3i1 = m_minfo.GetInfo(i);
		/*
		tstring artist1 =_T("");
		tstring title1 =_T("");
		GetFrameInfo(mp3i1, ID3TAG_TPE1, artist1);
		GetFrameInfo(mp3i1, ID3TAG_TPE1, title1);
		*/
		if (pDuplicates[i]==1)
			continue;
		
		for (j = i + 1; j<nCount; j++)
		{
			BOOL bEqual = FALSE;
		
			mp3i2 = m_minfo.GetInfo(j);

			if (mp3i1->GetFileSize() == mp3i2->GetFileSize())
			{
				bEqual = TRUE;
			}
			/*
			else
			{
				tstring artist2 = _T("");
				tstring title2 = _T("");
				GetFrameInfo(mp3i2, ID3TAG_TPE1, artist2);
				GetFrameInfo(mp3i2, ID3TAG_TPE1, title2);

				if ((artist1.compare(artist2) == 0) && (title1.compare(title2) == 0)) 
				{
					bEqual = TRUE;
				}
			}
			*/
			if (bEqual)
			{
				pDuplicates[i] = 1;				
				pDuplicates[j] = 1;
			}
			

		}

	}

	// ////////////////////////////////////
	// insert into listivew only duplicates
	LV_Redraw(FALSE);
	ListView_DeleteAllItems(m_hwndList); 

    lvi.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE; 
    lvi.state     = 0;     
	lvi.iSubItem  = 0;         
	lvi.stateMask = 0; 
    lvi.pszText   = LPSTR_TEXTCALLBACK;

	m_nTotalLength = 0;
	iItem = 0;

	for (i=0; i<nCount; i++)
	{
		if (pDuplicates[i]==1)
		{
			mp3i1 = m_minfo.GetInfo(i);
		
			// Add result items in listview 
			lvi.iItem = iItem; 
			lvi.iImage = 0; 
			lvi.lParam = (LPARAM)mp3i1;    // item data  
			ListView_InsertItem(m_hwndList, &lvi);  
			
			m_nTotalLength += mp3i1->GetPlayLength();
			iItem++;
		}
	}
	
	free(pDuplicates);

	LV_UpdateStatus();
	LV_Redraw(TRUE);

	ST_UpdateLengthSelected();
	SetCursor(hcurOld);
}


/* ************************************************************ */
void CDlgPlaylist::LV_DeadFiles(void)
{
	LVITEM lvi; 
	int i,nCount;
	DWORD attr;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;    // Retreive item data  MP3Info
		
		if (!mp3i) continue;

		attr = GetFileAttributes(mp3i->GetCFilename());
		
		ListView_SetItemState(m_hwndList, i, (attr == 0xFFFFFFFF) ? LVIS_SELECTED : 0 , LVIS_SELECTED);
	}
}


/* ************************************************************ */
void CDlgPlaylist::LV_Redraw(BOOL bUpdate)
{
	//turn the listview's redrawing on/off 
	
	SendMessage(m_hwndList, WM_SETREDRAW, bUpdate, 0); 
	
	if (bUpdate)
	{
		//turn the listview's redrawing back on and force it to draw 
		SendMessage(m_hwndList, WM_SETREDRAW, TRUE, 0); 
		InvalidateRect(m_hwndList, NULL, TRUE); 
		UpdateWindow(m_hwndList); 
	}
}

/* ********************************************************* */
/* Re/initialize the ListControl Columns */
void CDlgPlaylist::LV_ColumnResetDisplay(BOOL firstime)
{
    LV_COLUMN   lvc;
    int         i;
    int         nColumn;
    int         widths[COLUMN_MAX];
    int         order[COLUMN_MAX];
    int         shown[COLUMN_MAX];
    
    GetColumnWidths(widths);
    GetColumnOrder(order);
    GetColumnShown(shown);


    if (!firstime)
    {
        nColumn = LV_ColumnGetNum();

        // The first time thru this won't happen, on purpose
        for (i = 0; i < nColumn; i++)
        {
            widths[m_realColumn[i]] = ListView_GetColumnWidth(m_hwndList, i);
        }

        SetColumnWidths(widths);

        for (i = nColumn; i > 0; i--)
        {
            ListView_DeleteColumn(m_hwndList, i - 1);
        }
    }

    ListView_SetExtendedListViewStyle(m_hwndList,LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
	
	LV_ColumnsGrid();

    lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
    
	

    nColumn = 0;
    for (i = 0; i < COLUMN_MAX; i++)
    {
        if (shown[order[i]]==1)
        {
            lvc.mask      = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
			lvc.fmt       = gDefault_column[order[i]].nAlignment;
            lvc.pszText   = gDefault_column[order[i]].szName;
            lvc.iSubItem  = nColumn;
            lvc.cx        = widths[order[i]];
            ListView_InsertColumn(m_hwndList, nColumn, &lvc);

            m_realColumn[nColumn] = order[i];

            nColumn++;
        }
    }

    // Fill this in so we can still sort on columns NOT shown
    for (i = 0; i < COLUMN_MAX && nColumn < COLUMN_MAX; i++)
    {
        if (shown[order[i]]==0)
        {
            m_realColumn[nColumn] = order[i];

            nColumn++;
        }
    }

   
  	{
		int column;

		reverse_sort = FALSE;

		if ((column = GetSortColumn()) > 0)
		{
			m_last_sort = -1;
		}
		else
		{
			column = abs(column);
			m_last_sort = column;
		}

		LV_ColumnSort(column, TRUE);
	}

	gParam.NbCol = LV_ColumnGetNum();
}


/* ********************************************************* */
void CDlgPlaylist::LV_ColumnSort(int column, BOOL bColumn, BOOL bReverse)
{
    int save_column = 0;
    int use_column = column;

    if (bColumn) // Was a column enum used??
    {
        int i;

		for (i = 0; i < COLUMN_MAX; i++)
		{
			if(m_realColumn[i] == column)
			{
				use_column = i;
				break;
			}
        }
    }

    /* Sort 'em */
    save_column = m_realColumn[use_column] ;

    if (bReverse == TRUE)
		reverse_sort = (column == m_last_sort && !reverse_sort) ? TRUE : FALSE;


    
	ListView_SortItems(m_hwndList, ListViewCompareProc, m_realColumn[use_column]);
    m_last_sort = use_column;

    if (reverse_sort)
        save_column = -(save_column);

	SetSortColumn(save_column);

    Header_SetSortInfo(m_hwndList, use_column, !reverse_sort);

}


/* ********************************************************* */
int CDlgPlaylist::LV_ColumnGetNum()
{
    int  nColumn = 0;
    int  i;
    HWND hwndHeader;
    int  shown[COLUMN_MAX];

    GetColumnShown(shown);

    hwndHeader = GetDlgItem(m_hwndList, 0);

	nColumn = Header_GetItemCount(hwndHeader);
    
	if (nColumn < 1)
    {
        nColumn = 0;
        for (i = 0; i < COLUMN_MAX ; i++ )
        {
            if (shown[i])
                nColumn++;
        }
    }
    return nColumn;
}

/* ********************************************************* */
void CDlgPlaylist::LV_ColumnGetRealOrder(int order[])
{
	int tmpOrder[COLUMN_MAX];
	int nColumnMax;
	int i;
	
	nColumnMax = LV_ColumnGetNum();
	
	/* Get the Column Order and save it */
	ListView_GetColumnOrderArray(m_hwndList, nColumnMax, tmpOrder);
	
	for (i = 0; i < nColumnMax; i++)
	    order[i] = m_realColumn[tmpOrder[i]];

}



/* ********************************************************* */
void CDlgPlaylist::LV_ColumnSaveSetting()
{
	int widths[COLUMN_MAX];
	int order[COLUMN_MAX];
	int shown[COLUMN_MAX];
	int tmpOrder[COLUMN_MAX];
	int nColumnMax = 0;

	GetColumnOrder(order);
    GetColumnShown(shown);
    GetColumnWidths(widths);

    gParam.NbCol = nColumnMax = LV_ColumnGetNum();

    /* Get the Column Order and save it */
    ListView_GetColumnOrderArray(m_hwndList, nColumnMax, tmpOrder);

    for (int i = 0; i < nColumnMax; i++)
    {
        widths[m_realColumn[i]] = ListView_GetColumnWidth(m_hwndList, i);
        order[i] = m_realColumn[tmpOrder[i]];
    }
    
    SetColumnWidths(widths);
    SetColumnOrder(order);
}



/* ************************************************************ */
void CDlgPlaylist::LV_MatchTime(int nLengthMax)
{
	LV_SelectAll(FALSE);
	
	MatchTime(m_hwndList, nLengthMax);

	ST_UpdateLengthSelected();
}





/* ************************************************************************ */
void CDlgPlaylist::GetMP3Selected()
{

	// retreive information for the item selected
	CMP3Info *mp3iSel;
	LV_ITEM lvi;
	int i, nCount;
	int iSel = 0;
	
	m_bRefreshSelLength = FALSE;

	m_nSelCount = ListView_GetSelectedCount(m_hwndList);
	if (m_nSelCount < 1)
		return;


	// Vide l'ancienne selection en memoire et la recreer avec 
	// le nombre d'elements selectionne
	if (m_mp3iSelArray != NULL)
		free(m_mp3iSelArray);

	m_mp3iSelArray = (CMP3Info **)calloc(m_nSelCount, sizeof(CMP3Info *));


	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_STATE | LVIF_PARAM; 
	lvi.stateMask = LVIS_SELECTED; 
	 
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		if ((lvi.state & LVIS_SELECTED) != LVIS_SELECTED)
			continue;

		mp3iSel = (CMP3Info *)lvi.lParam;

		m_mp3iSelArray[iSel] = mp3iSel;
		iSel++;
	}

}

/* ************************************************************ */
void CDlgPlaylist::SetMP3Selected(void)
{
	LVITEM lvi; 
	int i,nCount, iSel;
	
	LV_SelectAll(FALSE);
 
	m_bRefreshSelLength = FALSE;

	ZeroMemory(&lvi, sizeof(lvi)); 
	lvi.mask = LVIF_PARAM; 

	nCount = ListView_GetItemCount(m_hwndList);
	
	for (i = 0; i< nCount; i++)
	{
		int bFirst = TRUE;
		
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;    // Retreive item data  MP3Info
		
		if (!mp3i) 
			continue;

		for (iSel=0; iSel < m_nSelCount; iSel++)
		{
			if (mp3i == m_mp3iSelArray[iSel])
			{
				ListView_SetItemState(m_hwndList, i, LVIS_SELECTED , LVIS_SELECTED);
	
				if (bFirst)
				{	bFirst = FALSE;	
					ListView_SetItemState(m_hwndList, i, LVIS_FOCUSED , LVIS_FOCUSED);
					ListView_EnsureVisible(m_hwndList, i, TRUE);
				}
				break;
			}
		}
	}

	m_bRefreshSelLength = TRUE;

	SetFocus(m_hwndList);
	ST_UpdateLengthSelected();
}


/* ************************************************************************ */
void CDlgPlaylist::ST_UpdateLengthSelected()
{
	if (m_bRefreshSelLength == FALSE)		
		return;
	
	// retreive length Time for the item selected
	m_nSelLength   = 0;

	int nSelCount = ListView_GetSelectedCount(m_hwndList);
	int nCount = ListView_GetItemCount(m_hwndList);

	if (nSelCount == nCount)
		m_nSelLength = m_nTotalLength;
	else
	{
		if( nSelCount >= 1)
		{
			LVITEM lvi; 
			ZeroMemory(&lvi, sizeof(lvi)); 
			lvi.mask      = LVIF_STATE | LVIF_PARAM; 
			lvi.stateMask = LVIS_SELECTED; 
			
			for (int i = 0; i< nCount; i++)
			{
				lvi.iItem  = i;
				ListView_GetItem(m_hwndList, &lvi);
				
				CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

				if (!mp3i) continue;

				if ((lvi.state & LVIS_SELECTED)== LVIS_SELECTED)
				{
					m_nSelLength += mp3i->GetPlayLength();
				}
				
			}
		}
	}

	TCHAR szTimeFrmt[100];
	TCHAR szTemp[100];
	Time2Str(m_nSelLength, &szTimeFrmt[0], TRUE);
	_stprintf(szTemp, TEXT_STATUS_SELLENGTH, szTimeFrmt);
	ST_ChangeText((LPTSTR)szTemp, 4);

} 

/*************************************************************
* Update the status bar
**************************************************************/
void CDlgPlaylist::UpdateStatus()
{
	TCHAR szTemp[255];
	TCHAR szTimeFrmt[100];
	TCHAR *szID3v[] = { _T("ID3v1"), _T("ID3v2"), _T("ID3 ?") };
	
	int nTotalFile = ListView_GetItemCount(m_hwndList);

	_stprintf(szTemp, TEXT_STATUS_TOTALFILE, nTotalFile);
	ST_ChangeText((LPTSTR)szTemp, 0);

	_tcscpy(szTemp, szID3v[gParam.nShowID3v1]);
	ST_ChangeText((LPTSTR)szTemp, 1);


	if (nTotalFile != 0)
		m_nAvgLength = m_nTotalLength / nTotalFile;
	else
		m_nAvgLength = 0;

	Time2Str(m_nAvgLength, &szTimeFrmt[0], TRUE);
	_stprintf(szTemp, TEXT_STATUS_AVGLENGTH, szTimeFrmt);
	ST_ChangeText((LPTSTR)szTemp, 2);

	Time2Str(m_nTotalLength, &szTimeFrmt[0], FALSE);
	_stprintf(szTemp, TEXT_STATUS_TOTALLENGTH, szTimeFrmt);
	ST_ChangeText((LPTSTR)szTemp, 3);
}


/* ********************************************************** */
BOOL CDlgPlaylist::ST_CreateStatus(int nMode)
{
	int aWidth[5];
	int aNumElt;
	
	switch(nMode)
	{
		case 0:	// creation	
			m_hwndStat = CreateStatusWindow( 
			  WS_CHILD | WS_BORDER | WS_VISIBLE,// window styles
			  _T(""),							// default window text
			  m_hDlg,							// parent window
			  IDC_STATUS);						// ID

			if(!m_hwndStat)
			   return FALSE;

		case 1:	// Define parts
			aWidth[0] = 100;
			aWidth[1] = 138;
			aWidth[2] = 235;
			aWidth[3] = 520;
			aWidth[4] = -1;

			aNumElt = 5;
			break;

		case 2:	// Define parts for scanning
			aWidth[0] = 100;
			aWidth[1] = -1;
			
			aNumElt = 2;
			break;

	}

	SendMessage(m_hwndStat, SB_SETPARTS , (LPARAM)aNumElt, (LPARAM)(LPINT)aWidth);
	return TRUE;
}

/* ********************************************************** */
BOOL CDlgPlaylist::ST_ChangeText(LPTSTR szText, int iPart )
{
    SendMessage(m_hwndStat, SB_SETTEXT, iPart, (LPARAM)szText);
	return TRUE;
}











// *********************************************************
// *********************************************************
/* Header code - Directional Arrows */
// *********************************************************
// *********************************************************
static LRESULT CALLBACK Header_WndProc( HWND hwnd,  UINT uMsg,  WPARAM wParam,  LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_ERASEBKGND:
        return Header_ListCtrlOnErase( hwnd, (HDC)wParam);

    case WM_DRAWITEM:
        {
            LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT)lParam;
            RECT rcLabel;
            int nSavedDC;
            HRGN hrgn;
            int nOffset;
            SIZE size;
            TCHAR tszText[256];
            HD_ITEM hdi;
            UINT uFormat;
            HDC hdc;

            hdc = lpdis->hDC;

            CopyRect(&rcLabel, &(lpdis->rcItem));

            /* save the DC */
            nSavedDC = SaveDC(hdc);

            /* set clip region to column */
            hrgn = CreateRectRgnIndirect(&rcLabel);
            SelectObject(hdc, hrgn);
            DeleteObject(hrgn);

            /* draw the background */
            FillRect(hdc, &rcLabel, GetSysColorBrush(COLOR_3DFACE));

            /* offset the label */
            GetTextExtentPoint32(hdc, _T(" "), 1, &size);
            nOffset = size.cx * 2;

            /* get the column text and format */
            hdi.mask = HDI_TEXT | HDI_FORMAT;
            hdi.pszText = tszText;
            hdi.cchTextMax = 255;
            Header_GetItem(GetDlgItem(hwnd, 0), lpdis->itemID, &hdi);

            /* determine format for drawing label */
            uFormat = DT_SINGLELINE | DT_NOPREFIX | DT_NOCLIP |
                DT_VCENTER | DT_END_ELLIPSIS;

            /* determine justification */
            if (hdi.fmt & HDF_CENTER)
                uFormat |= DT_CENTER;
            else if (hdi.fmt & HDF_RIGHT)
                uFormat |= DT_RIGHT;
            else
                uFormat |= DT_LEFT;

            /* adjust the rect if selected */
            if (lpdis->itemState & ODS_SELECTED)
            {
                rcLabel.left++;
                rcLabel.top += 2;
                rcLabel.right++;
            }

            /* adjust rect for sort arrow */
            if (lpdis->itemID == m_uHeaderSortCol)
            {
                rcLabel.right -= (3 * nOffset);
            }

            rcLabel.left += nOffset;
            rcLabel.right -= nOffset;

            /* draw label */
            if (rcLabel.left < rcLabel.right)
                DrawText(hdc, tszText, -1, &rcLabel, uFormat);

            /* draw the arrow */
            if (lpdis->itemID == m_uHeaderSortCol)
            {
                RECT rcIcon;
                HPEN hpenLight, hpenShadow, hpenOld;

                CopyRect(&rcIcon, &(lpdis->rcItem));

                hpenLight = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DHILIGHT));
                hpenShadow = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DSHADOW));
                hpenOld = (HPEN)SelectObject(hdc, hpenLight);

                if (m_fHeaderSortAsc)
                {
                    /* draw triangle pointing up */
                    MoveToEx(hdc, rcIcon.right - 2 * nOffset, nOffset - 1, NULL);
                    LineTo(hdc, rcIcon.right - 3 * nOffset / 2,
                        rcIcon.bottom - nOffset);
                    LineTo(hdc, rcIcon.right - 5 * nOffset / 2 - 2,
                        rcIcon.bottom - nOffset);
                    MoveToEx(hdc, rcIcon.right - 5 * nOffset / 2 - 1,
                        rcIcon.bottom - nOffset, NULL);

                    SelectObject(hdc, hpenShadow);
                    LineTo(hdc, rcIcon.right - 2 * nOffset, nOffset - 2);
                }
                else
                {
                    /* draw triangle pointing down */
                    MoveToEx(hdc, rcIcon.right - 3 * nOffset / 2, nOffset - 1,
                        NULL);
                    LineTo(hdc, rcIcon.right - 2 * nOffset - 1,
                        rcIcon.bottom - nOffset);
                    LineTo(hdc, rcIcon.right - 2 * nOffset - 1,
                        rcIcon.bottom - nOffset);
                    MoveToEx(hdc, rcIcon.right - 2 * nOffset - 1,
                        rcIcon.bottom - nOffset, NULL);

                    SelectObject(hdc, hpenShadow);
                    LineTo(hdc, rcIcon.right - 5 * nOffset / 2 - 1,
                        nOffset - 1);
                    LineTo(hdc, rcIcon.right - 3 * nOffset / 2,
                        nOffset - 1);
                }
                SelectObject(hdc, hpenOld);
                DeleteObject (hpenLight);
                DeleteObject (hpenShadow);
            }
            RestoreDC(hdc, nSavedDC);
            return 1;
        }

    case WM_NOTIFY:
        {
            HD_NOTIFY   *pHDN = (HD_NOTIFY*)lParam;

            // This code is for using bitmap in the background
            // Invalidate the right side of the control when a column is resized
            if (pHDN->hdr.code == HDN_ITEMCHANGINGW ||
                pHDN->hdr.code == HDN_ITEMCHANGINGA)
            {
               
			/*	
	
			*/
            }
        }
        break;
    }

    /* message not handled */
	return CallWindowProc(g_lpHeaderWndProc, hwnd, uMsg, wParam, lParam);

}

// *********************************************************
/* Subclass the Listview Header */
static void Header_Initialize(HWND hwndList)
{
    /* this will subclass the listview (where WM_DRAWITEM gets sent for
       the header control) */

#ifdef STRICT 
	g_lpHeaderWndProc = (WNDPROC)GetWindowLong(hwndList, GWL_WNDPROC);
#else 
	g_lpHeaderWndProc = (FARPROC)GetWindowLong(hwndList, GWL_WNDPROC);
#endif 

    SetWindowLong(hwndList, GWL_WNDPROC, (LONG)Header_WndProc);
}


// *********************************************************
/* Set Sort information needed by Header */
static void Header_SetSortInfo(HWND hwndList, int nCol, BOOL bAsc)
{
    HWND hwndHeader;
    HD_ITEM hdi;

    m_uHeaderSortCol = nCol;
    m_fHeaderSortAsc = bAsc;

    hwndHeader = GetDlgItem(hwndList, 0);

    /* change this item to owner draw */
    hdi.mask = HDI_FORMAT;
    Header_GetItem(hwndHeader, nCol, &hdi);
    hdi.fmt |= HDF_OWNERDRAW;
    Header_SetItem(hwndHeader, nCol, &hdi);

    /* repaint the header */
    InvalidateRect(hwndHeader, NULL, TRUE);
}

// *********************************************************
static BOOL Header_ListCtrlOnErase(HWND hWnd, HDC hDC)
{
	RECT rcClient;
	
    GetClientRect(hWnd, &rcClient);

	/* Use standard control face color */
	{
		HBRUSH hBrush = CreateSolidBrush(GetSysColor(COLOR_WINDOW));
        HBRUSH hOldBrush;

		hOldBrush = (HBRUSH)SelectObject(hDC, hBrush);
		FillRect(hDC, &rcClient, hBrush);
        SelectObject(hDC, hOldBrush);
		DeleteObject(hBrush);
	}

    return TRUE;
}





/* End of source file */




















