/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
// Misc Functions, no classes

#ifndef _ID3V1_H
#define _ID3V1_H

#include <windows.h>

// Length of ID3-Tag
#define ID3TAG_LENGTH 128
#define ID3TAG_TAG "TAG"
#define ID3TAG_MAX_GENRE 145
#define ID3_DEFGENRE 0		// default Genre: "none"; maps in the original ID3 to 255


// Specific for CMP3Info
struct ID3V1MP3Info
{
	// ID3-Tag internal info  add NULL characters at the end '\0'
	unsigned char SongName[31];
	unsigned char Artist[31];
	unsigned char Album[31];
	unsigned char Year[5];
	unsigned char Comment[31];
	unsigned char Genre;		
	int Track;
};


/***************************************************************************/
/* the structure to access the ID3V1 Tag */

struct ID3V1Info
{
	// ID3-Tag info (the last 128 bytes of the file)
	unsigned char Tag[3];	// if Tag == "TAG" then it exists
	unsigned char SongName[30];
	unsigned char Artist[30];
	unsigned char Album[30];
	unsigned char Year[4];
	unsigned char Comment[30];
	unsigned char Genre;		// private!!!
	int Track;
};

/***************************************************************************/
/* the functions to access the ID3V1 Tag */

#ifndef CGLOBAL
#ifdef __cplusplus
#define CGLOBAL extern "C"
#else
#define CGLOBAL extern
#endif
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

/* this function reads the ID3V1 info from a given file-pointer
 * the returnvalue is 0 for a successful read */
CGLOBAL int ReadID3V1Info(LPCTSTR file, struct ID3V1Info *id3i);

/* write an ID3v1 structure in the file
 * the returnvalue is 0 for a successful write */
CGLOBAL int WriteID3V1Info(LPCTSTR file, struct ID3V1Info *id3i, int flags);

/* 
 * Return TRUE if ID3V1 TAG exist in this file */
CGLOBAL BOOL ExistID3Tagv1(LPCTSTR file);


CGLOBAL unsigned char GenreStr2Int(LPCTSTR str);
CGLOBAL LPCTSTR GenreInt2Str(unsigned char genre);

/***************************************************************************/
/* some arrays, which are exported */

CGLOBAL LPTSTR ID3_Genres[];

/* these are the available flags for the WriteID3V1Info(): */
enum {
 ID3V1_WRITE_NORMAL,  /* overwrite if necessary, else add a new one */
 ID3V1_WRITE_REMOVE,  /* remove the ID3-Tag is possible */
 ID3V1_WRITE_REPLACE  /* write only, if there exists one */
};

#endif
