/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
/* This file defines some things about the fields used in the ID3V2 header
 *
 * you can include this file multiple times, to get the right array
 *
 * define ID3V2_CREATE_TAG_ARRAY 1
 *    creates the ID3Ident array
 *
 * if none of the above are defined, then the enumed-list is generated
 */



/* this defines the type of fields available */

#ifndef __TagArray
#define __TagArray 1
struct TagArray
{
	const char *IDStr;
	const char *ID2Str;
	LPCTSTR description;
	LPCTSTR shortDesc;
	unsigned char priority;			// priority for sorting
	unsigned short Type : 3;		// type of field
	unsigned short Lang : 1;		// language support?
	unsigned short StrSize : 3;	// size of STRING-field. 0-unlimited
	unsigned short Supported : 1;	// is field supported?
	unsigned short Multi : 1;		// are multiple fields allowed?
};

/* This array defines the possible values for the encoding.
 * String is always an array of unicode string
 */
enum{	ID3ENC_BINARY,
		ID3ENC_STRING,
		ID3ENC_PICTURE,
		ID3ENC_URL,
};

#endif


#ifdef ID3V2_CREATE_TAG_ARRAY

#ifndef YES
#define YES 1
#endif
#ifndef NO
#define NO 0
#endif

#define ID3ARR(pri, code,string,strold,type,lang,ssize,supp,mult,sdes, des) \
	{string, strold, des, sdes, pri, type, lang, ssize, supp, mult},
const struct TagArray ID3Frames[] = {

#else

CGLOBAL const struct TagArray ID3Frames[];
#define ID3ARR(pri, code,string,strold,type,lang,ssize,supp,mult,sdes, des) \
	code ,
enum {

#endif /* ID3V2_CREATE_TAG_ARRAY */

//     PRI  ENUMED       IDSTR  ID2STR  Type           Lang SS SUPP MULT Short Desc			Description
ID3ARR(255, ID3TAG_UFID, "UFID", "UMI", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("MP3 Ident"),	_T("Unique MP3 Identifier"))
ID3ARR(255, ID3TAG_TIT1, "TIT1", "TT1", ID3ENC_STRING, NO,  1, YES, NO,  _T("Group"),		_T("Content group description"))
ID3ARR(  1, ID3TAG_TIT2, "TIT2", "TT2", ID3ENC_STRING, NO,  1, YES, NO,  _T("Title"),		_T("Title/Songname/Content description"))
ID3ARR( 10, ID3TAG_TIT3, "TIT3", "TT3", ID3ENC_STRING, NO,  1, YES, NO,  _T("Opus"),		_T("Subtitle/Description refinement"))
ID3ARR(  2, ID3TAG_TPE1, "TPE1", "TP1", ID3ENC_STRING, NO,  1, YES, NO,  _T("Artist"),		_T("Lead artist/Lead performer/Soloist"))
ID3ARR(  9, ID3TAG_TPE2, "TPE2", "TP2", ID3ENC_STRING, NO,  1, YES, NO,  _T("Orchestra"),	_T("Band/Orchestra/Accompaniment"))
ID3ARR(  8, ID3TAG_TPE3, "TPE3", "TP3", ID3ENC_STRING, NO,  1, YES, NO,  _T("Conductor"),	_T("Conductor/Performer refinement"))
ID3ARR(255, ID3TAG_TPE4, "TPE4", "TP4", ID3ENC_STRING, NO,  1, YES, NO,  _T("Remixed"),	_T("Interpreted, remixed, or otherwise modified by"))
ID3ARR(  7, ID3TAG_TCOM, "TCOM", "TCM", ID3ENC_STRING, NO,  1, YES, NO,  _T("Composer"),	_T("Composer"))
ID3ARR(255, ID3TAG_TEXT, "TEXT", "TXT", ID3ENC_STRING, NO,  1, YES, NO,  _T("Lyricist"),	_T("Lyricist/text writer"))
ID3ARR(  5, ID3TAG_TCON, "TCON", "TCO", ID3ENC_STRING, NO,  1, YES, NO,  _T("Genre"),		_T("Content type"))
ID3ARR(  3, ID3TAG_TALB, "TALB", "TAL", ID3ENC_STRING, NO,  1, YES, NO,  _T("Album"),		_T("Album/movie/show title"))
ID3ARR(  4, ID3TAG_TRCK, "TRCK", "TRK", ID3ENC_STRING, NO,  1, YES, NO,  _T("Track"),		_T("Track number on album"))
ID3ARR(  6, ID3TAG_TYER, "TYER", "TYE", ID3ENC_STRING, NO,  1, YES, NO,  _T("Year"),		_T("Year"))
ID3ARR(255, ID3TAG_TDAT, "TDAT", "TDA", ID3ENC_STRING, NO,  1, YES, NO,  _T("Date"),		_T("Date"))
ID3ARR(255, ID3TAG_TIME, "TIME", "TIM", ID3ENC_STRING, NO,  1, YES, NO,  _T("Time"),		_T("Time"))
ID3ARR(255, ID3TAG_TMED, "TMED", "TMT", ID3ENC_STRING, NO,  1, NO,  NO,  _T("Media"),		_T("Media type"))
ID3ARR(255, ID3TAG_TBPM, "TBPM", "TBM", ID3ENC_STRING, NO,  1, YES, NO,  _T("BPM"),			_T("BPM (median value of main music)"))
ID3ARR( 11, ID3TAG_TCOP, "TCOP", "TCR", ID3ENC_STRING, NO,  1, YES, NO,  _T("Copyright"),	_T("Copyright message"))
ID3ARR(255, ID3TAG_TPUB, "TPUB", "TPB", ID3ENC_STRING, NO,  1, YES, NO,  _T("Publisher"),	_T("Publisher"))
ID3ARR( 13, ID3TAG_TENC, "TENC", "TMB", ID3ENC_STRING, NO,  1, YES, NO,  _T("Encoded by"),	_T("Encoded by"))
ID3ARR(255, ID3TAG_TSSE, "TSSE", "TSS", ID3ENC_STRING, NO,  1, YES, NO,  _T("Encoding setting"),_T("Software/hardware and settings used for mpeg coding"))
ID3ARR( 12, ID3TAG_TOPE, "TOPE", "TXX", ID3ENC_STRING, NO,  1, YES, NO,  _T("Orig. Artist"),		_T("Original artist(s)/performer(s)"))
ID3ARR(255, ID3TAG_TOFN, "TOFN", "TOF", ID3ENC_STRING, NO,  1, YES, NO,  _T("Orig. file"),		_T("Original filename"))
ID3ARR(255, ID3TAG_TLEN, "TLEN", "TLE", ID3ENC_STRING, NO,  1, NO,  NO,  _T("Length"),			_T("Length"))
ID3ARR( 14, ID3TAG_TSIZ, "TSIZ", "TSI", ID3ENC_STRING, NO,  1, YES, NO,  _T("Size"),			_T("Size"))
ID3ARR(255, ID3TAG_TDLY, "TDLY", "TDY", ID3ENC_STRING, NO,  1, NO,  NO,  _T("Playlist delay"),	_T("Playlist delay"))
ID3ARR(255, ID3TAG_TKEY, "TKEY", "TKE", ID3ENC_STRING, NO,  1, YES, NO,  _T("Init key"),		_T("Initial key"))
ID3ARR(255, ID3TAG_TXXX, "TXXX", "TXX", ID3ENC_STRING, NO,  0, NO,  NO,  _T("User info"),		_T("User defined text information frame"))
ID3ARR(255, ID3TAG_IPLS, "IPLS", "IPL", ID3ENC_STRING, NO,  1, YES, NO,  _T("Involved people"),	_T("Involved people list"))
ID3ARR(255, ID3TAG_MCDI, "MCDI", "MCI", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("CD Identifier"),	_T("Music CD Identifier"))
ID3ARR(255, ID3TAG_ETCO, "ETCO", "ETC", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Event timing"),	_T("Event timing codes"))
ID3ARR(255, ID3TAG_SYTC, "SYTC", "STC", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Synced tempo"),	_T("Synced tempo codes"))
ID3ARR(200, ID3TAG_USLT, "USLT", "ULT", ID3ENC_STRING, YES, 2, YES, YES, _T("Unsynched lyric"),	_T("Unsychronized lyric/text"))
ID3ARR(255, ID3TAG_SYLT, "SYLT", "SLT", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Synched lyric"),	_T("Synchronized lyric/text"))
ID3ARR(100, ID3TAG_COMM, "COMM", "COM", ID3ENC_STRING, YES, 2, YES, YES, _T("Comment"),			_T("Comments"))
ID3ARR(255, ID3TAG_RVAD, "RVAD", "RVA", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Relative Volume adjust"), _T("Relative volume adjustment"))
ID3ARR(255, ID3TAG_EQUA, "EQUA", "EQU", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Equalization"),	_T("Equalization"))
ID3ARR(255, ID3TAG_RVRB, "RVRB", "REV", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Reverb"),			_T("Reverb"))
ID3ARR( 99, ID3TAG_APIC, "APIC", "PIC", ID3ENC_PICTURE,NO,  0, YES, YES, _T("Picture"),			_T("Attached picture"))
ID3ARR(255, ID3TAG_GEOB, "GEOB", "GEO", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Object"),			_T("General encapsulated object"))
ID3ARR(255, ID3TAG_PCNT, "PCNT", "CNT", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Play counter"),	_T("Play counter"))
ID3ARR(255, ID3TAG_POPM, "POPM", "POP", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Popularimeter"),	_T("Popularimeter"))
ID3ARR(255, ID3TAG_RBUF, "RBUF", "BUF", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Buffer"),			_T("Recommended buffer size"))
ID3ARR(255, ID3TAG_ENCR, "ENCR", "CRM", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Encrypt"),			_T("Encrypted meta frame"))
ID3ARR(255, ID3TAG_AENC, "AENC", "CRA", ID3ENC_BINARY, NO,  0, NO,  NO,  _T("Audio encrypt"),	_T("Audio encryption"))
ID3ARR(255, ID3TAG_WPUB, "WPUB", "TWW", ID3ENC_STRING, NO,  1, YES, NO,  _T("Official webpage"),					_T("Official webpage"))
ID3ARR(255, ID3TAG_WCOM, "WCOM", "WXX", ID3ENC_STRING, NO,  1, YES, NO,  _T("Commercial information"),				_T("Commercial information"))
ID3ARR(255, ID3TAG_WCOP, "WCOP", "WXX", ID3ENC_STRING, NO,  1, YES, NO,  _T("Copyright/Legal information"),			_T("Copyright/Legal information"))
ID3ARR(255, ID3TAG_WOAF, "WOAF", "WXX", ID3ENC_STRING, NO,  1, YES, NO,  _T("Official audio file webpage"),			_T("Official audio file webpage"))
ID3ARR(255, ID3TAG_WOAR, "WOAR", "WXX", ID3ENC_STRING, NO,  1, YES, NO,  _T("Official artist/performer webpage"),	_T("Official artist/performer webpage"))
ID3ARR(255, ID3TAG_WOAS, "WOAS", "WXX", ID3ENC_STRING, NO,  1, YES, NO,  _T("Official audio source webpage"),		_T("Official audio source webpage"))
ID3ARR(255, ID3TAG_WORS, "WORS", "WXX", ID3ENC_STRING, NO,  1, YES, NO,  _T("Official internet radio station homepage"), _T("Official internet radio station homepage"))
ID3ARR(255, ID3TAG_WPAY, "WPAY", "WXX", ID3ENC_STRING, NO,  1, YES, NO,  _T("Payment"),								_T("Payment"))
ID3ARR(100, ID3TAG_WXXX, "WXXX", "WXX", ID3ENC_STRING, NO,  2, YES, NO,  _T("URL"),										_T("General URL"))

/* this must be the last field, to know how long the array is */
ID3ARR(255, ID3TAG_NUMTAGS, NULL, NULL, ID3ENC_BINARY, NO,  0, NO,  NO,  NULL, NULL)

#ifdef ID3V2_CREATE_TAG_ARRAY

};

#else /* not ID3V2_CREATE_TAG_ARRAY */

};

#endif /* ID3V2_CREATE_TAG_ARRAY */

#undef ID3ARR
