/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
/* Here are the frames implemented */

#include "priv.h"
#undef ID3V2_CREATE_TAG_ARRAY
#include "MP3Frame.h"
// #include "MP3PicFrame.h"
// #include "MP3ext.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

/* to create the arrays, define ID3V2_CREATE_TAG_ARRAY and include ID3V2fields.h again */
#define   ID3V2_CREATE_TAG_ARRAY 1
#include "ID3V2fields.h"
#undef    ID3V2_CREATE_TAG_ARRAY

// OK, first my global function to allocate a frame from a given FrameID
CMP3Frame *CreateMP3Frame(int fid, int sid)
{
	CMP3Frame *mp3f = NULL;

	// check, if ID is in the range
	if (fid<0 || fid > ID3TAG_NUMTAGS) return(NULL);

	switch(ID3Frames[fid].Type)
	{
		case ID3ENC_STRING:		mp3f = new CMP3StringFrame(fid);	break;
//		case ID3ENC_PICTURE:	mp3f = new CMP3PicFrame(fid, sid);	break;
		case ID3ENC_BINARY:
		default:				mp3f = new CMP3BinFrame(fid);		break;
	}
	return(mp3f);
}

CMP3Frame::CMP3Frame(int _FrameID, int SubID)
{
	assert(_FrameID>=0 && _FrameID<ID3TAG_NUMTAGS);
	FrameID = (_FrameID>=0 && _FrameID<ID3TAG_NUMTAGS) ? _FrameID : ID3TAG_TXXX;
	FrameSubID = SubID;	// none by default
}

CMP3Frame::~CMP3Frame()
{
}

BOOL CMP3Frame::operator != (CMP3Frame &mp3f)
{
	return(!(this == &mp3f));
}

// append information to the tstring str
void CMP3Frame::appendInfo(tstring& str, int max)	// get information about the frame
{
	// append nothing to unknown frames
	return;
}

/******************* The Binary Frame *****************/
CMP3BinFrame::CMP3BinFrame(int ID) : CMP3Frame(ID)
{
	Length = 0;
	Buffer = NULL;
}

CMP3BinFrame::~CMP3BinFrame()
{
	if (Buffer) free(Buffer);
}

CMP3Frame *CMP3BinFrame::Clone()
{
	CMP3BinFrame *mf = new CMP3BinFrame(GetID());
	if (mf)
	{
		int err;
		err = mf->SetID3v2Buffer(GetBuffer(), GetID3v2Length(), ID3V2_VERSION);
		if (err)		// setting new buffer failed
		{
			delete mf;
			mf = NULL;	
		}
	}
	return(mf);
}

void CMP3BinFrame::operator = (CMP3Frame &mf)
{
	CMP3BinFrame *mp3f = (CMP3BinFrame *)&mf;
	SetID3v2Buffer(mp3f->GetBuffer(), mp3f->GetID3v2Length(), ID3V2_VERSION);
}

BOOL CMP3BinFrame::operator == (CMP3Frame &mf)
{
	// 1. make sure the IDs of the frame are the same:
	if (GetID() != mf.GetID()) return(FALSE);

	CMP3BinFrame *mp3f = (CMP3BinFrame *)&mf;
	// 2. compare the size of the Buffers
	if (GetID3v2Length() != mp3f->GetID3v2Length()) return(FALSE);
	// 3. compare the buffers
	if (memcmp(GetBuffer(), mp3f->GetBuffer(), GetID3v2Length()) != 0) return(FALSE);

	return(TRUE);
}

// buffer is copied to the internal buffer. expand the buffer if new one is
// bigger
int CMP3BinFrame::SetID3v2Buffer(LPBUF buf, int length, int version)
{
	if (length>GetID3v2Length())		// do I have to allacte a new one?
	{
		LPBUF nbuf;
		nbuf = (LPBUF)realloc(Buffer, length);
		if (nbuf==NULL)
		{
			return(ID3V2_ERR_NOMEM);
		}
		// new mem is OK
		Buffer = nbuf;
		Length = length;
	}

	// memory is OK for copying
	memcpy(Buffer, buf, length);
	return(ID3V2_ERR_OK);
}

// return the length for a ID3v2 encoded Buffer
int CMP3BinFrame::GetID3v2Length()
{
	return(Length);
};

int CMP3BinFrame::GetID3v2Buffer(LPBUF buf, int size)
{
	int ret=ID3V2_ERR_BUFTOOSMALL;
	if (GetID3v2Length() <= size)
	{
		memcpy(buf, Buffer, size);
		ret=ID3V2_ERR_OK;
	}
	return(ret);
};


// append information to the tstring str
void CMP3BinFrame::appendInfo(tstring& str, int max)	// get information about the frame
{
	str += _T("size=");
	TCHAR buf[30];
	_itot(Length, buf, 10);
	str += buf;
	return;
}

BOOL CMP3BinFrame::IsDefault()
{
	return(GetID3v2Length() == 0);	// If buffer is empty, the frame is empty!
}

/***********************************************************/
/***** Methods for the tstring-Frame ************************/
CMP3StringFrame::CMP3StringFrame(int ID) : CMP3Frame(ID)
{
	static const tstring str = _T("");
	Initialize(str);
}

CMP3StringFrame::CMP3StringFrame(int ID, const tstring& str) : CMP3Frame(ID)
{
	Initialize(str);
}

void CMP3StringFrame::Initialize(const tstring& str)
{
	// initialize the locale
	locale = ('e'<<16)+('n'<<8)+'g';

	// now the tstring-specific setup
	int newsize = ID3Frames[GetID()].StrSize;

	SetSize(newsize);
	// fill in every tstring
	for (int i=0; i<newsize; i++)
	{
		if (i == DefaultString())
			SetString(str);
		else
			SetString(_T(""), i);
	}
}

CMP3StringFrame::~CMP3StringFrame()
{
}

CMP3Frame *CMP3StringFrame::Clone()
{
	CMP3StringFrame *nf = new CMP3StringFrame(GetID());
	if (nf)
	{
		CMP3Frame *mf = this;
		*nf = *mf;
	}

	return(nf);
}

void CMP3StringFrame::operator = (CMP3Frame &mf)
{
	// make sure the IDs of the frame are the same:
	if (GetID() != mf.GetID()) return;

	CMP3StringFrame *mp3f = (CMP3StringFrame *)&mf;

	SetSize(mp3f->GetSize());

	// now Clone the strings:
	for (int i=0; i<GetSize(); i++)
	{
		SetString(mp3f->GetString(i),i);	// copy it
	}
	// copy the locale-info
	SetLocale(mp3f->GetLocale());
}

BOOL CMP3StringFrame::operator == (CMP3Frame &mf)
{
	// 1. make sure the IDs of the frame are the same:
	if (GetID() != mf.GetID()) return(FALSE);
	CMP3StringFrame *mp3f = (CMP3StringFrame *)&mf;
	// 2. check if the number of strings are equal
	if (GetSize() != mp3f->GetSize()) return(FALSE);
	// 3. check if the strings are all equal; case sensitive!
	for (int i=0; i<GetSize(); i++)
	{
		if (GetString(i).compare(mp3f->GetString(i)) != 0) 
			return(FALSE);
	}
	return(TRUE);
}

// append information to the tstring str
void CMP3StringFrame::appendInfo(tstring& str, int max)	// get information about the frame
{
	str.append(GetString(), 0, max);
	return;
}

BOOL CMP3StringFrame::IsDefault()
{
	// check, if all strings are not Empty
	for (int i=0; i<GetSize(); i++)
	{
		if (!GetString(i).empty())
			// I found a non-empty tstring, so it is no default value
			return(FALSE);
	}
	return(TRUE);
}

/************************** ID3v2 routines *****************************/

/* scan the Buffer for the strings. the Buffer is organized as
 * char encoding[1];
 * [char language[3]; ]
 * {tstring '\0'}
 * tstring
 *
 * since the last tstring hasn't got a terminating \0, I copy the
 * Buffer in a temporary one
 */
int CMP3StringFrame::SetID3v2Buffer(LPBUF Buffer, int length, int version)
{
	int offset = 1;	// start with the second char; first is the encoding

	LPBUF buf;
	buf = (LPBUF)alloca(length+sizeof(WCHAR));
	if (buf == NULL) return(ID3V2_ERR_NOMEM);	// mem-error
	memcpy(buf, Buffer, length);
	buf[length] = buf[length+1] = 0;	// add terminating zero; two, because of unicode

	// ## Erreur avec Winamp lorsque il vide les frames 
	// la taille est = 1
	// Avant Bug  if (length<=1) return(ID3V2_ERR_BUF);
	if (length < 1) return(ID3V2_ERR_BUF);

	// if locale is required by this Tag:
	if (ID3Frames[GetID()].Lang)
	{
		if (length<=4) return(ID3V2_ERR_BUF);
		SetLocale((buf[offset]<<16) + (buf[offset+1]<<8) + (buf[offset+2]));
		offset += 3;
	}

	// now read the strings
	int stringNr = 0;	// just to count them, and check if the Tag is OK
	tstring str;
	while(offset<length)
	{
		switch(buf[0])		// buf[0] is the type of encoding
		{
			case ID3V2_UNICODE:
			{
				WCHAR bom = *((LPWSTR)&buf[offset]);
				if (bom == 0xFEFF) {	// I can read this directly
					offset += sizeof(WCHAR);
				}
				if (bom == 0xFFFE) {	// wrong byte-order :-(
					offset += sizeof(WCHAR);	// anyway, ignore the BOM
					LPWSTR wstr = (LPWSTR)&buf[offset];
					WCHAR w;
					while ((w=*wstr) != '\0') {
						// swap w-bytes, and write it back
						*wstr++ = (w>>8) + (w&0xFF<<8);
					}
				}
				w2tcpy(str, (LPWSTR)&buf[offset]);
				// adjust the offset
				offset += (wcslen((LPWSTR)&buf[offset])+1)*sizeof(WCHAR);
				break;
			}
			case ID3V2_ANSI:
			{
				a2tcpy(str, (LPSTR)&buf[offset]);
				// adjust the offset
				offset += (strlen((LPSTR)&buf[offset])+1)*sizeof(char);
				break;
			}
			default:
				return(ID3V2_ERR_ENCODING);
		}
		// now, add the string to the structure
		if (stringNr < GetSize())
			SetString(str, stringNr);	// replace old one
		else
			AppendString(str);			// append new

		// I have appended a tstring
		stringNr++;
	}

	// remove possible further existing strings.
	if (stringNr < GetSize())
	{
		StringArray.erase(StringArray.begin()+stringNr, StringArray.end());
	}

	return(ID3V2_ERR_OK);
}

// return the size of the buffer
int CMP3StringFrame::GetID3v2Length()
{
	int length = 0;

	// get the length of all the strings, including terminating 0
	for (int nr=0; nr<GetSize(); nr++)
		length += GetString(nr).length() + 1;

	// remove the last zero
	length--;

	// check which encoding is used, and adjust the length
	if (CheckEncoding() == ID3V2_UNICODE) {
		length++;	// add the BOM
		length *= sizeof(WCHAR);
	}

	// check the language
	if (ID3Frames[GetID()].Lang)
		length += 3;	// language field has 3 bytes

	// add a byte for the encoding-field
	length++;

	return(length);
}

// fill in a supplied buffer
int CMP3StringFrame::GetID3v2Buffer(LPBUF buf, int size)
{
	int encoding = CheckEncoding();
	int offset = 0;

	// check, if Buffer is bug enough
	if (size < GetID3v2Length()) return(ID3V2_ERR_BUFTOOSMALL);
	
	// the supplied buffer is big enough, so fill it
	// 1. the type of encoding, UNICODE or ANSI
	buf[offset++] = encoding;

	// 2. the language, if required
	if (ID3Frames[GetID()].Lang)
	{
		buf[offset++] = (GetLocale()>>16) & 0xff;
		buf[offset++] = (GetLocale()>>8) & 0xff;
		buf[offset++] = (GetLocale()) & 0xff;
	}

//	int slen;
	for (int nr=0; nr<GetSize(); nr++)
	{
		switch (encoding)
		{
			case ID3V2_UNICODE:
			{
				*((LPWSTR)&buf[offset]) = 0xFEFF;	// write unicode BOM
				offset += sizeof(WCHAR);
				t2wcpy((LPWSTR)&buf[offset], GetString(nr));
				offset += (wcslen((LPWSTR)&buf[offset])+1) * sizeof(WCHAR);
				break;
			}
			case ID3V2_ANSI:
			{
				t2acpy((LPSTR)&buf[offset], GetString(nr));
				offset += (strlen((LPSTR)&buf[offset])+1) * sizeof(char);
				break;
			}
		}
	}

	return(ID3V2_ERR_OK);
}

int CMP3StringFrame::CheckEncoding()
{
	// if one tstring requires unicode, then return ID3V2_UNICODE
	for (int nr=0; nr<GetSize(); nr++)
	{
		if (!canUseAnsi(GetString(nr)))
			return(ID3V2_UNICODE);
	}
	// all Strings are ANSI-compatible
	return(ID3V2_ANSI);
}

/*************************** tstring-specific addons *********************/


int CMP3StringFrame::DefaultString()
{
	switch(GetID()) {
	case ID3TAG_USLT:	return(1);
	case ID3TAG_COMM:	return(1);
	case ID3TAG_WXXX:	return(1);
	default:			return(0);
	}
}
