/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
// Misc Functions, no classes

#ifndef _MP3FRAME_H
#define _MP3FRAME_H 1

#include "priv.h"
#include "ID3V2.h"
#include "ID3V2Fields.h"
#include "StringConverter.h"

#ifndef LPBUF
typedef unsigned char * LPBUF;
#endif

class CMP3Frame
{
public:

	CMP3Frame(int ID=ID3TAG_TXXX, int SubID=-1);
	virtual ~CMP3Frame();

// some fundamental function to access the data of a frame
	int GetID() {return(FrameID);}
	int getSubID() {return(FrameSubID);}	// used to identify different frametypes, e.g. the APIC-frame
	bool isSupported() {return(ID3Frames[FrameID].Supported);}

	int GetType() {return(ID3Frames[FrameID].Type);}

	// Just to get a copy of the current Frame
	virtual CMP3Frame *Clone() = 0;
	virtual void operator = (CMP3Frame &mp3f) =0;
	virtual BOOL operator == (CMP3Frame &mp3f)=0;
	virtual BOOL operator != (CMP3Frame &mp3f);

// Some functions to do the basic ID3v2 Handling
	virtual int SetID3v2Buffer(LPBUF buf, int length, int version)=0;	// buffer is initialized
	virtual int GetID3v2Length()=0;								// return the size of the buffer
	virtual int GetID3v2Buffer(LPBUF buf, int size)=0;	// fill in a supplied buffer

	virtual void appendInfo(tstring& str, int max);	// get information about the frame

	virtual BOOL IsDefault()=0;

protected:
	void setSubID(int sid) {FrameSubID = sid;}

private:
	int FrameID;		/* Number of Identifier of this frame in Array ID3Frames*/
	int FrameSubID;	/* The Sub-ID*/
};

// The standard binary class
class CMP3BinFrame: public CMP3Frame
{
public:
	CMP3BinFrame(int ID);
	~CMP3BinFrame();

	// Just to get a copy of the current Frame
	CMP3Frame *Clone();
	void operator = (CMP3Frame &mp3f);
	BOOL operator == (CMP3Frame &mp3f);

// Some functions to do the basic ID3v2 Handling
	int SetID3v2Buffer(LPBUF buf, int length, int version);	// buffer is initialized
	int GetID3v2Length();								// return the size of the buffer
	int GetID3v2Buffer(LPBUF buf, int size);	// fill in a supplied buffer

	void appendInfo(tstring& str, int max);	// get information about the frame

	BOOL IsDefault();

// for this Frame only:
public:
	const LPBUF GetBuffer() {return((const LPBUF)Buffer);}

private:
	int   Length;		/* length of the Buffer */
	LPBUF Buffer;		/* the actual content of the frame
							 * the buffer is provided by the ID3v2 functions */
};

/* define a tstring-subclass
 *
 * This class is based on a unicode tstring stored in the buffer.
 * any normal tstring is stored in a sepearate buffer
 */

// CMP3MStringFrame is a frame for storing multiple strings, seperated by a '\0'.
class CMP3StringFrame: public CMP3Frame
{
private:

public:
// overwritten Methods:
	CMP3StringFrame(int ID);
	CMP3StringFrame(int ID, const tstring& str);
	~CMP3StringFrame();

	int SetID3v2Buffer(LPBUF buf, int length, int version);	// buffer is initialized
	int GetID3v2Length();								// return the size of the buffer
	int GetID3v2Buffer(LPBUF buf, int size);	// fill in a supplied buffer
	CMP3Frame *Clone();
	void appendInfo(tstring& str, int max);	// get information about the frame
	BOOL IsDefault();
	void operator = (CMP3Frame &mp3f);
	BOOL operator == (CMP3Frame &mp3f);


// New Methods
	tstring &GetString() {return(GetString(DefaultString()));}
	tstring &GetString(int nr)	{return(StringArray[nr]);}

	void  SetString(const tstring  &str)			{SetString(str, DefaultString());}
	void  SetString(const tstring  &str, int nr)	{StringArray[nr].assign(str);}

	void  AppendString(const tstring &tstr)		{StringArray.push_back(tstr);}

	int	GetSize() {return(StringArray.size());}
	void	SetSize(int size) {StringArray.resize(size);}	// set the size of the array

	int	GetLocale() {return(locale);}
	void	SetLocale(int loc) {if (ID3Frames[GetID()].Lang) locale=loc;}

private:
	int   DefaultString();					// give the number of the default tstring
	void  Initialize(const tstring &tstr);		// called from the construtor
	int	CheckEncoding();					// check which encoding can be used

private:
	StrArray StringArray;	// Array with the strings
	int locale;					// the language of these strings
};

// Function to allocate a frame
extern CMP3Frame *CreateMP3Frame(int fid, int sid=-1);


#endif
