/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
// Misc Functions, no classes

#ifndef _MP3INFO_H
#define _MP3INFO_H

#include "priv.h"
#include "ID3V2.h"
#include "ID3V1.h"
#include "definition.h"
#include "MP3Frame.h"
#include "StringConverter.h"

#define VERSION_ID3NO 0x0
#define VERSION_ID3V1 0x1
#define VERSION_ID3V2 0x2

/***************************************************************************
 * The big class CMP3Info
 ***************************************************************************/

class CMP3Info
{
/*** Attributes ***/
private:
	typedef vector<CMP3Frame *> FArray;

	struct mpg123info m_CoreMP3Info;
	struct ID3V1MP3Info *m_id3v1;
	
	int m_tagVersion;		// 0=no Tag, 1=ID3V1, 2=ID3V2, 3=ID3V1+ID3V2
	bool m_changed;			// Is the Tag changed?
	bool m_ReadOnly;		// Is the File readonly?
	FArray m_FrameArray;	// here are the Frames stored
	tstring m_Filename;		// the name of the file
	int	m_mp3Size;			// the size of the MP3-data: filesize - ID3v2-size + 128 (if fSaveID3V1 is set)
	int	m_diffSize;			// the difference of the size. this is filesize-TSIZ


/*** Methods ***/
public:
	CMP3Info(tstring& file, bool scanfile=true);	// on creation, load the info
	~CMP3Info();

	void SetFileEZ3(EZ3HEADERMP3INFO *headerfile);  // EZPlaylist --> CMP3Info
	void GetFileEZ3(EZ3HEADERMP3INFO *headerfile);  // CMP3Info --> EZPlaylist write
	
	bool Changed()			
		{return(m_changed);}	// only readable;
	
	bool HaveTag()			
		{return(m_tagVersion!=0);}	// check if there is a TAG
	
	bool HaveID3v1()
		{ return((m_tagVersion & VERSION_ID3V1) == VERSION_ID3V1); };

	bool HaveID3v2()
		{ return((m_tagVersion & VERSION_ID3V2) == VERSION_ID3V2); };

	int  GetTagVersion()	
		{return(m_tagVersion);}	// return the version of the TAG
	
	void SetTagVersion(int tagVersionNew)	
		{m_tagVersion = tagVersionNew;}	// set the version of the TAG
	
	struct ID3V1MP3Info *GetID3v1() 
		{return (m_id3v1);};

	BOOL SetID3v1(struct ID3V1MP3Info *id3v1New); 
		
	bool IsReadOnly()		
		{return(m_ReadOnly);}
	
	tstring& GetFilename()	
		{return(m_Filename);}
	
	LPCTSTR GetCFilename()	
		{return(m_Filename.c_str());}
	
	int GetFrameCount()		
		{return m_FrameArray.size();}
	
	void SetFilename(LPTSTR NewFilename) 
		{ a2tcpy(m_Filename, NewFilename);};
	
	CMP3Frame *GetIndexFrame(int index) 
		{return(index>=0 && index <m_FrameArray.size()?m_FrameArray[index]:NULL);}
	
	CMP3Frame *GetFrame(int ID, int subID=-1, int start=-1)
		{return(GetIndexFrame(FindFrame(ID, subID, start)));}
	
	int FindFrame(int ID, int subID=-1, int start=-1);		// get Tag after start-Tag
	
	int FindFrame(CMP3Frame *mp3f, int start=-1);	// get Tag after start-Tag
	
	CMP3StringFrame *GetStringFrame(int ID, int start=-1); // get Tag after start-Tag

	void sortFrames();	// sort the frames

	void ParseFilename();

	// These Function manage the basic ID3-stuff
	bool DeleteTag(int tagDelVersion);
	
	// Apply all changes to the file
	int SaveTag();

	// Copy info ID3v1 & ID3v2 mp3infoSrc --> mp3infodest
	void CopyAll(CMP3Info *mp3src);
	
	// Function to manage a single Frame
	bool ReplaceFrame(CMP3Frame *mp3f) {return(AddFrame(mp3f, true));};
	
	// Add a frame. if replace==TRUE, first one is replaced
	bool AddFrame(CMP3Frame *mp3f, bool replace=false);
	
	// delete a frame, if mp3f==NULL, then the first one found
	bool DeleteFrame(int ID, CMP3Frame *mp3f=NULL);

	// this is the main function to change a frame
	bool UpdateFrame(CMP3Frame *mp3f);

	// Functions to access the Core MP3 Info
	void GetMPEGVersionLayer(TCHAR *mpeglayer);

	int GetLayer()		{return(m_CoreMP3Info.lay);}
	int GetBitrate()	{return(m_CoreMP3Info.bitrate_index);}
	int GetFrequency()	{return(m_CoreMP3Info.sampling_frequency);}
	int GetPlayLength() {return(m_CoreMP3Info.length);}
	int GetFileSize()	{return(m_CoreMP3Info.filesize);}
	int GetMode()		{return(m_CoreMP3Info.mode);}
	int GetCopyright()	{return(m_CoreMP3Info.copyright);}
	int GetOriginal()	{return(m_CoreMP3Info.original);}
	int GetEmphasis()	{return(m_CoreMP3Info.emphasis);}
	int GetCRC()		{return(m_CoreMP3Info.error_protection);}
	int GetPrivate()	{return(0);}	//$$ a voir ??????????????????????
	int GetFrames()		{return(m_CoreMP3Info.num_frames);}
	int GetMP3Size()	{return(m_mp3Size);}
	int GetDiffSize()	{return(m_diffSize);}
	bool isVBR()		{return(m_CoreMP3Info.varbitrate!=0);}

private:
	int ReadID3V1();				 // if successful, the info has been read
	int ReadID3V2(int *ID3v2Length); // if successful, the info has been read
	int WriteID3V1();			     // if successful, the info has been read
	int WriteID3V2();		         // if successful, the info has been read
};



#endif
