/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
/* Here are the frames implemented */

#include "priv.h"
#undef ID3V2_CREATE_TAG_ARRAY
#include "MP3PicFrame.h"
//#include "MP3ext.h"
#include <sys/stat.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

CMP3PicFrame::CMP3PicFrame(int ID, int SubID) : CMP3Frame(ID, SubID)
{
	picData = NULL;
	picDataLength = picBufferLength = 0;
}

CMP3PicFrame::~CMP3PicFrame()
{
	if (picData) free(picData);
}

// read a string from the buffer, write it into wstr, and return the
// # of bytes read, incl. zero
static int readString(int encoding, LPBUF buf, tstring& str)
{
	if (encoding == ID3V2_UNICODE) {
		w2tcpy(str, (LPWSTR)buf);
		return((str.length()+1)*sizeof(WCHAR));
	} else {
		// read an ansi-string
		a2tcpy(str, (LPSTR)buf);
		return((str.length()+1)*sizeof(CHAR));
	}
}

/*
 * scan the buffer, and read in the values of the pic-frame
 *      Attached picture   "PIC"
 *      Frame size         $xx xx xx
 *      Text encoding      $xx
 *      Image format       $xx xx xx
 *      Picture type       $xx
 *      Description        <textstring> $00 (00)
 *      Picture data       <binary data>
 */
int CMP3PicFrame::SetID3v2Buffer(LPBUF buf, int length, int version)
{
	if (length < 6) return(ID3V2_ERR_CORRUPT);	// at least 6 bytes...

	int headoffset = 1;

	// get the image-type
	if (version == 2) {
		headoffset+=3;	// skip format
	} else {
		// skip string
		headoffset += strlen((LPCSTR)&buf[1]) + 1;
	}

	setSubID(buf[headoffset]);
	buf[headoffset++] = 0;	// zero, to terminate the id3v2.2 format-string....

	headoffset+= readString(buf[0], &buf[headoffset], description);
	LPBUF data = buf + headoffset;
	int dlen = length - headoffset;

	LPCSTR type = (LPCSTR)&buf[1];
	if (version == 2)
		if (strnicmp(type, "ICO", 3) == 0) type = "image/ico";

	tstring ttype;
	a2tcpy(ttype, type);
	return(setData(ttype.c_str(), data, dlen));
}

// return the length for a ID3v2 encoded Buffer
int CMP3PicFrame::GetID3v2Length()
{
	int Length;
	// the header
	Length = 3;
	Length += picFormat.length();
	// the descrition-string
	int slen = description.length() + 1;	// incl. trailing \0
	if (!canUseAnsi(description))
		slen *= sizeof(WCHAR);
	Length += slen;

	// add the length of the data:
	Length += getPicDataLength();
	return(Length);
}

int CMP3PicFrame::GetID3v2Buffer(LPBUF buf, int size)
{
	int ret=ID3V2_ERR_BUFTOOSMALL;
	if (GetID3v2Length() <= size)
	{
		int offset = 1;	// start here
		// first the picFormat:
		t2acpy((LPSTR)&buf[offset], picFormat);
		offset += strlen((LPSTR)&buf[offset]) + 1;
		// the picType:
		buf[offset++] = getPicType();
		LPBUF data = buf+offset;	// starting of the data, in case of an empty string
		// now the description:
		if (canUseAnsi(description))
		{
			buf[0] = ID3V2_ANSI;
			t2acpy((LPSTR)&buf[offset], description);
			data += (strlen((LPSTR)&buf[offset])+1)*sizeof(CHAR);
		}
		else
		{	// use unicode
			buf[0] = ID3V2_UNICODE;
			t2wcpy((LPWSTR)&buf[offset], description);
			data += (wcslen((LPWSTR)&buf[offset])+1)*sizeof(WCHAR);
		}
		// write the data:
		memcpy(data, picData, picDataLength);
		ret=ID3V2_ERR_OK;
	}
	return(ret);
};

// append information to the string str
void CMP3PicFrame::appendInfo(tstring& str, int max)	// get information about the frame
{
	TCHAR buf[300];
	_stprintf(buf, _T("type=%d format=%s size=%d"), getPicType(), getPicFormat().c_str(), getPicDataLength());
	str.append(buf);
	return;
}


BOOL CMP3PicFrame::IsDefault()
{
	return(picDataLength == 0);	// If buffer is empty, the frame is empty!
}

CMP3Frame *CMP3PicFrame::Clone()
{
	CMP3PicFrame *pf = new CMP3PicFrame(GetID(), getSubID());
	*pf = *(CMP3Frame *)this;
	return((CMP3Frame *)pf);
}

void CMP3PicFrame::operator = (CMP3Frame &mf)
{
	CMP3PicFrame *mp3f = (CMP3PicFrame *)&mf;
	setDescription(mp3f->description);
	setSubID(mp3f->getPicType());
	setData(mp3f->getPicFormat().c_str(), mp3f->getPicData(), mp3f->getPicDataLength());
}

BOOL CMP3PicFrame::operator == (CMP3Frame &mf)
{
	// 1. make sure the IDs of the frame are the same:
	if (GetID() != mf.GetID()) return(FALSE);

	CMP3PicFrame *mp3f = (CMP3PicFrame *)&mf;
	if (getPicType() != mp3f->getPicType()) return(false);	// different type
	if (getPicFormat().compare(mp3f->getPicFormat()) != 0) return(false);	// different format
	if (getPicDataLength() != mp3f->getPicDataLength()) return(false);
	// compare the buffer:
	if (memcmp(getPicData(), mp3f->getPicData(), getPicDataLength()) != 0) return(false);
	return(true);
}

// set the actual picture data
int CMP3PicFrame::setData(LPCTSTR format, LPBUF data, int dlen)
{
	// set the data:
	if (dlen>picBufferLength)		// do I have to allacte a new one?
	{
		LPBUF nbuf;
		nbuf = (LPBUF)realloc(picData, dlen);
		if (nbuf==NULL)
		{
			return(ID3V2_ERR_NOMEM);
		}
		// new mem is OK
		picData = nbuf;
		picBufferLength = dlen;
	}

	// memory is OK for copying
	memcpy(picData, data, dlen);
	picDataLength = dlen;
	// set the format	
	picFormat = format;

	return(ID3V2_ERR_OK);
}

struct {
	LPCTSTR mimetype;
	LPCTSTR extension;
} MimeTable[] =
{
	{_T("image/jpeg"), _T("jpg")}
};
#define NUMMIMETYPES NUMELEMENTS(MimeTable)

/**
 * get the mime-type from the filename.
 */
static void getMimeTypeFromFilename(LPCTSTR file, LPTSTR mime)
{
	*mime='\0';
	TCHAR extension[20];
	_tsplitpath(file, NULL, NULL, NULL, extension);
	_tcscpy(extension, extension+1);
	for (int i=0; i<NUMMIMETYPES; i++) {
		if (_tcsicmp(MimeTable[i].extension, extension) == 0)
		{
			_tcscpy(mime, MimeTable[i].mimetype);
			break;
		}
	}
	if (*mime == '\0')
		_stprintf(mime, _T("image/%s"), extension);	// default mime-type: "image/<filename-extension>"
}


/**
 * Load the file into the frame, and set the data. return true if successful.
 * This method loads the data, and sets the appropiate mime-type.
 */
bool CMP3PicFrame::loadFile(LPCTSTR filename)
{
	struct _stat sinfo;
	int err = _tstat(filename, &sinfo);
	if (err == 0)	// on error, do nothing
	{
		/* get the size of the file */
		int flen = sinfo.st_size;
		TCHAR mime[200];
		LPBUF buffer=NULL;
		// load the file
		if ((buffer = (LPBUF)malloc(flen)) != NULL) {
			FILE *fh;
			if ((fh = _tfopen(filename, _T("rb"))) != NULL) {
				int bytes = fread(buffer, 1, flen, fh);
				fclose(fh);
				if (bytes == flen) {
					getMimeTypeFromFilename(filename, mime);
					setData(mime, buffer, flen);
				} else
					DE(BUG("Failed to read image\n"));
			} else
				DE(BUG("Failed to open image\n"));
			free(buffer);
		} else
			DE(BUG("Failed to allocate mem for image\n"));
	} else
		DE(BUG("Error finding file\n"));

	return true;
}


/*
// set new type; returns old type
int CMP3PicFrame::setType(int newtype)
{
	int oldtype = getPicType();
	picType = newtype;
	return(oldtype);
}
/**/
