
#include "priv.h"

/*  www.codeguru.com
 * It is possible to add a button to the SHBrowseForFolder dialog 
 * (or indeed any other control desired). This is achieved by dynamically 
 * subclassing the dialog - ie writing a new WindowProc for the dialog to handle 
 * the messages you're interested in. You pass all other messages back to the 
 * old WindowProc. 
 */ 


// internal function 
int CALLBACK BrowseRecurseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData);
int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData);
 
extern BOOL gbCheckRecurseDir;


// Control ID of our button checkbox (recurse subdirectories)
#define ID_CHECK_RECURSE 200 

// Window commands to pass to our WindowProc 
#define WM_BROWSE_CREATECHECKBOX (WM_USER+100) 

// Control ID of the tree control in the dialog 
#define BROWSETREECTRLID 0x3741 

// internal function
LRESULT CALLBACK BrowseFolderWindowProc(HWND hwnd,UINT msg, WPARAM wParam, LPARAM lParam);
 



int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData) 
{ 
	if(uMsg == BFFM_INITIALIZED) 
	{ 
		// Set the current directory
		SendMessage(hwnd, BFFM_SETSELECTION, TRUE, pData);
	} 
	
	return 0; 
}


//=================================================================== 
// Function name : BrowseRecurseCallbackProc 
// Description : Callback function for Browse dialog 
// Entry : 
// : HWND hwnd 
// : UINT uMsg 
// : LPARAM lp 
// : LPARAM pData 
// 
// Return type : int 
//======================================================= 
int CALLBACK BrowseRecurseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData) 
{ 
	if(uMsg == BFFM_INITIALIZED) 
	{ 
		// Set the current directory
		SendMessage(hwnd, BFFM_SETSELECTION, TRUE, pData);
		
		// Save Old Window proc 
		WNDPROC OldWndProc = (WNDPROC)SetWindowLong(hwnd,GWL_WNDPROC, (LONG)BrowseFolderWindowProc ); 
		SetWindowLong(hwnd, GWL_USERDATA, (LONG)OldWndProc ); 
		
		// Add a new checkbox
		SendMessage( hwnd, WM_BROWSE_CREATECHECKBOX, 0, (LPARAM)pData ); 
	} 
	
	return 0; 
} 

//============================================================ 
// Function name : BrowseFolderWindowProc 
// Description : Window procedure for browse dialog 
// Entry : 
// : HWND hwnd 
// : UINT msg 
// : WPARAM wParam 
// : LPARAM lParam 
// 
// Return type : LRESULT 
//================================================================ 
LRESULT CALLBACK BrowseFolderWindowProc(HWND hwnd,UINT msg, WPARAM wParam, LPARAM lParam) 
{ 
	static WNDPROC OldWndProc = NULL; 
	static bool bGotWndProc = false; 
	static bool bButtonCreated = false; 

	switch ( msg ) 
	{ 
		case WM_BROWSE_CREATECHECKBOX: 
		{ 
			if ( bButtonCreated == false ) 
			{ 
			
				HWND hWndCheckbox = CreateWindow(
					"BUTTON", 
					TEXT_SHBROWSEFOLDER_CHECKBOX, 
					BS_AUTOCHECKBOX | WS_CHILDWINDOW | WS_VISIBLE | WS_TABSTOP | BS_NOTIFY , 
					12,						// x
					8,						// y
					280,					// window width
					20,						// window height
					hwnd,					// Handle to parent or window
					(HMENU)ID_CHECK_RECURSE,	// Menu
					GetModuleHandle(NULL), 
					NULL);

			
				CheckDlgButton(hwnd, ID_CHECK_RECURSE, BST_CHECKED); 
				gbCheckRecurseDir = TRUE;
			} 
		} 
		break; 
	
	
		case WM_COMMAND: 
		{ 
			switch ( LOWORD(wParam) ) 
			{ 
				case ID_CHECK_RECURSE: 
				{ 
					if (IsDlgButtonChecked(hwnd,ID_CHECK_RECURSE))	
						gbCheckRecurseDir = TRUE;	
					else
						gbCheckRecurseDir = FALSE;	

				} 
			} 
		} 
		break; 
	} 


	// Call original window procedure 
	if ( bGotWndProc == false ) 
	{ 
		OldWndProc = (WNDPROC) GetWindowLong( hwnd, GWL_USERDATA ); 
		if ( OldWndProc != NULL ) 
			bGotWndProc = true; 
		else 
			OldWndProc = (WNDPROC) GetWindowLong( hwnd, GWL_WNDPROC ); 
	} 
	
#ifdef STRICT 
	return CallWindowProc((WNDPROC)OldWndProc, hwnd, msg, wParam, lParam); 
#else 
	return CallWindowProc((FARPROC)OldWndProc, hwnd, msg, wParam, lParam); 
#endif 
	
} 









