/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
// Misc Functions, no classes

#ifndef _STRINGCONVERTER_H
#define _STRINGCONVERTER_H

/***************************************************************************/
/* the structure to access the ID3V1 Tag */

#ifdef __cplusplus
#include <string>
using namespace std;
#endif

#ifndef CGLOBAL
#ifdef __cplusplus
#define CGLOBAL extern "C"
#else
#define CGLOBAL extern
#endif
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif



#ifdef __cplusplus

extern string *wstr2str(const wstring& wstr, string& str);
extern string *wstr2str(const wstring& wstr);
extern wstring *str2wstr(const string& str, wstring& wstr);
extern wstring *str2wstr(const string&   str);
extern bool canUseAnsi(tstring& tstr);	// check if it is possible to convert without error
extern void loadString(tstring& str, LPCTSTR file);	// load contents of file into the tstring
extern void saveString(tstring& str, LPCTSTR file);	// save the tstring into a file
extern void copyStringToClipboard(tstring& str);

#endif

/* These are conversion-routines from our internal used tstring-class
 * into a hard-coded ASCII-char, or WCHAR-array
 * Note: At the moment the n-variant is not supported completely by a2t & w2t
 */

#ifdef UNICODE
#define t2ancpy(a, tstr, n) WideCharToMultiByte(CP_ACP, 0, tstr.c_str(), -1, a, n, NULL, NULL)
#define t2acpy(a, tstr)     t2ancpy(a,tstr,tstr.length()); (a)[tstr.length()]='\0'
#define a2tncpy(tstr, a, n) str2wstr(a, tstr)
#define a2tcpy(tstr, a)     str2wstr(a, tstr)
#define t2wncpy(w, tstr, n) wcsncpy(w, tstr.c_str(), n)
#define t2wcpy(w, tstr)     wcscpy(w, tstr.c_str())
#define w2tncpy(tstr, w, n) tstr.assign(w, n)
#define w2tcpy(tstr, w)	    tstr.assign(w)
#else	// no unicode
#define t2ancpy(a, tstr, n) strncpy(a, tstr.c_str(), n)
#define t2acpy(a, tstr)     strcpy(a, tstr.c_str())
#define a2tncpy(tstr, a, n) tstr.assign(a, n)
#define a2tcpy(tstr, a)	    tstr.assign(a)
#define t2wncpy(w, tstr, n) MultiByteToWideChar(CP_ACP, 0, tstr.c_str(), -1, w, n)
#define t2wcpy(w, tstr)     t2wncpy(w, tstr, tstr.length()); (w)[tstr.length()]=L'\0'
#define w2tncpy(tstr, w, n) wstr2str(w, tstr)
#define w2tcpy(tstr, w)     wstr2str(w, tstr)
#endif

#endif


