#include "priv.h"
#include <windows.h>

#include "gen.h"
#include "definition.h"
#include "util.h"		
#include "MP3Info.h"	// Use only for MatcTime function
#include "resource.h"

// external declaration
extern winampGeneralPurposePlugin plugin;
extern INITSTRUCT gParam;

extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);



char    *CSSFontNameA[4]   = { "Arial, Helvetica, sans-serif",  "\"Times New Roman\", Times, serif", "\"Courier New\", Courier, mono", "Verdana, Arial, Helvetica, sans-serif"};
wchar_t *CSSFontNameW[4]   = {L"Arial, Helvetica, sans-serif", L"\"Times New Roman\", Times, serif",L"\"Courier New\", Courier, mono",L"Verdana, Arial, Helvetica, sans-serif"};
char    *CSSFontStyleA[2]  = {  "normal", "italic"};
wchar_t *CSSFontStyleW[2]  = { L"normal",L"italic"};
char    *CSSFontWeightA[2] = {  "normal", "bold"};
wchar_t *CSSFontWeightW[2] = { L"normal",L"bold"};
char    *CSSFontSizeA[7]   = {  "xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large"};
wchar_t *CSSFontSizeW[7]   = { L"xx-small",L"x-small",L"small",L"medium",L"large",L"x-large",L"xx-large"};


// Shortcut Hook procedure
// stuff to install the accelerator Main Window EZPlaylist
static LRESULT CALLBACK MyHookProcMain(int nCode, WPARAM wParam, LPARAM lParam);
static HACCEL gMainAccel = NULL;
static HHOOK gOldMainMsgHook = NULL;
HWND ghWndMainDlg = NULL;

static LRESULT CALLBACK MyHookProcEditTag(int nCode, WPARAM wParam, LPARAM lParam);
static HACCEL gAccelEditTag = NULL;
static HHOOK gOldMsgHookEditTag = NULL;
HWND ghWndDlgEditTag = NULL;


typedef int (*compfn)(const void*, const void*);
int compare(struct aMatchTime *elem1, struct aMatchTime *elem2);
	
struct aMatchTime
{	
	int nIndex;
	int nLength;
};


/* ********************************************************* */
void SetColumnOrder(int order[])
{
    for (int i = 0; i < COLUMN_MAX; i++)
        gParam.column_order[i] = order[i];
}

/* ********************************************************* */
void GetColumnOrder(int order[])
{
    for (int i = 0; i < COLUMN_MAX; i++)
        order[i] = gParam.column_order[i];
}

/* ********************************************************* */
void SetColumnShown(int shown[])
{
    for (int i = 0; i < COLUMN_MAX; i++)
        gParam.column_shown[i] = shown[i];
}

/* ********************************************************* */
void GetColumnShown(int shown[])
{
    for (int i = 0; i < COLUMN_MAX; i++)
        shown[i] = gParam.column_shown[i];
}

/* ********************************************************* */
void SetColumnWidths(int width[])
{
    for (int i=0; i < COLUMN_MAX; i++)
        gParam.column_width[i] = width[i];
}
/* ********************************************************* */
void GetColumnWidths(int width[])
{
    for (int i=0; i < COLUMN_MAX; i++)
        width[i] = gParam.column_width[i];
}

/* ********************************************************* */
void SetSortColumn(int column)
{
    gParam.sort_reverse = (column < 0) ? TRUE : FALSE;
    gParam.sort_column = abs(column) ; 
}

/* ********************************************************* */
int GetSortColumn(void)
{
    int column = gParam.sort_column ; 

    if (gParam.sort_reverse)
        column = -(column);

    return column;
}







// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=




















/* ************************************************************************ */
void SetHookAccelMain(BOOL bActive, int idAccel, HWND hDlg)
{
	if (bActive)
	{
		if (gOldMainMsgHook == NULL)
		{
			ghWndMainDlg = hDlg;
			gOldMainMsgHook = SetWindowsHookEx(WH_MSGFILTER, (HOOKPROC)MyHookProcMain, NULL, GetCurrentThreadId());
			gMainAccel = LoadAccelerators(plugin.hDllInstance, MAKEINTRESOURCE(idAccel));
		}
	}
	else
	{
		UnhookWindowsHookEx(gOldMainMsgHook);
		DestroyAcceleratorTable(gMainAccel);
	
		gOldMainMsgHook = NULL;
		gMainAccel = NULL;
		ghWndMainDlg = NULL;
	}
}

/* ******************************************************* */
// WH_MSGFILTER hook procedure / Shortcut Accelerator
static LRESULT CALLBACK MyHookProcMain(int nCode, WPARAM wParam, LPARAM lParam)
{
	//$$ if (nCode == MSGF_DIALOGBOX) 
	{
		if(ghWndMainDlg && gMainAccel)
			if (TranslateAccelerator(ghWndMainDlg, gMainAccel, (LPMSG)lParam))
				return(TRUE);	
	}

	// default: call the next hook
	return CallNextHookEx(gOldMainMsgHook, nCode, wParam, lParam);
}



/* ************************************************************************ */
void SetHookAccelEditTag(BOOL bActive, int idAccel, HWND hDlg)
{
	if (bActive)
	{
		if (gOldMsgHookEditTag == NULL)
		{
			ghWndDlgEditTag = hDlg;
			gOldMsgHookEditTag = SetWindowsHookEx(WH_MSGFILTER, (HOOKPROC)MyHookProcEditTag, NULL, GetCurrentThreadId());
			gAccelEditTag = LoadAccelerators(plugin.hDllInstance, MAKEINTRESOURCE(idAccel));
		}
	}
	else
	{
		UnhookWindowsHookEx(gOldMsgHookEditTag);
		DestroyAcceleratorTable(gAccelEditTag);
	
		gOldMsgHookEditTag = NULL;
		gAccelEditTag = NULL;
		ghWndDlgEditTag = NULL;
	}
}

/* ******************************************************* */
// WH_MSGFILTER hook procedure / Shortcut Accelerator
static LRESULT CALLBACK MyHookProcEditTag(int nCode, WPARAM wParam, LPARAM lParam)
{
	if (nCode == MSGF_DIALOGBOX) 
	{
		if(ghWndDlgEditTag && gAccelEditTag)
			if (TranslateAccelerator(ghWndDlgEditTag, gAccelEditTag, (LPMSG)lParam))
				return(TRUE);	
	}

	// default: call the next hook
	return CallNextHookEx(gOldMsgHookEditTag, nCode, wParam, lParam);
}



/**************************************************************************** */
// DupString - allocates a copy of a string. 
// lpsz - address of the null-terminated string to copy.  
LPTSTR DupString(LPTSTR lpsz) 	
{     
	int cb = _tcslen(lpsz) + 1; 
    LPTSTR lpszNew = (LPTSTR)GlobalAlloc(LMEM_FIXED, cb);     
	if (lpszNew != NULL) 
        CopyMemory(lpszNew, lpsz, cb);     
	
	return lpszNew; 	
} 


/* *********************************************************** */ 
TCHAR *GetFormattedDateTime(void) 
{ 
	static TCHAR szTime[64]; 
    int cch = 0; 
 
 	// Date format
	cch = GetDateFormat( 
	        LOCALE_USER_DEFAULT, 
	        0, 
	        NULL,			// current date 
	        _T("yyyy/MM/dd"),   
	        szTime, 
	        sizeof szTime); 
 
    // cch includes null terminator, change it to 
    // a space to separate from time. 
    szTime[ cch - 1 ] = ' '; 
    
 	// Time format
    GetTimeFormat( 
	    LOCALE_USER_DEFAULT, 
	    TIME_NOSECONDS, 
	    NULL,   // use current time 
	    NULL,   // use default format 
	    szTime + cch, 
	    (sizeof szTime) - cch ); 
 
    return szTime; 
} 


/* ************************************************************** */
void  ListboxMoveItem(HWND hwndlist, BOOL bMoveUp)
{
	int nCount, i;
	TCHAR szTmp[40];
	int pos1, pos2;

	nCount = SendMessage(hwndlist, LB_GETCOUNT, 0, 0);

	i = SendMessage(hwndlist, LB_GETCURSEL, 0, 0);

	if (i == LB_ERR) 
		return;
	if (nCount == 0)
		return;
	if (i <= 0 && bMoveUp == TRUE)
		return;
	if ((i+1) >= nCount && bMoveUp== FALSE)
		return;

	if (bMoveUp == FALSE)
	{
		pos1 = i;	pos2 = i+1;
	}
	else
	{
		pos1 = i;	pos2 = i-1;
	}
	
	// get the string from the listbox
	SendMessage(hwndlist,LB_GETTEXT, pos1, (LPARAM)szTmp);	
	// Delete old Item
	SendMessage(hwndlist,LB_DELETESTRING, pos1, 0);
	// insert Item at new position
	i=pos2;
	if (i >= SendMessage(hwndlist,LB_GETCOUNT, 0, 0)) 
		i = -1;	// if last item, append tstring
	SendMessage(hwndlist,LB_INSERTSTRING, i, (LPARAM)szTmp);
	SendMessage(hwndlist,LB_SETCURSEL, pos2, 0);

}


/* ************************************************************ */
void SetFilter(TCHAR *szFilter)
{
	// Use for OPENFILENAME 
	TCHAR chReplace;
	UINT u;

    chReplace  = szFilter[_tcslen(szFilter) - 1];
    for (u = 0; szFilter[u] != '\0'; u++)
        if (szFilter[u] == chReplace)
            szFilter[u] = '\0';
}

/* ******************************************************** */
void Time2Str(int nSecond, TCHAR *pTimeFrmt, BOOL bShortFormat)
{
	int s, m, h;
	s = m = h = 0;
	
	s = nSecond % 60;
	m = nSecond / 60;
	h = m / 60;
	m = m % 60;

	if (bShortFormat)
	{
		if (h == 0)
			_stprintf(pTimeFrmt, _T("%02d:%02d"), m,s);
		else
			_stprintf(pTimeFrmt, _T("%02d:%02d:%02d"), h, m,s);
	}
	else
	{
		if (h == 0)
			_stprintf(pTimeFrmt, TEXT_SHORTTIME_FRMT, m,s);
		else
			_stprintf(pTimeFrmt, TEXT_FULLTIME_FRMT, h, m,s);
	}
	
}


/* ******************************************************** */
BOOL CheckExtension(tstring& file)
{
	int pos = file.find_last_of('.');
	
	if (pos == tstring::npos) 
		return(false);	// no extension

	tstring& ext = file.substr(pos);
	
	if (_tcsicmp(_T(".mp3"), ext.c_str()) == 0) 
		return(true);
	if (_tcsicmp(_T(".mp2"), ext.c_str()) == 0) 
		return(true);
	
	// not found
	return(false);
}




// ***********************************

/* ******************************************************** */
bool IsStringEmpty(const string& s)
{
	int len = s.length();
	for (int i=0; i<len; i++)
		if (!isspace(s[i])) return(false);
	return true;
}
bool IsStringEmpty(string *s)
{
	if (s==NULL) return(true);		// NULL pointer is empty string
	return(IsStringEmpty(*s));
}

bool IsStringEmpty(wstring *s)
{
	if (s==NULL) return(true);		// NULL pointer is empty string
	return(IsStringEmpty(*s));
}
bool IsStringEmpty(const wstring& s)
{
	int len = s.length();
	for (int i=0; i<len; i++)
		if (!iswspace(s[i])) return(false);
	return true;
}
/* ******************************************************** */
void TrimString(tstring& str)
{
	int begin=0;
	int end=str.length()-1;
	bool changed = false;
	while(_istspace(str[begin])) {changed = true; begin++;}
	// now the end
	while(_istspace(str[end])) {changed = true; end--;}
	if (!changed) return;
	str = str.substr(begin, end-begin+1);
}



/* ******************************************************** */
void MatchTime(HWND hListview, int nLengthMax)
{
	struct aMatchTime  *pArray;
	
	LVITEM lvi; 
	int i, nCount;
	int nLength = 0;
	
	nCount = ListView_GetItemCount(hListview);
	
	pArray = (struct aMatchTime*)malloc(nCount*sizeof(struct aMatchTime));

	if (pArray == NULL)
		return;

	// Copy in array temp Index listview and PlayLength
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_PARAM; 
	
	for (i=0; i<nCount;i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(hListview, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;    // Retreive item data  MP3Info
		
		if (!mp3i) continue;

		pArray[i].nIndex = i;
		pArray[i].nLength = mp3i->GetPlayLength();		
	}

	// Order array
	// MSDN April 1999, qsort:Article ID: Q73853 
	qsort((void *)pArray, nCount, sizeof(struct aMatchTime), (compfn)compare);


	// Selected songs from match Time array
	for (i=0, nLength=0; i<nCount;i++)
	{
		int idx;
		
		idx = pArray[i].nIndex;
		nLength += pArray[i].nLength;

		if (nLength < nLengthMax)
		{
			// Selected the song Match Time
			ZeroMemory(&lvi, sizeof(lvi)); 	
			lvi.mask      = LVIF_STATE; 
			lvi.stateMask = LVIS_SELECTED; 
			lvi.iItem     = idx;
			ListView_SetItemState(hListview, idx, LVIS_SELECTED , LVIS_SELECTED);
		}
		else
			i = nCount;  // force to exit loop
		
	}

	ListView_EnsureVisible(hListview, pArray[0].nIndex, TRUE);

	free(pArray); 

	SetFocus(hListview);
}

/* ******************************************************** */
int compare(struct aMatchTime *elem1, struct aMatchTime *elem2)
{
	if ( elem1->nLength < elem2->nLength)
		return -1;

	else if (elem1->nLength > elem2->nLength)
		return 1;

	else
		return 0;
}
