// Winamp general purpose plug-in mini-SDK
// Copyright (C) 1997, Justin Frankel/Nullsoft

#include "priv.h"
#include <process.h>
#include <commctrl.h>
#include <prsht.h>

#include "gen.h"
#include "resource.h"
#include "definition.h"
#include "Dlg.h"
#include "Volume.h"


// internal function
int init();
void quit();
void config();
void config_write();
void config_read();
void config_getinifn(TCHAR *ini_file);
void do_icons();
void winamp_previous_track(HWND hwnd);
void winamp_play_pause(HWND hwnd);
void winamp_stop(HWND hwnd);
void winamp_next_track(HWND hwnd);
void winamp_stop(HWND hwnd);
void winamp_openfile(HWND hwnd);
void winamp_volume_up(HWND hwnd);
void winamp_volume_down(HWND hwnd);

BOOL ezplaylist_init(HANDLE hInstance);

BOOL CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
LRESULT CALLBACK SysTrayWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);


// external systray.cpp
BOOL systray_add(HWND hwnd, UINT uID, HICON hIcon, LPTSTR lpszTip);
BOOL systray_del(HWND hwnd, UINT uID);



// Setting LINK Check library
//
// Standart library
// kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib advapi32.lib ole32.lib oleaut32.lib uuid.lib
//
// Add specific 
//    shell32.lib winmm.lib comctl32.lib zlibd.lib version.lib 




// Declaration variables
#define IDM_SYSTRAY_EXTPLAYLIST     WM_USER + 100
#define IDM_SYSTRAY_PREVIOUSTRACK   WM_USER + 101
#define IDM_SYSTRAY_PLAYPAUSE       WM_USER + 102
#define IDM_SYSTRAY_STOP            WM_USER + 103
#define IDM_SYSTRAY_NEXTTRACK       WM_USER + 104
#define IDM_SYSTRAY_LOADFILE        WM_USER + 105
#define IDM_SYSTRAY_VOLUME          WM_USER + 106

#define NUM_ICONSYSTRAY 8

#define ENABLE_PREV 1
#define ENABLE_PLAY 2
#define ENABLE_STOP 4
#define ENABLE_NEXT 8
#define ENABLE_EJECT 16
#define ENABLE_VOLUP 32
#define ENABLE_EZLIST 64
#define ENABLE_EZMENU 128



#ifdef STRICT 
 WNDPROC lpWndProcOld;	
#else 
 FARPROC lpWndProcOld;		
#endif 

// Global declaration
HWND ghWndMain = NULL;

CDlgPlaylist *dlg = NULL;
TCHAR szAppName[] = _T("EZ-PlayList");


winampGeneralPurposePlugin plugin =
{
	GPPHDR_VER,
	_T(""),
	init,
    config,
	quit,
};


// Internal  declaration
int config_enabled=1;

HICON Icons[NUM_ICONSYSTRAY];
TCHAR *tips[NUM_ICONSYSTRAY] = 
{
	TEXT_TIPS_EXTPLAYLIST,	
	TEXT_TIPS_PREVIOUSTRACK,
	TEXT_TIPS_PLAYPAUSE,	
	TEXT_TIPS_STOP,			
	TEXT_TIPS_NEXTTRACK,	
	TEXT_TIPS_LOADFILE,		
	TEXT_TIPS_VOLUME,
	TEXT_TIPS_EZMENU
};




/* ************************************************************************ */
//BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
BOOL WINAPI DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	return TRUE;
}

/* ************************************************************************ */
// this is the only exported symbol. returns our main header.
// if you are compiling C++, the extern "C" { is necessary, so we just #ifdef it
extern "C" 
__declspec( dllexport ) winampGeneralPurposePlugin* winampGetGeneralPurposePlugin()
{
	return &plugin;
}


/* ************************************************************************ */
int init()
{
	// /////////////////////////////
	// Read config plugin.ini 
	// /////////////////////////////
	static TCHAR c[MAX_PATH];
	TCHAR filename[MAX_PATH],*p;
	GetModuleFileName(plugin.hDllInstance,filename,sizeof(filename));
	p = filename+lstrlen(filename);
	while (p >= filename && *p != '\\') p--;
	_stprintf((plugin.description=c), _T("%s Plug-In v1.4 (%s)"), szAppName,p+1);

	config_read();
	
	for (int i = 0; i < NUM_ICONSYSTRAY; i ++)
		Icons[i] = LoadIcon(plugin.hDllInstance,MAKEINTRESOURCE(IDI_ICON0+i));
	
	do_icons();


	// /////////////////////////////
	// Hook procedure SysTrayWndProc 
	// /////////////////////////////
#ifdef STRICT 
	lpWndProcOld = (WNDPROC)GetWindowLong(plugin.hwndParent,GWL_WNDPROC);
#else 
	lpWndProcOld = (FARPROC)GetWindowLong(plugin.hwndParent,GWL_WNDPROC);
#endif 
	SetWindowLong(plugin.hwndParent, GWL_WNDPROC, (LONG)SysTrayWndProc);


	return 0;
}

/* ************************************************************************ */
void quit()
{
	config_write();
	config_enabled=0;
	do_icons();

	// clean up Dialogbox EZ-playlist
	DestroyWindow(ghWndMain); 
	UnregisterClass(szAppName, plugin.hDllInstance);
	
	if (dlg != NULL) 
		delete dlg;

}


/* ************************************************************************ */
void config_read()
{
	TCHAR ini_file[MAX_PATH];

	config_getinifn(ini_file);
	
	config_enabled = GetPrivateProfileInt(szAppName, _T("ButtonsEnabled") ,config_enabled,ini_file);
}

/* ************************************************************************ */
void config_write()
{
	TCHAR ini_file[MAX_PATH];
	TCHAR string[32];

	config_getinifn(ini_file);
	
	_stprintf(string,_T("%d"),config_enabled);
	WritePrivateProfileString(szAppName, _T("ButtonsEnabled"),string,ini_file);
}

/* ************************************************************************ */
void config_getinifn(TCHAR *ini_file)
{	
	char *p;

	// makes a .ini file in the winamp directory named "plugin.ini"
	GetModuleFileName(plugin.hDllInstance, ini_file, MAX_PATH);
	p = ini_file + _tcslen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	_tcscat(ini_file, _T("plugin.ini"));
}

/* ************************************************************************ */
void do_icons()
{
	static int l=0;
	int i;
	if (l == config_enabled) return;
	for (i = NUM_ICONSYSTRAY-1; i >= 0; i --)
	{
		if (l & (1<<i))
			systray_del(plugin.hwndParent,i);
	}
	
	l = config_enabled;	
	for (i = 0; i < NUM_ICONSYSTRAY; i ++)
	{
		if (config_enabled & (1<<i))
			systray_add(plugin.hwndParent,i,Icons[i],tips[i]);
	}
}

/* ************************************************************************ */
void config()
{
	DialogBox(plugin.hDllInstance,
		MAKEINTRESOURCE(IDD_DLGCONFIGURE),
		plugin.hwndParent,
		(DLGPROC)ConfigProc);
}



/* ************************************************************************ */
LRESULT CALLBACK SysTrayWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{

#define N_BORDER_SIZE	2
#define N_BITMAP_SIZE	16

	
	const int WMA_SYSTRAY = WM_USER+27;
	static int m_nFontHeight = 0;
	static HFONT m_menuFont = NULL;

	switch(message)
	{
		// WM_MEASUREITEM (OwnerDraw Menu with icon)		
		case WM_MEASUREITEM: 
		{
			 LPMEASUREITEMSTRUCT lpMeasureItemStruct = (LPMEASUREITEMSTRUCT) lParam;

			 if (lpMeasureItemStruct->CtlType != ODT_MENU)
				break;

			TCHAR  str[] = _T("Very Long String");
			HDC hDC = GetWindowDC(NULL);

			if (!(HFONT)m_menuFont) 
			{
				NONCLIENTMETRICS info;
				info.cbSize = sizeof(info);
				SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(info), &info, 0);
				m_menuFont = CreateFontIndirect(&info.lfMenuFont);
			}
			SelectObject (hDC, &m_menuFont);

			SIZE size;
			::GetTextExtentPoint32(hDC, str, _tcslen(str), &size);
			ReleaseDC(NULL, hDC);

			lpMeasureItemStruct->itemHeight = N_BORDER_SIZE + ((size.cy > N_BITMAP_SIZE) ? size.cy : N_BITMAP_SIZE) + N_BORDER_SIZE;    
			lpMeasureItemStruct->itemWidth  = 2*N_BORDER_SIZE + N_BITMAP_SIZE + 2*N_BORDER_SIZE + size.cx + N_BORDER_SIZE;    
			m_nFontHeight = size.cy;
			break;
		}

		// WM_DRAWITEM (OwnerDraw Menu with icon)
		case WM_DRAWITEM:
		{
			LPDRAWITEMSTRUCT lpDrawItemStruct = (LPDRAWITEMSTRUCT) lParam; 
			
			if (lpDrawItemStruct->CtlType != ODT_MENU)
				break;

			HDC hDC = lpDrawItemStruct->hDC;
			LPCTSTR text   = (LPCTSTR) lpDrawItemStruct->itemData;
			BOOL bSelected = (lpDrawItemStruct->itemState) & ODS_SELECTED;
			BOOL bGrayed   = (lpDrawItemStruct->itemState) & ODS_GRAYED;
			BOOL bChecked  = (lpDrawItemStruct->itemState) & ODS_CHECKED;

			COLORREF crBack;
			crBack = bSelected ? ::GetSysColor (COLOR_HIGHLIGHT) : ::GetSysColor (COLOR_MENU);
			SetBkColor (hDC,crBack);
			RECT rect;
			rect = lpDrawItemStruct->rcItem;
			rect.left += N_BORDER_SIZE + 2*N_BORDER_SIZE + N_BITMAP_SIZE;
			rect.right += N_BORDER_SIZE + 2*N_BORDER_SIZE + N_BITMAP_SIZE;
			::SetBkColor(hDC, crBack);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
			POINT ptText;
			ptText.x = lpDrawItemStruct->rcItem.left + 2*N_BORDER_SIZE + N_BITMAP_SIZE + 2*N_BORDER_SIZE; 
			ptText.y = lpDrawItemStruct->rcItem.top + (lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top - m_nFontHeight)/2 - 1;

			SetBkMode(hDC, TRANSPARENT);
			int nLen = _tcslen(text);

			if (!bGrayed)
			{	if (bSelected)
				{	SetTextColor (hDC, ::GetSysColor (COLOR_HIGHLIGHTTEXT));
					TextOut (hDC, ptText.x, ptText.y, text, nLen);
				}
				else
				{	SetTextColor (hDC, ::GetSysColor (COLOR_MENUTEXT));
					TextOut (hDC, ptText.x, ptText.y, text, nLen);
				}
			}
			else
			{	if (bSelected)
				{	SetTextColor (hDC, GetSysColor(COLOR_MENU));
					TextOut (hDC, ptText.x, ptText.y, text, nLen);
				}
				else
				{	SetTextColor(hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
					TextOut (hDC, ptText.x + 1, ptText.y + 1, text, nLen);

					SetTextColor (hDC, GetSysColor(COLOR_GRAYTEXT));
					TextOut (hDC, ptText.x, ptText.y, text, nLen);
				}
			}

			POINT pt;
			pt.x = lpDrawItemStruct->rcItem.left + 2*N_BORDER_SIZE;
			pt.y = lpDrawItemStruct->rcItem.top + (lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top - N_BITMAP_SIZE)/2;

			UINT uIcon = IDI_EMPTY;

			switch(lpDrawItemStruct->itemID)
			{
				case IDM_SYSTRAY_EXTPLAYLIST:   uIcon = IDI_ICON0 ; break;
				case IDM_SYSTRAY_PREVIOUSTRACK: uIcon = IDI_ICON1 ; break;
				case IDM_SYSTRAY_PLAYPAUSE:     uIcon = IDI_ICON2 ; break;
				case IDM_SYSTRAY_STOP:          uIcon = IDI_ICON3 ; break;
				case IDM_SYSTRAY_NEXTTRACK:     uIcon = IDI_ICON4 ; break;
				case IDM_SYSTRAY_LOADFILE:      uIcon = IDI_ICON5 ; break;
			}
			HICON hIcon = NULL;
			
			hIcon = LoadIcon(plugin.hDllInstance, MAKEINTRESOURCE(uIcon));
			if(hIcon)	
			{	
				DrawIconEx (hDC, pt.x, pt.y, hIcon, N_BITMAP_SIZE, N_BITMAP_SIZE, NULL,	NULL, DI_NORMAL);
				DeleteObject (hIcon);
			}
			

			break;
		}

		case WM_DESTROY:
			if ((HFONT)m_menuFont) 
				DeleteObject(m_menuFont);		
			break;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			// Via Popup Menu
			case IDM_SYSTRAY_EXTPLAYLIST:	ezplaylist_init(plugin.hDllInstance);	return 0;
			case IDM_SYSTRAY_PREVIOUSTRACK:	winamp_previous_track(hwnd);			return 0;
			case IDM_SYSTRAY_PLAYPAUSE:		winamp_play_pause(hwnd);				return 0;
			case IDM_SYSTRAY_STOP:			winamp_stop(hwnd);						return 0;
			case IDM_SYSTRAY_NEXTTRACK:		winamp_next_track(hwnd);				return 0;
			case IDM_SYSTRAY_LOADFILE:		winamp_openfile(hwnd);					return 0;
			}
			break;

		case WMA_SYSTRAY:
		{
			// Via Taskbar
			int which = LOWORD(wParam) - 1024;

			switch (LOWORD(lParam))
			{
				case WM_LBUTTONDOWN:
					switch (which)
					{
						case 0:	ezplaylist_init(plugin.hDllInstance);	return 0;
						case 1: winamp_previous_track(hwnd);			return 0;
						case 2: winamp_play_pause(hwnd);				return 0;
						case 3: winamp_stop(hwnd);						return 0;
						case 4: winamp_next_track(hwnd);				return 0;
						case 5: winamp_openfile(hwnd);					return 0;
						case 6:	winamp_volume_up(hwnd);					return 0;
						case 7: 
							{
							HMENU hMenu = CreatePopupMenu();
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_EXTPLAYLIST,   TEXT_TIPS_EXTPLAYLIST); 
							AppendMenu(hMenu, MF_SEPARATOR, NULL, "");
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_PREVIOUSTRACK, TEXT_TIPS_PREVIOUSTRACK);
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_PLAYPAUSE,     TEXT_TIPS_PLAYPAUSE);
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_STOP,          TEXT_TIPS_STOP);
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_NEXTTRACK,     TEXT_TIPS_NEXTTRACK);
							AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_LOADFILE,      TEXT_TIPS_LOADFILE);
						
							POINT pos;
							GetCursorPos(&pos);
							SetForegroundWindow(hwnd);  

							TrackPopupMenu (hMenu, 0, pos.x, pos.y,0,hwnd, NULL);
					
							// BUGFIX: See "PRB: Menus for Notification Icons Don't Work Correctly"
							PostMessage(hwnd, WM_NULL, 0, 0);
							DestroyMenu(hMenu);
							
							return 0;
							}

					}
					return 0;
			
				case WM_RBUTTONDOWN:
					switch (which)
					{
					case 0:	
					case 1:	
					case 2:	
					case 3:	
					case 4:	
					case 5:	
					case 7:	
						SetForegroundWindow(hwnd);
						DialogBox(plugin.hDllInstance,MAKEINTRESOURCE(IDD_DLGCONFIGURE),hwnd,(DLGPROC)::ConfigProc);
						return 0;	
					case 6:	
						winamp_volume_down(hwnd);				
						return 0;
					}
					return 0;
			}
		}
	}

	return CallWindowProc(lpWndProcOld,hwnd,message,wParam,lParam);
	
}


/* ************************************************************************ */

// -----------------------------------------------
void winamp_previous_track(HWND hwnd)
{
	int a;

	// not playing, let's hit prev
	if ((a=SendMessage(hwnd,WM_USER,0,104)) == 0) 
		// Previous
		SendMessage(hwnd,WM_COMMAND,40044,0);
	else if (a != 3 && SendMessage(hwnd,WM_USER,0,105) > 2000) // restart
		//Play
		SendMessage(hwnd,WM_COMMAND,40045,0);
	else 
		// previous
		SendMessage(hwnd,WM_COMMAND,40044,0);
}

// -----------------------------------------------
void winamp_play_pause(HWND hwnd)
{
	int a;

	// not playing, let's hit play
	if ((a=SendMessage(hwnd,WM_USER,0,104)) != 1) 
		SendMessage(hwnd,WM_COMMAND,40045,0);
	else  // prev
		SendMessage(hwnd,WM_COMMAND,40046,0);
}	

// -----------------------------------------------
void winamp_stop(HWND hwnd)
{
	if (GetKeyState(VK_SHIFT) & (1<<15))
		SendMessage(hwnd,WM_COMMAND,40147,0);
	else
		SendMessage(hwnd,WM_COMMAND,40047,0);
}

// -----------------------------------------------
void winamp_next_track(HWND hwnd)
{
	SendMessage(hwnd,WM_COMMAND,40048,0);
}

// -----------------------------------------------
void winamp_openfile(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	if (GetKeyState(VK_CONTROL) & (1<<15))
		SendMessage(hwnd,WM_COMMAND,40185,0);	// internet location
	else if (GetKeyState(VK_SHIFT) & (1<<15))
		SendMessage(hwnd,WM_COMMAND,40187,0);	// open directory
	else
		SendMessage(hwnd,WM_COMMAND,40029,0);	// open file
}


// -----------------------------------------------
void winamp_volume_up(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	int k, nMax;
	if (GetKeyState(VK_CONTROL) & (1<<15))
		nMax = 20;
	else
		nMax = 2;

	for (k=0; k<nMax; k++)
		SendMessage(hwnd,WM_COMMAND,40058,0);
}

// -----------------------------------------------
void winamp_volume_down(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	int k, nMax;
	if (GetKeyState(VK_CONTROL) & (1<<15))
		nMax = 20;
	else
		nMax = 2;

	for (k=0; k<nMax; k++)
		SendMessage(hwnd,WM_COMMAND,40059,0);
}




/* ************************************************************************ */
BOOL CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	
	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			for (int i = 0; i < NUM_ICONSYSTRAY; i++)
				CheckDlgButton(hwndDlg,IDC_PREV1+i,(config_enabled & (1<<i))?BST_CHECKED:BST_UNCHECKED);

			if (IsDlgButtonChecked(hwndDlg,IDC_PREV8) == BST_CHECKED)
			{	for (i = 0; i < NUM_ICONSYSTRAY-1; i++)
				{	CheckDlgButton(hwndDlg,IDC_PREV1+i, BST_UNCHECKED);
					EnableWindow(GetDlgItem(hwndDlg,IDC_PREV1+i), FALSE);
				}
			}
		}
		return FALSE;

	case WM_PAINT:
		{
			HDC hDC;
			PAINTSTRUCT ps;
			hDC = BeginPaint(hwndDlg, &ps);
			
			for (int i = 0; i < NUM_ICONSYSTRAY-1; i++)
			{
				RECT rc;
				POINT pt;
				HWND hwndChk = GetDlgItem(hwndDlg,IDC_PREV1+i);
				
				GetWindowRect(hwndChk, &rc);
				HICON hIcon = (HICON)LoadImage(plugin.hDllInstance, MAKEINTRESOURCE(IDI_ICONSMALL1+i),
					   IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), 0);

				pt.x = rc.left - 25;
				pt.y = rc.top + 2;
				ScreenToClient(hwndDlg, &pt);

				if(hIcon)	
					DrawIconEx(hDC, pt.x, pt.y, hIcon, 0, 0, 0, NULL, DI_NORMAL );
			}
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
			case IDC_PREV8:		// checkbox Group in systray menu
				if (HIWORD(wParam) == BN_CLICKED)	
				{
					BOOL bEnable = (IsDlgButtonChecked(hwndDlg,IDC_PREV8) == BST_UNCHECKED);
					for (int i = 0; i < NUM_ICONSYSTRAY-1; i++)
					{
						CheckDlgButton(hwndDlg,IDC_PREV1+i, (bEnable ? BST_CHECKED:BST_UNCHECKED));
						EnableWindow(GetDlgItem(hwndDlg,IDC_PREV1+i), bEnable);
					}
				}
				return TRUE;
		
			case IDC_APPLY:
			case IDOK:
				config_enabled=0;
				{
					for (int i = 0; i < NUM_ICONSYSTRAY; i++)
						if (IsDlgButtonChecked(hwndDlg,IDC_PREV1+i))
							config_enabled |= 1<<i;
				}
				do_icons();
			case IDCANCEL:
				if (LOWORD(wParam) != IDC_APPLY) 
					EndDialog(hwndDlg,0);
				return FALSE;
		}
	}
	return FALSE;
}




/* ************************************************************************ */
BOOL ezplaylist_init(HANDLE hInstance)
{
/*	
// ## MODAL MODE
	if (ghWndMain  == NULL)
	{
		CDlgPlaylist dlg;
	    dlg.DoModal(IDD_DLGEZPLAYLIST, plugin.hwndParent);
	}
	else
	{   ShowWindow (ghWndMain, SW_SHOWNORMAL);
		UpdateWindow (ghWndMain);
		SetForegroundWindow(ghWndMain);
		return (TRUE);
	}

	return TRUE;
}
*/

// ## MODELESS MODE

	static bInit = FALSE;

	if (bInit == FALSE)
	{
		WNDCLASS  wc;

		// Fill in window class structure with parameters 
		// that describe the main window.                     
		memset(&wc,0,sizeof(wc));
		wc.lpfnWndProc = DefDlgProc;			// our window procedure
		wc.hInstance = (HINSTANCE)hInstance;	// hInstance of DLL
		wc.lpszClassName = szAppName;			// our window class name

		if (!RegisterClass(&wc)) 
			return (FALSE); 
		
		bInit= TRUE;
	}


	if (ghWndMain != NULL)
	{
	    ShowWindow (ghWndMain, SW_SHOWNORMAL);
		UpdateWindow (ghWndMain);
		SetForegroundWindow(ghWndMain);
		return (TRUE);
	}
	
	// Perform initializations that apply to a specific instance 
	// Make the window visible; update its client area; and return "success" 

	dlg = new CDlgPlaylist;		

	if (dlg == NULL)
		return FALSE;

	ghWndMain = dlg->DoModeless(IDD_DLGEZPLAYLIST, plugin.hwndParent);

	// If window could not be created, return "failure"
	if (!ghWndMain)
	{
		delete dlg;
		return (FALSE);
	}
	
	ShowWindow (ghWndMain, SW_SHOW);
	UpdateWindow (ghWndMain);
	SetForegroundWindow(ghWndMain);

	return TRUE;
}



