/*****************************************************************************
    This file is part of MP3ext.
    Copyright (C) 1999  Michael Mutschler

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    email: michael@mutschler.de
******************************************************************************/
/* here is a function to read the MP3-info from a file */


#include <stdio.h>
#include <string.h>
#include <sys/stat.h>		// use stat for file-info
#include <TCHAR.H>

#include "mpg123.h"
#include "DXhead.h"

/*
#ifdef WIN32
#ifdef _DEBUG
#define _CRTDBG_MAP_ALLOC
#endif
#include <crtdbg.h>
#endif
*/
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif


#define MAXFRAMESIZE 1792	/* max = 1728 */

#define         MPG_MD_STEREO           0
#define         MPG_MD_JOINT_STEREO     1
#define         MPG_MD_DUAL_CHANNEL     2
#define         MPG_MD_MONO             3


#define guint32 unsigned long


/***********************************
 * some arrays
 ********************************* */
int tabsel_123[2][3][16] =
{
	{
    {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448,},
       {0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384,},
       {0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320,}},

	{
       {0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256,},
	    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160,},
	    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160,}}
};

long mpg123_freqs[9] = {44100, 48000, 32000, 22050, 24000, 16000, 11025, 12000, 8000};


char    *ModeNameA[4] = {  "Stereo",  "Joint-Stereo",  "Dual-Channel",  "Single-Channel" };
wchar_t *ModeNameW[4] = { L"Stereo", L"Joint-Stereo", L"Dual-Channel", L"Single-Channel" };
char    *BoolNameA[2] = { "No",  "Yes"};
wchar_t *BoolNameW[2] = {L"No", L"Yes"};
char    *EmphasisNameA[4] = { "None",  "50/15 ms",  ".",  "CCIT J.17"};
wchar_t *EmphasisNameW[4] = {L"None", L"50/15 ms", L".", L"CCIT J.17"};
char    *ID3TagNameA[4] = { "None",  "ID3v1", "ID3v2", "ID3v1+v2"};
wchar_t *ID3TagNameW[4] = { L"None",  L"ID3v1", L"ID3v2", L"ID3v1+v2"};


int mpg123_head_check(unsigned long head);
int mpg123_decode_header(struct mpg123info *fr, unsigned long newhead);
double mpg123_compute_tpf(struct mpg123info *fr);
double mpg123_compute_bpf(struct mpg123info *fr);


/* *******************************************************************
 * definition of the first 4 bytes in an MPEG 1,2,3 file
 * 33222222222211111111110000000000
 * 10987654321098765432109876543210
 * |  b0  ||  b1  ||  b2  ||  b3  |
 *
 * 31-21 = sync (0xfff) (or search the stream until sync is found)
 * 20-19 = version	( 2=MPEG2, 3=MPEG1 0=MPEG2.5)
 * 18-17 = layer   (Layernumber = 4 - layer)
 * 16    = error protect
 * 15-12 = bitrate index
 * 11-10 = sample rate
 * 9     = pad
 * 8     = extension
 * 7-6   = mode
 * 5-4   = mode extension
 * 3     = Copyright
 * 2     = original
 * 1-0   = emphasis
 * ******************************************************************* */

void mpg123_readinfo(const char *filename, struct mpg123info *fr, BOOL id3_found)
{
	FILE *fh;
	
	if ((fh = fopen(filename, "rb")) != NULL)
	{
		guint32 head;
		unsigned char tmp[4];
		
		rewind(fh);
		if (fread(tmp, 1, 4, fh) != 4)
		{
			fclose(fh);
			return;
		}
		head = ((guint32) tmp[0] << 24) | ((guint32) tmp[1] << 16) | ((guint32) tmp[2] << 8) | (guint32) tmp[3];
		
		while (!mpg123_head_check(head))
		{
			head <<= 8;
			if (fread(tmp, 1, 1, fh) != 1)
			{
				fclose(fh);
				return;
			}
			head |= tmp[0];
		}
		
		if (mpg123_decode_header(fr, head))
		{
			double tpf, bpf;
			unsigned char *buf;
			unsigned long len;
			XHEADDATA xing_header;

			buf = malloc(fr->framesize + 4);
			fseek(fh, -4, SEEK_CUR);
			fread(buf, 1, fr->framesize + 4, fh);
			xing_header.toc = NULL;
			
			tpf = mpg123_compute_tpf(fr);
			bpf = mpg123_compute_bpf(fr);
			
			fseek(fh, 0, SEEK_END);
			len = ftell(fh);
	
			
			if (mpg123_get_xing_header(&xing_header, buf))
			{
				fr->varbitrate = TRUE;
				fr->num_frames = xing_header.frames;
				fr->bitrate_index = (int) ((xing_header.bytes * 8) / (tpf * xing_header.frames * 1000));
			}
			else
			{
				if (id3_found)
					len -= 128;
				
				fr->varbitrate = FALSE;
				fr->num_frames = (int) ((len) / bpf + 1);
				fr->bitrate_index = tabsel_123[fr->lsf][fr->lay - 1][fr->bitrate_index];
			}

			fr->sampling_frequency  = mpg123_freqs[fr->sampling_frequency];
			fr->length = (int)(fr->num_frames * tpf);


			free(buf);
		}
		fclose(fh);
	}

}

/* ************************************************************************ */
int mpg123_head_check(unsigned long head)
{
	if ((head & 0xffe00000) != 0xffe00000)
		return FALSE;
	if (!((head >> 17) & 3))
		return FALSE;
	if (((head >> 12) & 0xf) == 0xf)
		return FALSE;
	if (!((head >> 12) & 0xf))
		return FALSE;
	if (((head >> 10) & 0x3) == 0x3)
		return FALSE;
	if (((head >> 19) & 1) == 1 && ((head >> 17) & 3) == 3 && ((head >> 16) & 1) == 1)
		return FALSE;
	if ((head & 0xffff0000) == 0xfffe0000)
		return FALSE;
	
	return TRUE;
}


/* ************************************************************************ */
/* the code a header and write the information into the frame structure     */
int mpg123_decode_header(struct mpg123info *fr, unsigned long newhead)
{
	if (newhead & (1 << 20))
	{
		fr->lsf = (newhead & (1 << 19)) ? 0x0 : 0x1;
		fr->mpeg25 = 0;
	}
	else
	{
		fr->lsf = 1;
		fr->mpeg25 = 1;
	}

	fr->lay = 4 - ((newhead >> 17) & 3);

	if (fr->mpeg25)
	{
		fr->sampling_frequency = 6 + ((newhead >> 10) & 0x3);
	}
	else
		fr->sampling_frequency = ((newhead >> 10) & 0x3) + (fr->lsf * 3);

	fr->error_protection = ((newhead >> 16) & 0x1) ^ 0x1;

	if (fr->mpeg25)		/* allow Bitrate change for 2.5 ... */
		fr->bitrate_index = ((newhead >> 12) & 0xf);

	fr->bitrate_index = ((newhead >> 12) & 0xf);
	fr->padding = ((newhead >> 9) & 0x1);
	fr->extension = ((newhead >> 8) & 0x1);
	fr->mode = ((newhead >> 6) & 0x3);
	fr->mode_ext = ((newhead >> 4) & 0x3);
	fr->copyright = ((newhead >> 3) & 0x1);
	fr->original = ((newhead >> 2) & 0x1);
	fr->emphasis = newhead & 0x3;

	fr->stereo = (fr->mode == MPG_MD_MONO) ? 1 : 2;

	if (!fr->bitrate_index)
		return (0);

	switch (fr->lay)
	{
		case 1:
			fr->framesize = (long) tabsel_123[fr->lsf][0][fr->bitrate_index] * 12000;
			fr->framesize /= mpg123_freqs[fr->sampling_frequency];
			fr->framesize = ((fr->framesize + fr->padding) << 2) - 4;
			break;
		case 2:
			fr->framesize = (long) tabsel_123[fr->lsf][1][fr->bitrate_index] * 144000;
			fr->framesize /= mpg123_freqs[fr->sampling_frequency];
			fr->framesize += fr->padding - 4;
			break;
		case 3:
			fr->framesize = (long) tabsel_123[fr->lsf][2][fr->bitrate_index] * 144000;
			fr->framesize /= mpg123_freqs[fr->sampling_frequency] << (fr->lsf);
			fr->framesize = fr->framesize + fr->padding - 4;
			break;
		default:
			return (0);
	}
	if(fr->framesize > MAXFRAMESIZE)
		return 0;
		
	return 1;
}

/* ************************************************************************ */
double mpg123_compute_tpf(struct mpg123info *fr)
{
	static int bs[4] =
	{0, 384, 1152, 1152};
	double tpf;

	tpf = (double) bs[fr->lay];
	tpf /= mpg123_freqs[fr->sampling_frequency] << (fr->lsf);
	return tpf;
}

/* ************************************************************************ */
double mpg123_compute_bpf(struct mpg123info *fr)
{
	double bpf;

	switch (fr->lay)
	{
		case 1:
			bpf = tabsel_123[fr->lsf][0][fr->bitrate_index];
			bpf *= 12000.0 * 4.0;
			bpf /= mpg123_freqs[fr->sampling_frequency] << (fr->lsf);
			break;
		case 2:
		case 3:
			bpf = tabsel_123[fr->lsf][fr->lay - 1][fr->bitrate_index];
			bpf *= 144000;
			bpf /= mpg123_freqs[fr->sampling_frequency] << (fr->lsf);
			break;
		default:
			bpf = 1.0;
	}

	return bpf;
}

